/*              Name:           callbacks.c
                Version:        1.2.0
                Date:           11/7/2002
				Functions called on actions from the GTK interface
*/
#include <glib.h>
#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <bfd.h>                      /* Callbacks from GTK are caught here*/
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "callbacks.h"
#include "breakcalls.h"
#include "interface.h"
#include "serial.h"
#include "misc.h"
#include "view.h"
#include "chump.h"

/* the following is a lock asscociated with the communication across the line */




typedef enum {
  FREE,
  CONSOLE_UPDATE,
  MEM_AND_REG_UPDATE
} lock_state;		  		  		  

lock_state serial_lock=FREE; /* this is a variable to hold the lock state */


uchar board_startcommand = BR_START|0x30;
int board_running=FALSE;

void callback_button_load (GtkButton* button, gpointer entry)  /* Load button pressed*/
{ /*                Load button       Text entry*/
 char* filename = g_strdup(gtk_entry_get_text (GTK_ENTRY (entry))); /* Get filename*/
 bfd *abfd;
 asection *section;
 int tosend;
 uchar* address;
 GList* list;
 GList* element; 
 long storage_needed;
 long i;
 asymbol **bfd_symbol_table;


 bfd_set_error (bfd_error_no_error);
 abfd = bfd_openr (filename, NULL); 
 if (VERBOSE)
     g_print("Loading: %s\n",filename);
 if (bfd_get_error ()) 
   {
   if (VERBOSE)
       g_print("ELF load error:%s\n",bfd_errmsg (bfd_get_error()));
       return;
   }
 if (!bfd_check_format(abfd, bfd_object)) 
  {
   if (VERBOSE) 
       g_print("File is not BFD format");
   bfd_close(abfd); gtk_widget_show (view_fileerror);
   return;
  } /* on any error just return */
// if(bfd_get_error()){g_print("%s\n",bfd_errmsg (bfd_get_error()));}
 bfd_set_error (bfd_error_no_error);
 
 storage_needed = bfd_get_symtab_upper_bound (abfd);
 if (storage_needed < 0) 
   {
   if (VERBOSE)
         g_print("ELF load error in symbol table load\n"); 
         return;
   }
 if (storage_needed != 0) 
   {
    if (symbol_table)
        {
          while(symbol_count--) g_free(symbol_table[symbol_count].name);
          g_free(symbol_table);
        }
    bfd_symbol_table = (asymbol **) g_malloc (storage_needed);
    symbol_count = bfd_canonicalize_symtab (abfd, bfd_symbol_table);
    symbol_table = g_new(symbol, symbol_count);
    if (symbol_count < 0)
       {
         if (VERBOSE)g_print("ELF load error in symbol table load\n"); 
	 return;
       }
    
    for (i = 0; i < symbol_count; i++) 
      {
        symbol_table[i].flags=bfd_symbol_table[i]->flags;
        symbol_table[i].name=g_strdup(bfd_symbol_table[i]->name);
        symbol_table[i].value=bfd_symbol_table[i]->value + bfd_symbol_table[i]->section->vma; //add the value to the offset of the section
      //  g_print("symbol:%8x %8x %8x %s\n",symbol_table[i].value, symbol_table[i].flags, bfd_symbol_table[i]->section->vma + bfd_symbol_table[i]->value, symbol_table[i].name);
      }
    g_free(bfd_symbol_table);
   } // end if
 
 
 section=abfd->sections;
 while(section)
   {
    if (section->flags & 1)
      {
        uchar *sectionContents;
        size_t sectionSize = section->_cooked_size; /* get file */
        size_t count=0;
        uchar* start = view_int2chararr(board_memory_ptr_width, section->vma);
        sectionContents = g_new (uchar, sectionSize);
        bfd_get_section_contents (abfd, section, (PTR) sectionContents, 0, sectionSize);
        if(bfd_get_error ())
	    {
	      if (VERBOSE)
		g_print("ELF loads error5\n");
	        bfd_close(abfd);
	        return;
	    }     
        while (count<sectionSize)
           {
                tosend=MIN(sectionSize-count,80);
                address = view_chararrAddint(board_memory_ptr_width, start, count); /* send data 80 at a time*/
                board_set_memory (tosend, address, sectionContents+count, 1);
                g_free(address);
	        if (board_micro_ping()) count +=tosend;                 /* if errored with microping then do again*/
	        gtk_progress_bar_update (GTK_PROGRESS_BAR(view_progressbar),(float)count/sectionSize);/* update progress bar*/
                while (gtk_events_pending())  gtk_main_iteration();          /*update display and do user stuff*/
	   }
       g_free(sectionContents);
       g_free(start);
     }
    section=section->next;
  } // end while
  
   bfd_close(abfd);
      
   list = gtk_object_get_data (GTK_OBJECT(entry),"list");   /* append filename to list of past filenames*/
   if (!(element = g_list_find_custom(list, filename, (GCompareFunc) g_strcasecmp)))
                list = g_list_prepend (list, filename);
   else         list = g_list_concat (element, g_list_remove_link(list,element));
   gtk_object_set_data (GTK_OBJECT (entry), "list", list);
   gtk_combo_set_popdown_strings (GTK_COMBO (GTK_WIDGET(entry)->parent), list);
 gtk_entry_set_text (GTK_ENTRY(entry), g_strdup(filename));
 if (element) g_free(filename);
 callback_memory_refresh(); /* refresh all */
}

void callback_load_binary_address(GtkEntry* entry, gpointer label)
{
 uchar* address = g_new0(uchar,board_memory_ptr_width);
 char* text = gtk_editable_get_chars(GTK_EDITABLE(view_binary_load_address),0,-1);
 if (!view_hexstr2chararr(board_memory_ptr_width, text, address))
   		{        /* get address from entry*/
                g_free(text);
                g_free(address);
                gtk_label_set_text(label, "Not Valid!");
                return;
		}
 g_free(text);
 text = view_chararr2hexstrbe (board_memory_ptr_width, address);
 gtk_label_set_text(label, text);
 g_free(text);
 g_free(address);
 
}

void callback_load_binary(GtkButton* button, gpointer entry)
{
 char* filename = g_strdup(gtk_entry_get_text (GTK_ENTRY (entry))); /* Get filename*/
 GList* list;
 GList* element; 
 int file;
 int rep;
 uchar buf[80];
 uchar* address = g_new0(uchar,board_memory_ptr_width);
 struct stat status;
 int sofar = 0;
 char* text = gtk_editable_get_chars(GTK_EDITABLE(view_binary_load_address),0,-1);
 if (!view_hexstr2chararr(board_memory_ptr_width, text, address))        /* get address from entry*/
                {
  		 g_free(text); 
  		 g_free(address);
  		 return;
		}
 g_free(text);
 file = open(filename, O_RDONLY);
 if (!*filename) return;
 fstat(file, &status);
 while ((rep = read(file, buf,80)))   
   {
    sofar+=rep;
    board_set_memory (rep, address, buf, 1);
    if (!board_micro_ping()) 
            {
            board_set_memory (rep, address, buf, 1);
            if (!board_micro_ping()) return;
            }                    /* if errored with microping then do again*/
    view_chararrAddintsame(board_memory_ptr_width, address, rep); /* send data 80 at a time*/
    gtk_progress_bar_update (GTK_PROGRESS_BAR(view_progressbar),(float)sofar/status.st_size);/* update progress bar*/
    while (gtk_events_pending())  gtk_main_iteration();          /*update display and do user stuff*/
   }
 gtk_widget_hide(view_fileerror);
 gtk_progress_bar_update (GTK_PROGRESS_BAR(view_progressbar),1);/* update progress bar*/
 g_free(address);
 close (file);
 list = gtk_object_get_data (GTK_OBJECT(entry),"list");   /* append filename to list of past filenames*/
 if (!(element = g_list_find_custom(list, filename, (GCompareFunc) g_strcasecmp)))
                list = g_list_prepend (list, g_strdup(filename));
   else         list = g_list_concat (element, g_list_remove_link(list,element));
 gtk_object_set_data (GTK_OBJECT (entry), "list", list);
 gtk_combo_set_popdown_strings (GTK_COMBO (GTK_WIDGET(entry)->parent), list);
 gtk_entry_set_text (GTK_ENTRY(entry), g_strdup(filename));
 if (element) g_free(filename);
 callback_memory_refresh(); /* refresh all */
}





void callback_button_compile (GtkButton* button, gpointer entry)  /* Compile button pressed*/
{
 GList* list;
 char* filename = gtk_entry_get_text (GTK_ENTRY (entry)); /* Get filename*/
 if (!*filename) return ;
 if (!fork()) {
                  close(1);
                  dup2(compile_communication[1],1);
                  close(2);
                  dup2(compile_communication[1],2);
                  close(0);
                  if (VERBOSE) g_print("Compiling: %s\n",filename);
                  setpgid(0,0);
                  execlp("kmd_compile", "kmd_compile", filename, board_cpu_name, "0", NULL);
                  _exit(0);
              }
   list = gtk_object_get_data (GTK_OBJECT(entry),"list");   /* append filename to list of past filenames*/
   if (!g_list_find_custom(list, filename, (GCompareFunc) g_strcasecmp))
             list = g_list_prepend (list, g_strdup(filename));
   gtk_object_set_data (GTK_OBJECT (entry), "list", list);
   gtk_combo_set_popdown_strings (GTK_COMBO (GTK_WIDGET(entry)->parent), list);

}



void callback_button_ok_file(GtkButton* button, gpointer entry)  /* on fileentry ok button send to text entry*/
{
  char* text= gtk_file_selection_get_filename (GTK_FILE_SELECTION
                                              (GTK_WIDGET(button)->parent->parent->parent));
  gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1); /* clear entry*/
  gtk_entry_append_text (GTK_ENTRY(entry), text);/* insert new text*/
}




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Check output Xilinx bit file header                                        */
/* Returns TRUE if no errors found                                            */

int callback_FPGA_header(uchar* buffer, gpointer container, unsigned int* size,
                         int load)
{
int position;
char* name;

if (buffer[0] != 0) return (0==1);          /* WHAT? WHY? */

position = 1;                                 /* Pointer to length of ???     */
position = position + buffer[position] + 5;                 /* Skip ??? field */
name     = &buffer[position + 1];                   /* Pointer to design name */
position = position + buffer[position] + 3;  /* Skip design name field et al. */

/* position now points to the length of the device identifier string (I think)*/
/* The identifier string starts at the next address                           */
if (g_strcasecmp(board_features[GPOINTER_TO_INT(gtk_object_get_data(
               GTK_OBJECT(container),"feature"))].data.xilinx_fpga.filestring,
               &buffer[position+1]) != 0)       /* Compare device identifiers */
  return (0==1);                                  /* Zero means strings match */

/* if (VERBOSE) g_print("%d\n", rep); rep was No. bytes read from file */

if (load)                                                      /* Design name */
  gtk_label_set_text((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
                                                 "labelfpganame"))), name);
else
  gtk_label_set_text((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
                                                 "labelfilename"))), name);

position = position + buffer[position] + 3;
                                /* Skip device identifier string field et al. */

if (load)
  gtk_label_set_text((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
                 "labelfpgadate"))), &buffer[position+1]); // date  = position+1
else
  gtk_label_set_text((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
                 "labelfiledate"))), &buffer[position+1]); // date  = position+1

position = position + buffer[position] + 3;    /* Skip date string field etc. */

if (load)
  gtk_label_set_text ((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
              "labelfpgatime"))), &buffer[position+1]); //  time  = position+1
else
  gtk_label_set_text ((GTK_LABEL(gtk_object_get_data(GTK_OBJECT(container),
              "labelfiletime"))), &buffer[position+1]); //  time  = position+1

position = position + buffer[position] + 2;    /* Skip time string field etc. */

*size = (buffer[position]<<24) + (buffer[position+1]<<16)
      + (buffer[position+2]<<8) + buffer[position+3]  + position + 4;
                                                      /* Total length of file */
return (0==0);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/








/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Complete download sequence to selected FPGA                                */

void callback_fpgaload(GtkButton* button, gpointer container)
{
char* filename = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(
                                    GTK_OBJECT(container), "fileentry")));
                                                              /* Get filename */
GList* list;
FILE* file;
uchar data[FPGA_LOAD_BLK_LENGTH];
unsigned int length_read, position, pkt_length;
unsigned int size, originalsize;
char ch;
int first_block, terminate;

if (*filename == '\0') return;                            /* If real filename */
file = fopen(filename, "r"); if (file == NULL) return;           /* Open file */

first_block = (0==0);
terminate   = (0==1);

while (!terminate
    && ((length_read = fread(data, 1, FPGA_LOAD_BLK_LENGTH, file)) > 
0))/*TEST@@@ */
  {
  if (first_block)                 /* Check device compatibility; send header */
    {
    if ((length_read == FPGA_LOAD_BLK_LENGTH)/* First block must be full (?!) */
      && callback_FPGA_header(data, container, &size, 0==0))
      {
      if (VERBOSE) g_print("size:%x\n",size);

      originalsize = size;
      board_sendchar(BR_FR_FILE);                          /* Download header */
      board_sendchar(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(container),
                                                  "feature"))); /* Feature No */
      board_sendb32(size);                                     /* File length */

      board_getchar(&ch); if ('A' != ch) terminate = (0==0);          /* Ack. */
      first_block = (0==1);
      }
    else terminate = (0==0);                            /* Some sort of error */
    }

  for (position = 0;
      (position < length_read) && !terminate;
       position = position + FPGA_LOAD_PKT_LENGTH)
    {
    pkt_length = MIN(size, FPGA_LOAD_PKT_LENGTH);
    if (pkt_length != 0)                   /* "size" is no of bytes remaining */
      {
      board_sendchar(BR_FR_SEND);                          /* Download packet */
      board_sendchar(GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(container),
                                              "feature")));      /*Feature No */
      board_sendchar(pkt_length);                            /* Packet length */
      board_sendchararray(pkt_length, &data[position]);               /* Data */

      board_getchar(&ch);
      if ('A' != ch) terminate = (0==0);                  /* Ack. from client */
      size = size - pkt_length;
      }
    if (size == 0) terminate = (0==0);             /* Should terminate anyway */
    }

  /* Intersperse loading with other services */
  if (!board_micro_ping()) terminate = (0==0);
             /* if errored with microping then do again (NOT MY COMMENT! @@@) */

  gtk_progress_bar_update(GTK_PROGRESS_BAR(gtk_object_get_data(
            GTK_OBJECT(container),"progressbar")),1-(float)size/originalsize);
                                                       /* update progress bar */
  while (gtk_events_pending()) gtk_main_iteration();
                                          /* update display and do user stuff */
  }

        /* DOES THIS MALLOC?  IT ISN'T FREED @@@ */
list = gtk_object_get_data(GTK_OBJECT(container), "list");
                                 /* append filename to list of past filenames */

if (!g_list_find_custom(list, filename, (GCompareFunc) g_strcasecmp))
  list = g_list_prepend(list, g_strdup(filename));

gtk_object_set_data(GTK_OBJECT (container), "list", list);
gtk_combo_set_popdown_strings(GTK_COMBO(gtk_object_get_data(
                              GTK_OBJECT(container),"combo")), list);

fclose (file);
return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/





/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Load button pressed                                                        */
/* Read the header information from a Xilinx bit file                         */
/* Display some of it (?)                                                     */


void callback_fpgaupdate(GtkButton* button, gpointer container)
{
char* filename = gtk_entry_get_text(GTK_ENTRY(gtk_object_get_data(
                                    GTK_OBJECT(container), "fileentry")));
                                                              /* Get filename */
uchar data[FPGA_HEADER_BUFFER];
FILE* file;     
unsigned int size;

/*   REPLACED  if (!*filename) return;  */

if (*filename == '\0') return;                            /* If real filename */
file = fopen(filename, "r"); if (file == NULL) return;           /* Open file */

if (fread(data, 1, FPGA_HEADER_BUFFER, file) == FPGA_HEADER_BUFFER)
       /* Get first few bytes from file; quit if not enough bytes read (WHY?) */
  callback_FPGA_header(data, container, &size, 0==1);

fclose (file);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/





/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void callback_client_set_status(int running, int timer_value)
{
  if (view_global_refresh_timer) gtk_timeout_remove(view_global_refresh_timer);
  view_global_refresh_timer = timer_value;
  board_running = running;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (view_refreshbutton), running);
  return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/



/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* start button pressed - either from panel or from neu entry                 */

void callback_button_start(GtkButton* button, gpointer steps)
{
//   mem_window_list* mlist = view_memwindowlist;
//   reg_window_list* rlist = view_regwindowlist;
int board_state; /* stores the current state of the board */

board_state=board_enq(); 
if ( board_state== CLIENT_STATE_RUNNING_SWI ||
     board_state== CLIENT_STATE_RUNNING ||
     board_state== CLIENT_STATE_MEMFAULT ||	
     board_state== CLIENT_STATE_BUSY) return;  /* might need modification */

board_sendchar(board_startcommand);
board_sendb32(*((int*)steps));                            /* start board */
callback_client_set_status(TRUE, FALSE);
  
//   while (NULL != rlist)
//      { /* for all regbanks */
//      if (!rlist->reg_data_ptr->timer)
//          rlist->reg_data_ptr->timer = g_timeout_add(rlist->reg_data_ptr->wait,
//                   callback_update_regwindow,rlist->reg_data_ptr);
//                                          /* update every timer milisecs*/
//      rlist=rlist->next;
//      }
//      
//   while (NULL != mlist)
//      { /* for all memlists */
//      if (!mlist->mem_data_ptr->timer)
//          mlist->mem_data_ptr->timer = g_timeout_add(mlist->mem_data_ptr->wait,
//                    callback_update_memwindow, mlist->mem_data_ptr);
//                                          /* update every timer milisecs*/
//      mlist=mlist->next;
//      }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/








/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* called when stop is chosen in the cases as above			      */


void callback_button_stop(GtkButton* button,gpointer ignore)
{

//   mem_window_list* mlist = view_memwindowlist;
//   reg_window_list* rlist = view_regwindowlist;

board_sendchar(BR_STOP);
gtk_label_set_text(GTK_LABEL(view_enqlabel),"Stopped");
callback_client_set_status(FALSE, FALSE);             /* unset refresh button */
callback_global_refresh();

//   while (NULL != rlist){ /* for all mem lists */
//      if (rlist->reg_data_ptr->timer)
//          gtk_timeout_remove(rlist->reg_data_ptr->timer);
//    view_updateregwindow(rlist->reg_data_ptr);     /* stop timer and refresh*/
//      rlist->reg_data_ptr->timer=0;
//      rlist=rlist->next;
//      }
//   while (NULL != mlist){ /* for all regbanks */
//      if (mlist->mem_data_ptr->timer)
//          gtk_timeout_remove(mlist->mem_data_ptr->timer);
//      view_updatememwindow(mlist->mem_data_ptr);  /* stop timer and refresh */
//      mlist->mem_data_ptr->timer=0;
//      mlist=mlist->next;
//      }

}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  called when continue is chosen in the cases as in 'start'		      */

void callback_button_continue(GtkButton* button, gpointer ignore)
{
board_sendchar(BR_CONTINUE);
callback_client_set_status(TRUE, FALSE);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* walk button pressed or menu entry activated				      */

void callback_button_walk(GtkButton* button, gpointer steps)
{
board_sendchar(board_startcommand);
board_sendb32(*((int*)steps));                                  /* walk board */
callback_global_refresh();
gtk_label_set_text (GTK_LABEL(view_enqlabel),"Walking");
callback_client_set_status(FALSE,
                           g_timeout_add(view_step_freq, callback_walk, steps));
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/





/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

void callback_start_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
if ((gtk_toggle_button_get_active(togglebutton)
  != GPOINTER_TO_INT(user_data) & 0x100))
  board_startcommand = board_startcommand |  GPOINTER_TO_INT(user_data);
else
  board_startcommand = board_startcommand & ~GPOINTER_TO_INT(user_data);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/





/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  Called when service to interrupts is enabled or disabled	              */
/*  user_data is 0x2 if called from IRQ					      */
/*  user_data is 0x1 if called from FIQ			                      */
/*									      */

void callback_rtf_toggle (GtkToggleButton *togglebutton, gpointer user_data)
{
 uchar rtf; 				/* used to store response from board  */
 
 board_sendchar(BR_RTF_GET);
 if (!board_getchar(&rtf))		/* if no response received	      */
   if (!board_mini_ping(0) ||
       !board_getchar(&rtf)) return; 
 					/* try again and abort if unsuccessful*/
 if (gtk_toggle_button_get_active(togglebutton)) /* if the toggle button is now activated */
        rtf |=  GPOINTER_TO_INT(user_data); /* set the appropriate service (in a bitwise manner) */
 else   rtf &= ~GPOINTER_TO_INT(user_data);
/* if the toggle button is now deactivated rtf is set to unset the appropriate service */
 
 
 board_sendchar(BR_RTF_SET);		/* send header			 */
 board_sendchar(rtf); 			/* send new rtf			 */
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/







/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  Currently does nothing, but the state of the toggle button is inspected   */
/*  in other functions to enable the right behaviour			      */
/*									      */
	
void callback_refresh_toggle (GtkToggleButton *togglebutton, gpointer user_data)
{
  
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/





/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  Called when the ping button is pressed or its entry in the menu is	      */ 
/*  activated								      */
/*									      */

void callback_button_ping (GtkButton* ignore, gpointer ignore2)
{
  board_micro_ping();	     /* ping the board 					*/
  board_enq(); 		     /* get and display the current state of the board	*/
  callback_global_refresh(); /* refresh all memory and register lists		*/
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* reset button pressed or menu entry activated				      */

void callback_button_reset (GtkButton* ignore, gpointer ignore2)
{
board_sendchar(BR_RESET);
board_micro_ping();
callback_client_set_status(FALSE, FALSE);             /* unset refresh button */
board_micro_ping();
board_enq();                                /* Update client behaviour report */
callback_global_refresh();                                     /* refresh all */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/




/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  Refresh all the memory lists and register banks			      */
/*									      */

void callback_global_refresh(void)
{
  mem_window_list* mlist = view_memwindowlist; /* get list of memory windows */
  reg_window_list* rlist = view_regwindowlist; /* get list of register banks */
  int regbank;				       /* index to hold the current  */
  					       /* register bank processed    */
  
  while (NULL != rlist)
    {
     view_updateregwindow(rlist->reg_data_ptr);     /* refresh 		*/
     rlist=rlist->next;				    /* and step ahead 	*/
    }
   for (regbank=0;regbank<board_num_regbanks;regbank++)
         if (1 == board_reg_banks[regbank].pointer) /* if pointer==1 	*/ 
            if (!board_get_regbank(regbank))	    /* if unable to get reg. bank */
	      {
                if (!board_mini_ping(0) || !board_get_regbank(regbank)) 
		   return; /* if seems like access is lost, abort refresh */
              }

  while (NULL != mlist)
     {
     view_updatememwindow(mlist->mem_data_ptr); /* refresh memory	*/
     mlist=mlist->next;				/* and step ahead 	*/
     }
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/



/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  Refresh all the memory lists one by one		 		      */
/*									      */

void callback_memory_refresh(void)
{
  mem_window_list* mlist = view_memwindowlist; /* get list of memory windows */
  while (NULL != mlist)
    {
     view_updatememwindow(mlist->mem_data_ptr); /* refresh 		*/
     mlist=mlist->next;				/* and step ahead 	*/
     }
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/






/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*  called when the user changes the step number for multistep		      */
/*									      */

void callback_step_number(GtkEditable *editable, gpointer value)
{   
  *((int*)value) = gtk_spin_button_get_value_as_int   /* change the step number for multistep*/
                              (GTK_SPIN_BUTTON (editable));
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/




/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* A new representation for memory has been picked and new settings need to   */
/* be taken into consideration						      */

void callback_memwindow_length (GtkMenuItem* menuitem, gpointer newvalue) /* the line width and gran */
{
  int temp;
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (menuitem)); /* memory window pointer */
  /***** the ascii entry and dis. will have to check state of column visibility and callback the ascii and dis. if true */
  switch (GPOINTER_TO_INT(newvalue))
  {  		
    /* the following cases bind a new values to reflect the representation and
       make sure the appropraite callbacks are invoked w.r.t. ASCII and disassembly */
         
     case REPRESENTATION_BYTE:     memwin->width = 1;
                   		   memwin->gran = 1;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
				   break; 
     case REPRESENTATION_2_BYTES:  memwin->width = 2;
                   		   memwin->gran = 1;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
		                   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break; 			          
     case REPRESENTATION_4_BYTES:  memwin->width = 4;
                   		   memwin->gran = 1;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
		                   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
		                   break;    
     case REPRESENTATION_8_BYTES:  memwin->width = 8;
                   		   memwin->gran = 1;				   
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;
     case REPRESENTATION_16_BYTES: memwin->width = 16;
                   		   memwin->gran = 1;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;		                  
     case REPRESENTATION_HALFWORD: memwin->width = 2;
                   		   memwin->gran = 2;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;			   				   	 	 
     case REPRESENTATION_2_HALFWORDS:
       				   memwin->width = 4;
                   		   memwin->gran = 2;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);		                   
		                   break;
     case REPRESENTATION_4_HALFWORDS:
       				   memwin->width = 8;
                   		   memwin->gran = 2;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;			   			   
     case REPRESENTATION_8_HALFWORDS:
       				   memwin->width = 16;
                   		   memwin->gran = 2;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;			   				   
     case REPRESENTATION_WORD:     memwin->width = 4;
                   		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
		                   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
		                   break;
     case REPRESENTATION_2_WORDS:  memwin->width = 8;
                  		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
		                   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);		                   
		                   break;			   				   
     case REPRESENTATION_4_WORDS:  memwin->width = 16;
                  		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);		                   
		                   break; 
     case REPRESENTATION_8_WORDS:  memwin->width = 32;
                  		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);		                   
		                   break;			   
     case REPRESENTATION_16_WORDS: memwin->width = 64;
                  		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
		                   break;
     case REPRESENTATION_ASCII_MAP: 
                                   memwin->width = 64;
                  		   memwin->gran = 4;	     	   
       				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE); 			   				  				   
				   break;				   
     default:		           if (VERBOSE)
		     			g_print("Program unexpectedly went out of control. Inexistent representation was passed.\n");
     			   	   memwin->width = 4;
                   		   memwin->gran = 4;
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),FALSE);
		                   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->ascii_button),TRUE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),FALSE);
				   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memwin->dis_button),TRUE);
		                   break;		   
				   		 
  }	       		 
 
gtk_clist_freeze (GTK_CLIST(memwin->clist_ptr)); /* freeze the current column list */ 
    
if (GPOINTER_TO_INT(newvalue)==REPRESENTATION_ASCII_MAP) /* abnormal settings for this exceptional case */
 {  
    for (temp=ASCII_ENTRY-1;temp>0;temp--) /* all hex to be disabled */
    gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp, FALSE);
    gtk_widget_hide(memwin->hex_entry); /* hide hex entry */
    gtk_widget_set_usize (memwin->ascii_entry, 450, -2); /* set ASCII column size */
 } 
   
else /* normal hex representation */
 {
  gtk_widget_show(memwin->hex_entry); /* display hex entry in case of return from ascii map */
  for (temp=ASCII_ENTRY-1;temp>(memwin->width/memwin->gran);temp--)
    gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp, FALSE); /* disable all columns that need not be displayed */
  
  while(temp) /* from the top hex column e.g. 4 for 4th hex, where 0 is address column */
    {
      gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp,TRUE);/* make hex columns visible where necessary*/
      gtk_clist_set_column_width (GTK_CLIST (memwin->clist_ptr), temp,
                                      (memwin->width*25)/(memwin->width/memwin->gran)-7); /* and set their width */
      temp--; /* decrement until reaching 0 which is address column */
    }    
  gtk_widget_set_usize (memwin->hex_entry, (memwin->width*25), -2); /* set size of hex entry */	
  gtk_clist_set_column_width (GTK_CLIST (memwin->clist_ptr), ASCII_ENTRY,
                                      (memwin->width*10)); /* set width of ascii entry */
  gtk_widget_set_usize (memwin->ascii_entry, (memwin->width*10)+7, -2); /* set size of ascii entry */
//  if (8 == memwin->width) gtk_widget_set_usize (memwin->dis_entry, 507, -2);/* if double disassembly*/
//  else                    gtk_widget_set_usize (memwin->dis_entry, 257, -2);
  }  
  
  
gtk_clist_thaw (GTK_CLIST(memwin->clist_ptr)); /* display the list as it was newly defined */
view_updatememwindow(memwin); /* update the contents displayed */
}






/**** NOTE **** NOTE **** NOTE **** NOTE **** NOTE **** NOTE **** NOTE **/
/** This can probably be cleaned up or merged				*/

void callback_set_memory_window_size(GtkMenuItem* menuitem, gpointer new_representation) /* the line width and gran */
{
  int temp;
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (menuitem));
  memwin->width = 32;          /* currently no choice, but can be 
  				  extended to a switch statement 	*/
  memwin->gran = 4;
  gtk_clist_freeze (GTK_CLIST(memwin->clist_ptr)); 

  for (temp=ASCII_ENTRY-1;temp>(memwin->width/memwin->gran);temp--)
    gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp, FALSE);
  while(temp)
   {
    gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp,TRUE);/* make hex columns visible or not*/
    gtk_clist_set_column_width (GTK_CLIST (memwin->clist_ptr), temp,
                                      (memwin->width*25)/(memwin->width/memwin->gran)-7);
    temp--;
   }                                   
  gtk_clist_thaw (GTK_CLIST(memwin->clist_ptr)); 
  view_updatememwindow(memwin);
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/






/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* A new ISA description has been chosen from the selection menu e.g MIPS     */
/*									      */

void callback_memwindow_isa (GtkMenuItem* menuitem, gpointer newvalue) /* the line width and gran */
{
  mem_window* memwin;			/* will be pointer to current memory window */
  
  memwin= view_getmemwindowptr(GTK_WIDGET (menuitem));
  memwin->isa = (GList*) newvalue;	/* save the new value */
  view_updatememwindow(memwin);		/* update the memory window that is pointed to */
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/








/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* change number of rows - a new value has now been selected		      */
/* to show more memory entries						      */

void callback_memwindow_listsize (GtkEditable *editable, gpointer ignore)
{
  char *startdata[] = {"Address","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","ASCII","DISS"};
  int new_number_of_rows;
  mem_window* memwin;		/* will be pointer to current memory window */
		  
  new_number_of_rows = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (editable));
 			 /* retrieve number of rows */
  memwin = view_getmemwindowptr(GTK_WIDGET (editable)); 
  	/* get a pointer to the memory window currently requiring new num. of rows */
  gtk_clist_freeze (GTK_CLIST(memwin->clist_ptr)); 
  			/* freeze the current state of the memory list */
  if (memwin->count<new_number_of_rows)	/* if decrease in num. of rows */
    while (memwin->count<new_number_of_rows)
	  gtk_clist_insert (GTK_CLIST(memwin->clist_ptr),++memwin->count,startdata); /* add or kill rows*/
  if (memwin->count>new_number_of_rows)        /* if increase in num. of rows */
    while (memwin->count>new_number_of_rows)
      gtk_clist_remove (GTK_CLIST(memwin->clist_ptr),--memwin->count);
  view_updatememwindow(memwin); 		/* update the window */
  gtk_clist_thaw (GTK_CLIST(memwin->clist_ptr)); /* refresh the clist state */
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/




/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Display/hide ASCII column						      */
/* Called when relevant toggle buttons are pressed			      */

void callback_memwindow_ascii_toggle (GtkToggleButton* tbutton, gpointer ignore)
{
  mem_window* memwin;
		  
  memwin = view_getmemwindowptr(GTK_WIDGET (tbutton)); 
  				/* pointer to current memory window */   
  
  if(gtk_toggle_button_get_active (tbutton)) /* currently inactive	 */
    {
      gtk_widget_show (memwin->ascii_entry);
      gtk_clist_set_column_visibility (GTK_CLIST (memwin->clist_ptr), ASCII_ENTRY, TRUE);
    }
    
  else					     /* already active		 */
    {
      gtk_widget_hide (memwin->ascii_entry);
      gtk_clist_set_column_visibility (GTK_CLIST (memwin->clist_ptr), ASCII_ENTRY, FALSE);
    }
}
  
/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/




void callback_memwindow_elf_symbols_toggle(GtkToggleButton* tbutton, gpointer ignore)
{
  mem_window* memwin;
  
  memwin = view_getmemwindowptr(GTK_WIDGET (tbutton)); 
  				/* pointer to current memory window */       
  view_updatememwindow(memwin);
  /* update the memory window in case symbols should be retrieved or not now */
}

/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Display/hide Disassembly column					      */
/* Called when relevant toggle buttons are pressed		     	      */

void callback_memwindow_dis_toggle (GtkToggleButton* tbutton, gpointer ignore)
{
  mem_window* memwin;
  
  memwin = view_getmemwindowptr(GTK_WIDGET (tbutton)); 
  				/* pointer to current memory window */      
  
  if(gtk_toggle_button_get_active (tbutton))/* currently inactive	 */
    {
      gtk_widget_show (memwin->dis_entry);
      gtk_clist_set_column_visibility (GTK_CLIST (memwin->clist_ptr), DIS_ENTRY, TRUE);
    }
    
  else 					   /* already active		 */
    {
      gtk_widget_hide (memwin->dis_entry);
      gtk_clist_set_column_visibility (GTK_CLIST (memwin->clist_ptr), DIS_ENTRY, FALSE);
    }
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/





void callback_memwindow_address (GtkEditable* editable, gpointer ignore)
{
  int ptr=0;
  char saved;
  int regbanks=-1;
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));      /*New address passed*/
  char* text = gtk_editable_get_chars(editable,0,-1);
  memwin->regoffset = board_nulreg;
  while (' '==text[ptr]) ptr++; /* step to beginning of address */
  if ('#'==text[ptr])
   {                                          /*if starts with # then link to register*/
    text[ptr] = ' ';
    while ('+'!=text[ptr] && '-'!=text[ptr] &&text[ptr]) ptr++;
    saved = text[ptr];
    text[ptr] = '\0';
    ptr=-1;
    while (ptr<0 && ++regbanks<board_num_regbanks) 
      {
        if (board_reg_banks[regbanks].pointer)
	        ptr = view_findstrinarr(text, board_reg_banks[regbanks].names,
                                           board_reg_banks[regbanks].number); /* find link register value address*/
      }    			
    if (ptr<0)
	  {
	   g_free(text);
	   return;
	  }
    memwin->regoffset=&board_reg_banks[regbanks].values  /* memwin->regoffset is linked register value*/
                [ptr * board_reg_banks[regbanks].width];  
    for (ptr=0;text[ptr];ptr++);
    text[ptr]=saved;
    text[--ptr]='0';                                            /* replace Register Name with '0'*/
   }
   view_hexstr2chararr(board_memory_ptr_width, &text[ptr], memwin->address); /* get address */
   g_free(text);
   view_updatememwindow(memwin);
 }




void callback_memwindow_hex (GtkEditable* editable, gpointer ignore) /* return pressed in hex entry*/
{
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));
  char* text = gtk_editable_get_chars(GTK_EDITABLE(memwin->address_entry),0,-1);
  uchar* hex;
  int row;
  uchar* address = g_new(uchar,board_memory_ptr_width);
  uchar* new_address;

  if (!view_hexstr2chararr(board_memory_ptr_width, text, address))  /* get address from entry*/
        {
    	g_free(text);
	 g_free(address);
	 return;
	}
  if (board_wordalign) address[0] &= -(memwin->gran); /* word align */
  g_free(text);
  hex = g_new(uchar,memwin->gran);
  text = gtk_editable_get_chars(GTK_EDITABLE(memwin->hex_entry),0,-1); /* get hex value*/
  if (!view_hexstr2chararr(memwin->gran, text, hex))
     {
       g_free(text); 
       g_free(hex);
        g_free(address);
	return;
     }
  g_free(text);
  board_set_memory(1,address,hex,memwin->gran);      /* set memory*/
  g_free(hex);
  new_address = view_chararrAddchararr(board_memory_ptr_width,
                         memwin->address, memwin->regoffset);
  row = view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, new_address)
                                                              + memwin->gran; /* move row one forward*/
  g_free(new_address);
  g_free(address);
  if (row == memwin->count * memwin->width)
   {  /* if at end then move list one foward*/
    view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
    row -= memwin->width;
   }
  if (row < memwin->count * memwin->width)
        gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
                                       ,(row%memwin->width)/memwin->gran+1);
       /* selection the appropriate row now */                             
 
  callback_memory_refresh(); /* refresh all */
  while (gtk_events_pending())  gtk_main_iteration();                /*update display */
  gtk_entry_select_region (GTK_ENTRY(memwin->hex_entry),0,-1); /* select all of region*/
}





/******************************************************************************/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Takes ascii from editable and displays hex in the relevant region          */
/* 									      */

void callback_memwindow_ascii (GtkEditable* editable, gpointer ignore)
{ 
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));
  char* ascii_from_entry = gtk_editable_get_chars(GTK_EDITABLE(memwin->ascii_entry),0,-1);
  					 /* get ascii entry */
  char* ascii_as_hex=g_new(uchar,board_memory_ptr_width); 
  					/* variable to hold the above ascii sequence as hexadecimal */
  char* memory_address_string = gtk_editable_get_chars(GTK_EDITABLE(memwin->address_entry),0,-1); 
  					/* get address */
  int i=0, ascii_as_hex_index=0, ascii_entry_size; /* index */
  uchar* memory_address_unsigned = g_new(uchar,board_memory_ptr_width); 
  					/* unsigned to hold to dest. memory address */ 
  uchar* new_memory_contents_unsigned=g_new(uchar,board_memory_ptr_width); 
  					/* unsigned to hold to dest. memory contents */
  char current_character; 		/* to hold character by character */
  char top_nibble; 			/* to hold top nibble of character */
  char bottom_nibble; 			/* to hold bottom nibble of character */
  
  
  while (ascii_from_entry[i]!='\0') i++; /* find length of ascii entry */
  ascii_entry_size=i; 			/* save it - for readability */
  if (ascii_entry_size>=memwin->gran) 		/* only for proper size entries */
  {
  for (i=memwin->gran-1;i>=0;i--) 	/* for all expected characters - extra chars will be ignored */
    {
     current_character=ascii_from_entry[i]; // step character by character
     top_nibble = get_ascii_of_char_top_nibble(current_character);
     bottom_nibble = get_ascii_of_char_bottom_nibble(current_character);
     ascii_as_hex[ascii_as_hex_index]=top_nibble; // see below
     ascii_as_hex_index++; // advance position
     ascii_as_hex[ascii_as_hex_index]=bottom_nibble;    // put the bytes in 2 correct positions where  e.g. 4th character goes
 						        // into 7th and 8th position
     ascii_as_hex_index++; // advance position
    }
  ascii_as_hex[ascii_as_hex_index]='\0'; 		// break the string

  view_hexstr2chararr(memwin->gran, memory_address_string, memory_address_unsigned); /* unsigned for memory address */
  view_hexstr2chararr(memwin->gran, ascii_as_hex, new_memory_contents_unsigned); /* unsigned for new contents */
  board_set_memory(1,memory_address_unsigned,new_memory_contents_unsigned,memwin->gran); /* set memory */  
  
  callback_memory_refresh();
  }
}

/*									      */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/******************************************************************************/







void callback_memwindow_dis (GtkEditable* editable, gpointer ignore)
{
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));
  char* text = gtk_editable_get_chars(GTK_EDITABLE(memwin->address_entry),0,-1);
  uchar* hex;
  int row;
  int temp;
  uchar* address = g_new(uchar,board_memory_ptr_width);
  uchar* new_address;
  Asm* asmed;
  GList* asmlist;
  if (!view_hexstr2chararr(board_memory_ptr_width, text, address))  /* get address from entry*/
    {
     g_free(text);
     g_free(address);
     return;
    }
  if (board_wordalign) address[0] &= -(memwin->gran); /* word align */
  g_free(text);  
  text = gtk_editable_get_chars(GTK_EDITABLE(memwin->dis_entry),0,-1); /* get hex value*/
  asmlist = asm_assemble(text,memwin->isa, view_chararr2int(board_memory_ptr_width, address),1);
  if (!asmlist)
    {
      g_free(text); 
      g_free(address);
       return;
    }
  asmed = (Asm*) (asmlist->data);
  if (asmed->binary == &dis_error) 
     {
       g_free(text);
       g_free(address); 
       return;
     }
  temp = asm_sbprint(asmed->binary, &hex);
  if (asmlist->next) 
      if (VERBOSE) 
           g_print("Multiple possible assemblies, size:%d",g_list_length(asmlist));
  while(asmlist) 
    {
      asm_freestringlist(((Asm*)asmlist->data)->binary);
      g_free(asmlist->data);
      asmlist = g_list_remove(asmlist,asmlist->data);
    }    
  if (memwin->gran != temp)
       {
          g_free(hex); 
	  g_free(text); 
	  g_free(address); 
	  return;
       } 
  board_set_memory(1,address,hex,memwin->gran);      /* set memory*/
  g_free(hex);
  new_address = view_chararrAddchararr(board_memory_ptr_width,
                         memwin->address, memwin->regoffset);
  row = view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, new_address)
                                                              + memwin->gran; /* move row one forward*/
  g_free(new_address);
  g_free(address);
  if (row == memwin->count * memwin->width)
    {  /* if at end then move list one foward*/
      view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
      row -= memwin->width;
    }
  if (row < memwin->count * memwin->width)
      gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
                                       ,(row%memwin->width)/memwin->gran+1);
  gtk_editable_select_region (GTK_EDITABLE(memwin->dis_entry),0,-1); /* select all of region*/
  if (gtk_events_pending()>2)
      while (gtk_events_pending())  gtk_main_iteration();                /*update display */
  else callback_memory_refresh(); /* refresh all */
}





void callback_memwindow_scrollmove(GtkAdjustment *adjustment,gpointer ignore2) /* move scroll bar*/
{
  uchar temp[4];
  mem_window* memwin = gtk_object_get_data (GTK_OBJECT(adjustment),"mem_window");
  if (ABS(adjustment->value)<0.05) 
      {
       view_updatememwindow(memwin);
       return;
      }
  view_chararrCpychararr(board_memory_ptr_width,memwin->address,temp);
  if (adjustment->value<0) view_chararrAddintsame(board_memory_ptr_width,   /* move to new position*/
		  memwin->address ,-pow(16,-adjustment->value) * memwin->width);
  else  view_chararrAddintsame(board_memory_ptr_width,
		  memwin->address , pow(16, adjustment->value) * memwin->width);  
  view_updatememwindow(memwin);
  view_chararrCpychararr(board_memory_ptr_width,temp, memwin->address); /* refresh and put back to original */
}

gboolean callback_memwindow_scrollrelease(GtkWidget *scroll,gpointer ignore1, /* let go off scroll bar*/
                                                            gpointer ignore2)
{
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (scroll));
  if (ABS(gtk_range_get_adjustment(GTK_RANGE(scroll))->value)<0.1) return FALSE;
  if (gtk_range_get_adjustment(GTK_RANGE(scroll))->value<0)
   view_chararrAddintsame(board_memory_ptr_width, memwin->address,
    -pow(16,-gtk_range_get_adjustment(GTK_RANGE(scroll))->value) * memwin->width);   /* move to new position */
  else
   view_chararrAddintsame(board_memory_ptr_width, memwin->address,
     pow(16, gtk_range_get_adjustment(GTK_RANGE(scroll))->value) * memwin->width);
  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(scroll)),0);     /* move scroll bar back */
  view_updatememwindow(memwin);
  return FALSE;
}




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* a line in the memory window was selected                                   */

/*  Change =>  callback_memwindow_clist  with "wibble" set false @@@  */

void  callback_memory_window_row_selected(GtkCList *clist, gint row, gint 
column,GdkEventButton *event,gpointer ignore)
{
  mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (clist));
  uchar* new_address = view_chararrAddchararr(board_memory_ptr_width,
                              memwin->address, memwin->regoffset);
  uchar* address;
  char* address_text;
  uchar* memdata;
   
  if (board_wordalign) new_address[0] &= -memwin->width;              /* word align */
  address = view_chararrAddint(board_memory_ptr_width,
           new_address, row * memwin->width + 
                  ((column > 8) ? 0 : (column <= 0) ? 0 : column-1) * 
memwin->gran);   /* work out which column  */
  
  if (event)
   if (GDK_2BUTTON_PRESS == event->type) 
  {
    unsigned int worda;
    unsigned int wordb;
    board_sendchar(BR_BP_GET);
    if (4 != board_getb32(&worda)) {g_free(address);return;}
    if (4 != board_getb32(&wordb)) {g_free(address);return;}
    {
      int temp,temp2;
      uchar repc;
      int flag=FALSE;
      uchar* value=g_new(uchar,board_memory_ptr_width);
      for (temp=0;temp<32;temp++)
        if ((worda>>temp)&1)
            {
            value=g_new(uchar,board_memory_ptr_width);
            board_sendchar(BR_BP_READ);
            board_sendchar(temp);
            for (temp2=2;temp2;temp2--)
                if (1 != board_getchar(&repc)) {g_free(address);return;}
            if (board_memory_ptr_width != 
                board_getchararray(board_memory_ptr_width,value))
                                               {g_free(address);return;}
            for (temp2=board_memory_ptr_width+16;temp2;temp2--)
                if (1 != board_getchar(&repc)) {g_free(address);return;}
            if (!view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, value))
                    {
                      flag=TRUE;
                      board_sendchar(BR_BP_SET);
                      board_sendb32(0);
                      board_sendb32(1<<temp);
                    }
            }
       g_free(value);
       if (flag==TRUE) 
            {
             g_free(address);
             breakpoint_refresh (0);
             callback_memory_refresh(); /* refresh all */
             return;
            }
    }
    worda = (~worda) & wordb;
    wordb = 0;
    if (!worda)                 //if worda is 0x0
      {
      g_free(address);
      return;
      }
    while (!((worda>>wordb)&1)) wordb++;
    board_sendchar(BR_BP_WRITE);
    board_sendchar(wordb);
    board_sendb16(-1);
    board_sendchararray(board_memory_ptr_width, address);
    for (worda = board_memory_ptr_width; worda; worda--) board_sendchar(0xff);
    for (worda = 16; worda; worda--)                     board_sendchar(0);
    g_free(address);
    breakpoint_refresh (0);
    callback_memory_refresh(); /* refresh all */
    return;
 }

  memdata = g_new(uchar,memwin->gran);
  address_text = view_chararr2hexstrbe(board_memory_ptr_width,address);
  gtk_entry_set_text(GTK_ENTRY(memwin->address_entry),address_text);
  g_free(address_text);
  if  (!board_get_memory (1, address, memdata, memwin->gran))
    {    /* get value into the text entry*/
        if (!board_mini_ping(0) ||
       !board_get_memory (1, address, memdata, memwin->gran))
        { /* if that didnt work then dont panic just try again*/
        
        }
    } /* sorry but board is lost*/
  
  address_text = view_chararr2hexstrbe (memwin->gran,memdata);  /* set hex entry*/
  gtk_entry_set_text(GTK_ENTRY(memwin->hex_entry),address_text);
  g_free(address_text); 
  
  g_free(address);
  g_free(memdata);
  
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* a line in the column list representing the memory was pressed              */

void callback_memwindow_clist(GtkCList *clist, gint row, gint column,
                              GdkEventButton *event,gpointer ignore
/* INTERFACE CHANGE   int wibble  */
)
{
mem_window* memwin;
uchar* new_address;
uchar* address;
char*  address_text;
uchar* memdata;
int    error;

void callback_memwindow_set_hex()
  {
  address_text = view_chararr2hexstrbe(memwin->gran,memdata);/* set hex entry */
  gtk_entry_set_text(GTK_ENTRY(memwin->hex_entry), address_text);
  g_free(address_text); 
  return;
  }

void callback_memwindow_set_ASCII()
  {
  address_text = view_chararr2asciistr(memwin->gran,memdata);/*set ASCII entry*/
  gtk_entry_set_text(GTK_ENTRY(memwin->ascii_entry), address_text);  
  g_free(address_text);
  return;
  }

void callback_memwindow_set_dis()
  {
  address_text = view_dis(address, memdata, memwin->gran, memwin->isa );
                                                             /* set dis entry */
  gtk_entry_set_text(GTK_ENTRY(memwin->dis_entry), address_text);  
  g_free(address_text);
  return;
  }

error = 0;
memwin      = view_getmemwindowptr(GTK_WIDGET(clist));
new_address = view_chararrAddchararr(board_memory_ptr_width, memwin->address,
                                                             memwin->regoffset);

if (board_wordalign) new_address[0] &= -memwin->width;          /* word align */


address = view_chararrAddint(board_memory_ptr_width, new_address,
  row*memwin->width + ((column>16)?0 : (column<=0)?0 : column-1) * memwin->gran);
                                                     /* work out which column */

g_free(new_address);                                           /* ADDED  JDG  */

if (event)
  if (GDK_2BUTTON_PRESS == event->type) 
    {
    unsigned int worda, wordb;

    board_sendchar(BR_BP_GET);                      /* Read breakpoint status */
    if ((4 != board_getb32(&worda)) || (4 != board_getb32(&wordb))) error = 1;

    if (error == 0)
      {
      int brkpt_no, temp;
      uchar repc;
      int flag = FALSE;
      uchar* value;

      value = g_new(uchar, board_memory_ptr_width);  /* Breakpoint address(?) */

      for (brkpt_no = 0; brkpt_no < 32; brkpt_no++)
        if (((worda>>brkpt_no) & 1) != 0)    /* Defined? (active or inactive) */
          {
          board_sendchar(BR_BP_READ);           /* Read breakpoint definition */
          board_sendchar(brkpt_no);
          if ((1 != board_getchar(&repc))
           || (1 != board_getchar(&repc))
           || (board_memory_ptr_width != 
               board_getchararray(board_memory_ptr_width,value)))
              error = 2;
          else
            for (temp = board_memory_ptr_width + 16;  /* 16  WHAT? @@@ */
                 temp && (error == 0);
                 temp--)
              if (1 != board_getchar(&repc)) error = 3;

          if ((error == 0)
          && (!view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, value)))
            {                                          /* Addresses match (?) */
            flag = TRUE;
            board_sendchar(BR_BP_SET);               /* Set breakpoint status */
            board_sendb32(0);                                       /* Delete */
            board_sendb32(1<<brkpt_no);
            }
          }

      g_free(value);

      if (flag == TRUE)    /* breakpoint(s) matched and deleted ??? */
        {  
        breakpoint_refresh(0);
        callback_memory_refresh();                             /* refresh all */
        }
      else                                    /* See if we can set breakpoint */
        {
        temp = (~worda) & wordb;              /* Undefined => possible choice */
        if (temp != 0)                             /* If any free entries ... */
          {                                        /* Define (set) breakpoint */
          for (brkpt_no = 0; (((temp>>brkpt_no) & 1) == 0); brkpt_no++);
                                                     /* Choose free(?) number */

          board_sendchar(BR_BP_WRITE);                   /* Define breakpoint */
          board_sendchar(brkpt_no);
          board_sendb16(-1);                    /* Really two byte parameters */
                /* Should send 2*words address then two*double words data @@@ */
          board_sendchararray(board_memory_ptr_width, address);
          for (temp=board_memory_ptr_width; temp; temp--) board_sendchar(0xff);
          for (temp=16; temp; temp--)                     board_sendchar(0);

          breakpoint_refresh(0);
          callback_memory_refresh();                           /* refresh all */
          }
	}
      }	//end if not error
   } // end if two buttons pressed
      
      	/* the following will fill the appropriate fields upon a single mouse press */	      
        
        memdata = g_new(uchar, memwin->gran);

        address_text = view_chararr2hexstrbe(board_memory_ptr_width, address);
        gtk_entry_set_text(GTK_ENTRY(memwin->address_entry), address_text);
        g_free(address_text);

        if (!board_get_memory(1, address, memdata, memwin->gran))
          {                                  /* get value into the text entry */
          if (!board_mini_ping(0)
           || !board_get_memory(1,address,memdata,memwin->gran))
            {          /* if that didn't work then don't panic just try again */
            }
          }                                        /* sorry but board is lost */

        callback_memwindow_set_hex();

/*    @@@  FIX THIS UP WHEN INTERFACE CHANGES
        if (wibble)
*/
          {
          callback_memwindow_set_ASCII();
          callback_memwindow_set_dis();
          }

          g_free(memdata);
          g_free(address);
          return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 	the following is called when a new value has been put into a reg.     */


void callback_regwindow_hex (GtkEditable* editable,gpointer ignore)   /* register hex set*/
{
  reg_window* regwin = view_getregwindowptr(GTK_WIDGET (editable));
  char* text = gtk_editable_get_chars(GTK_EDITABLE(regwin->address_entry),0,-1);
  uchar* hex;
  int count;
  
  count = view_findstrinarr(text, board_reg_banks[regwin->regbank_no].names
		                        , board_reg_banks[regwin->regbank_no].number); /* find which register*/
  g_free(text);
  if (count<0) {return;}   /* if not there then do nothing */ 
  hex = g_new(uchar,MAX(1,board_reg_banks[regwin->regbank_no].width)); /* make space for the value*/
  text = gtk_editable_get_chars(GTK_EDITABLE(regwin->hex_entry),0,-1); /* get chars */
  if (!view_hexstr2chararr
          (MAX(1,board_reg_banks[regwin->regbank_no].width), text, hex))/* convert to bin*/
          {
            g_free(text);
	    g_free(hex);
	    return;
	  }
  g_free(text);
  board_set_register (regwin->regbank_no,count,hex); /* set register*/ 
  g_free(hex);
  view_updateregwindow(regwin); /* update regbank*/
  /* need to update all here!!! */
  callback_global_refresh(); /* refresh all */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/








void  callback_regwindow_clist (GtkCList *clist, gint row, gint column, /* c-list register bank pressed*/
                                GdkEventButton *event, gpointer ignore)
{
  reg_window* regwin = view_getregwindowptr(GTK_WIDGET (clist));
  char* value;
  if (board_reg_banks[regwin->regbank_no].width)                           /* get register value*/
    value = view_chararr2hexstrbe(board_reg_banks[regwin->regbank_no].width,
                             &board_reg_banks[regwin->regbank_no].values[
                              board_reg_banks[regwin->regbank_no].width * row]);
  else if (board_reg_banks[regwin->regbank_no].values[row]) value = "1";   /* if flag then 1 or 0*/
       else                                                 value = "0";
  
  gtk_entry_set_text(GTK_ENTRY(regwin->address_entry),
                     board_reg_banks[regwin->regbank_no].names[row]);
  gtk_entry_set_text(GTK_ENTRY(regwin->hex_entry),value);
  if (board_reg_banks[regwin->regbank_no].width)
     g_free(value);
}







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

gint callback_walk(gpointer steps)
{ 
int client_status = board_enq();

if (CLIENT_STATE_CLASS_RUNNING == client_status & CLIENT_STATE_CLASS_MASK)
  return 1;                              /* no purpose for walking if running */

if (CLIENT_STATE_STOPPED != client_status)    /* SURELY GENERIC STOP ? @@@ */
  callback_client_set_status(FALSE, FALSE);           /* unset refresh button */
else 
  {
  board_sendchar(board_startcommand | 1);
  board_sendb32(*((int*)steps));                                /* walk board */
  gtk_label_set_text (GTK_LABEL(view_enqlabel),"Walking");
  }

callback_global_refresh();
while (gtk_events_pending()) gtk_main_iteration();
                                          /* update display and do user stuff */
return 1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* the following 2 functions are callbacks for the console feature 	      */

void callback_comms(GtkEditable* entry, gpointer comms_console)
{
char* tosend = gtk_entry_get_text(GTK_ENTRY(entry));
                                                 /* get what needs to be sent */
int length = strlen(tosend);                                    /* set length */
int index = 0;
int sub;
feature* comms = comms_console;            /* reference to the console window */
char accepted;

if (-1 < board_version)
  {             /* board version is currently 0, but this needs to be changed */
  while (length>index)
    {
    board_sendchar(BR_FR_WRITE);                            /* begins a write */
    board_sendchar(comms->dev_number);              /* tells where to send it */
    sub = MIN(length - index, 255);             /* length of message in bytes */
    board_sendchar(sub);                                  /* send that length */
    board_sendchararray(sub, tosend + index);             /* send the message */
    board_getchar(&accepted);                       /* find what was accepted */
    index += sub;                                      /* step within message */
    }                                 /* NB This version ignores overruns @@@ */
 }

gtk_entry_set_text(GTK_ENTRY(entry), "");
return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/







gint callback_update_comms(gpointer comms)
{
 char* string; /* string to get the message from the board */
 char current_char; /* and one for storing chars  of string above */
   //NOTE: current_char is declared as an array (string) in order for GTK to cope
 uchar length;
 int i; // loop index
 feature* comms_console=comms;
 //if (VERBOSE)
 //   g_print("==========Polling=======\n");
 do
  {
   if (serial_lock==FREE)
   {
    serial_lock=CONSOLE_UPDATE;
    board_sendchar(BR_FR_READ);
    board_sendchar(comms_console->dev_number);
    board_sendchar(MAX_LENGTH); /* should be MAX_LENGTH */
    board_getchar(&length);
    if (length != 0) // non-zero received from board
       {
        string = g_new(char, length);
        board_getchararray(length,string);
	for (i=0;i<length;i++)
	  {
	   current_char=string[i]; /* get the Ith character */
	   if (current_char>=' ') /* if printable ASCII */
	     {
#ifndef GTK2
      		  gtk_text_insert (GTK_TEXT (comms_console->data.console.text), NULL, NULL, NULL,&current_char , 1);
#endif

#ifdef GTK2
     		  gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(comms_console->data.console.text),&current_char,1);
#endif
             }  
	    else // if below space, i.e. special ASCII
	     {	       
	          current_char=current_char+0x40; /* set it to the visible ASCII that follows the ^ */
		  switch (current_char) /* special actions */
		    {
		      case 'G': g_print("\a"); /* ring bell */
		                break;
		      case 'M': // new line
			        gtk_text_insert (GTK_TEXT (comms_console->data.console.text), NULL, NULL, NULL,"\n" , 1);
			        break;
		      case 'H':	 // backspace
				gtk_text_backward_delete(GTK_TEXT (comms_console->data.console.text),1);
				break;
		      case '!': // left shift pressed
			        break;
		      case '"':	// right shift pressed
			        break;	
		      default: /* no special action was carried out */         
		    	  
#ifndef GTK2
      		 		 gtk_text_insert (GTK_TEXT (comms_console->data.console.text), NULL, NULL, NULL,"^" , 1); // Print ^
		 		 gtk_text_insert (GTK_TEXT (comms_console->data.console.text), NULL, NULL, NULL,&current_char , 1);
#endif

#ifdef GTK2
     		 		 gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(comms_console->data.console.text),"^",1); // Print ^
		 		 gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(comms_console->data.console.text),&current_char,1);
#endif    	        
				 break;
	            }	 
	     }
	  }
	 g_free(string);  
       }
      serial_lock=FREE; 
    } // end if
  } //end do  
 while (255 == length); 
 return 1;
}






/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* wrapper function to go from timer interrupts                               */

gint callback_updateall(gpointer ignore)
{
 // board_micro_ping();  /* NOTE: seems to be redundant, causes double update */

if (serial_lock == FREE)
  {
  serial_lock = MEM_AND_REG_UPDATE;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view_refreshbutton)))
   {
   if (CLIENT_STATE_CLASS_RUNNING != (board_enq() & CLIENT_STATE_CLASS_MASK))
    {                              /* if the client claims to be not running */
    if (board_running)
      {        // if the board is believed within the program to be running
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(view_refreshbutton), FALSE);
      }
    board_running = FALSE;          /* unset the running flag in the program */
    }
  callback_global_refresh();

  while (gtk_events_pending()) gtk_main_iteration();
                                           /*update display and do user stuff */
   }
  serial_lock = FREE;      /* NB MOVED @@@ */
  }
return 1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* update either a register bank or a memory frame			      */

gint callback_update_memreg_window(int window_type, gpointer windowptr)
{
if (CLIENT_STATE_CLASS_RUNNING != (board_enq() & CLIENT_STATE_CLASS_MASK))
                                    /* if the client is not currently running */
  if (view_global_refresh_timer && board_running)
    {
    gtk_timeout_remove(view_global_refresh_timer);
    view_global_refresh_timer = FALSE;
    board_running = TRUE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (view_refreshbutton), FALSE);
    }

if (window_type == MEMORY_WINDOW)
  view_updatememwindow((mem_window*) windowptr);
else
  view_updateregwindow((reg_window*) windowptr);            /* Only two types */

while (gtk_events_pending()) gtk_main_iteration();
                                           /*update display and do user stuff */
return 1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* CURRENTLY UNREACHABLE - wrapper functions to go from timer interrupts and  */
/* update memory							      */

gint callback_updatememwindow(gpointer memwindowptr)    /* wrapper functions to go from timer interrupts */
{
  callback_update_memreg_window(MEMORY_WINDOW, memwindowptr);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/




/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* CURRENTLY UNREACHABLE - wrapper functions to go from timer interrupts and  */
/* update memory							      */


gint callback_updateregwindow(gpointer regwindowptr)
{
  callback_update_memreg_window(REGISTER_WINDOW, regwindowptr);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/






gint callback_console_update (gpointer data, gint source,
                                 GdkInputCondition condition)
{
  char getbuff[20];
  int  got;
  
got = read(compile_communication[0], getbuff, 20);
#ifndef GTK2
    gtk_text_insert (GTK_TEXT (view_console), NULL, NULL, NULL,getbuff , got);
#endif

#ifdef GTK2
    gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(view_console),getbuff,got);
#endif 
  return 1;
}


/* the following function will set or unset a flag i.e. change some bits in the
CPSR (as a response to flag press */


void callback_flag_pressed(GtkToggleButton *togglebutton, gpointer user_data)
{
char *cpsr_in_current;  		/* var. to store cspr in 'current' */
char msbyte_in_cpsr; 			/* character to stand for 4 flags */
unsigned char *new_value_to_set;	/*  @@@ */
reg_bank* regbank = &board_reg_banks[0];  /* get current reg. bank */
cpsr_in_current= view_chararr2hexstrbe (regbank->width,
                              &regbank->values[16 * regbank->width]); /* get CPSR */

/* get state of buttons */
nflag_current_boolean 
		= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nflag_current)); 
zflag_current_boolean 
		= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(zflag_current));
cflag_current_boolean
		= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cflag_current));
vflag_current_boolean 
		= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vflag_current));
msbyte_in_cpsr			/* make a byte out of the four flags */
	=byte_of_bits(nflag_current_boolean,zflag_current_boolean,cflag_current_boolean,
		            vflag_current_boolean); 
cpsr_in_current[0]=msbyte_in_cpsr;		 /* build new value of cpsr */

new_value_to_set=malloc(4); 
view_hexstr2chararr(8,cpsr_in_current,new_value_to_set);

  {   
   int x;  /* Temporary */
   x = board_enq() & CLIENT_STATE_CLASS_MASK;
   if ((x == CLIENT_STATE_CLASS_RESET) || (x == CLIENT_STATE_CLASS_STOPPED)) 
     board_set_register(0, CPSR, &new_value_to_set[0]);
     /* only set the register if the board is not running to avoid dirty read */
  }
free(new_value_to_set); /* freeing up to avoid memory leak */  
callback_global_refresh();  
}



/* the following function will set or unset a flag i.e. change some bits in the
SPSR  */

void callback_flag_pressed_saved(GtkToggleButton *togglebutton, gpointer user_data)
{
char *spsr_in_current;  		/* var. to store spsr in 'current' */
char msbyte_in_spsr; 			/* character to stand for 4 flags */
unsigned char *new_value_to_set;	/*  @@@ */
reg_bank* regbank = &board_reg_banks[0];  /* get current reg. bank */
spsr_in_current= view_chararr2hexstrbe (regbank->width,
                              &regbank->values[17 * regbank->width]); /* get spsr */
  
nflag_saved_boolean =  			/* get state of buttons */
		 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(nflag_saved));
zflag_saved_boolean =
		 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(zflag_saved));
cflag_saved_boolean =
		 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cflag_saved));
vflag_saved_boolean =
		 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vflag_saved));
msbyte_in_spsr=byte_of_bits(nflag_saved_boolean,zflag_saved_boolean,cflag_saved_boolean,
		            vflag_saved_boolean); /* make a byte out of the four flags */
spsr_in_current[0]=msbyte_in_spsr;  /* build new value of SPSR */

new_value_to_set=malloc(4);
view_hexstr2chararr(8,spsr_in_current,new_value_to_set);
  {
   int x;  /* Temporary */
   x = board_enq() & CLIENT_STATE_CLASS_MASK;
   if ((x == CLIENT_STATE_CLASS_RESET) || (x == CLIENT_STATE_CLASS_STOPPED))
     board_set_register(0, SPSR, &new_value_to_set[0]);
    /* only set the register if the board is not running to avoid dirty read */
  }
free(new_value_to_set);  /* freeing up to avoid memory leak */   
callback_global_refresh();
}


/* the following is called when the user made a selection of current mode */
/* CPSR will get a new value depending on the mode chosen */
void callback_mode_selection(GtkMenuItem* menuitem, gpointer newmode)
{ 
   callback_mode_selection_all((int)newmode, CPSR);
}    

/* the following is called when the user made a selection of current mode */
/* SPSR will get a new value depending on the mode chosen */
void callback_mode_selection_saved(GtkMenuItem* menuitem, gpointer newmode)
{		
   callback_mode_selection_all((int) newmode, SPSR);
}

void callback_mode_selection_all(int newmode, int PSR)
{
reg_bank* regbank = &board_reg_banks[0];
char     *psr_in_current;
unsigned char* new_value_to_set;               /* Temporary parameter space */
psr_in_current = view_chararr2hexstrbe (regbank->width,
                            &regbank->values[PSR * regbank->width]);
 /* get value of PSR in 'current' */  
switch (newmode)                 /* set to right mode according to input */
  {
  case 1: psr_in_current[7] = '0'; break;
  case 2: psr_in_current[7] = '3'; break;
  case 3: psr_in_current[7] = '7'; break;
  case 4: psr_in_current[7] = 'B'; break;
  case 5: psr_in_current[7] = '2'; break;
  case 6: psr_in_current[7] = '1'; break;
  case 7: psr_in_current[7] = 'F'; break;
  case 8: psr_in_current[7] = '4'; break;
  default:
    if (VERBOSE) g_print("Errorneous input to callback_mode_selection()\n");
    break;
  }
new_value_to_set=malloc(4);
view_hexstr2chararr(8, psr_in_current, new_value_to_set);
board_set_register (0, PSR, new_value_to_set);
free(new_value_to_set);
callback_global_refresh();    
}    



gint callback_console_keysnoop  (GtkWidget *grab_widget,
                                               GdkEventKey *event,
                                               gpointer comms)
{
  int key_pressed; /* to save the value of the key that invoked this function */
  uchar response_from_board;
  feature* comms_console=comms; /* record the feature from which the callback came */
  if ((grab_widget==text_in_console[comms_console->dev_number]))
    		/* only when called from the window defined and character is valid */
       {
  	//g_print("Got a key\n");
	key_pressed=event->keyval;
	if ((key_pressed & 0xFFFFFF00)>0) /* if more than one byte */
	    key_pressed=mask_one_byte(key_pressed);
	//g_print("Key val sent: %x\n",key_pressed);
	board_sendchar(BR_FR_WRITE); /* begins a write */
        board_sendchar(comms_console->dev_number); /* tells where to send it */
        board_sendchar(1); /* send length 1*/
        board_sendchar(key_pressed); /* send the message - 1 char currently*/
	board_getchar(&response_from_board);
        }
  gtk_propagate_event (grab_widget,(gpointer)event); /* fix warning*/
  return 1;
}

void callback_clear_console(GtkButton* ignore,gpointer console)
{
  int text_length;
  feature* comms_console=console; 
  text_length=gtk_text_get_length(GTK_TEXT (comms_console->data.console.text));
  gtk_text_backward_delete(GTK_TEXT (comms_console->data.console.text),text_length);	
}


/* user has selected new mode to be chosen in the register bank notebook */
void callback_mode_selected_from_menu(GtkWidget* ignore,gpointer mode_number)
{
gtk_notebook_set_page(GTK_NOTEBOOK(register_notebook),(int)mode_number);
}

void callback_flags_selected_from_menu(GtkWidget* ignore,gpointer mode_number)
{
gtk_notebook_set_page(GTK_NOTEBOOK(flags_notebook),(int)mode_number);
}

void callback_main_quit(gpointer ignore, gpointer ignore2)
{
exit(1);
}
