#ifndef CALLBACKS_H
#define CALLBACKS_H
#include <glib.h>
#include <gtk/gtk.h>
#include <stdio.h>

#define MEMORY_WINDOW   0 /* this is used when updating either memory or registers */
#define REGISTER_WINDOW 1 /* this is used when updating either memory or registers */


#define uchar unsigned char
#define REFRESH_PERIOD 1000 /* the interval between each refresh on the screen

				i.e. how often the values on board are inspected */
#define POLLING_PERIOD 100
#define MAX_LENGTH 32 /* maximum length of message returned from the board */		


#define CPSR 16 /* defining CPSR and SPSR register numbers */
#define SPSR 17

#define FPGA_LOAD_PKT_LENGTH 0x10               /* POSITION */
#define FPGA_LOAD_BLK_LENGTH 0x100/*Must be a multiple of FPGA_LOAD_PKT_LENGTH*/

#define FPGA_HEADER_BUFFER 0x80           /* Large enough for any header?  */
					
void callback_memwindow_elf_symbols_toggle(GtkToggleButton* tbutton, gpointer ignore);
gint callback_update_memreg_window(int window_type, gpointer windowptr);
void callback_client_set_status(int running, int timer_value);					
int callback_FPGA_header(uchar* buffer, gpointer container, unsigned int* size,int load);
void callback_mode_selected_from_menu(GtkWidget*,gpointer); 
void callback_flags_selected_from_menu(GtkWidget*,gpointer);
/* user has selected new mode to be chosen in the register bank notebook */
void callback_clear_console(GtkButton*, gpointer);
void callback_button_load (GtkButton*, gpointer);
void callback_load_binary_address(GtkEntry*, gpointer);
void callback_load_binary(GtkButton*, gpointer);
void callback_button_compile (GtkButton*, gpointer);
void callback_button_ok_file(GtkButton*, gpointer);
void callback_fpgaload (GtkButton*, gpointer);/* called in features when download pressed*/
void callback_fpgaupdate (GtkButton*, gpointer);  /* called in features when update pressed*/
void callback_button_start (GtkButton*, gpointer); 
void callback_button_stop (GtkButton*, gpointer);
void callback_button_walk (GtkButton*, gpointer);
void callback_button_continue (GtkButton*, gpointer);
void callback_button_ping (GtkButton*, gpointer);
void callback_start_toggle (GtkToggleButton*, gpointer);
void callback_rtf_toggle (GtkToggleButton*, gpointer);
void callback_refresh_toggle (GtkToggleButton*, gpointer);
void callback_button_reset (GtkButton*, gpointer);
void callback_global_refresh(void);
void callback_memory_refresh(void);
void callback_step_number (GtkEditable*, gpointer);
void callback_memwindow_length (GtkMenuItem*, gpointer);
void callback_memwindow_isa (GtkMenuItem*, gpointer);
void callback_memwindow_listsize (GtkEditable*, gpointer);
void callback_memwindow_no (GtkMenuItem*, gpointer);
void callback_memwindow_ascii_toggle (GtkToggleButton*, gpointer);
void callback_memwindow_dis_toggle (GtkToggleButton*, gpointer);
void callback_memwindow_address (GtkEditable*, gpointer);
void callback_memwindow_hex (GtkEditable*, gpointer);
void callback_memwindow_ascii (GtkEditable*, gpointer);
void callback_memwindow_dis (GtkEditable*, gpointer);
void callback_memwindow_clist (GtkCList*, gint, gint, GdkEventButton*,gpointer); /* called when user selects row from a memory frame */
void callback_memory_window_row_selected(GtkCList*, gint, gint, GdkEventButton*,gpointer); /* called when user selects row from memory window */
void callback_memwindow_scrollmove(GtkAdjustment*,gpointer);
gboolean callback_memwindow_scrollrelease(GtkWidget*,gpointer,gpointer);
void callback_regwindow_hex (GtkEditable*, gpointer);
void callback_regwindow_clist (GtkCList*, gint, gint, GdkEventButton*,gpointer);
void callback_flag_pressed(GtkToggleButton *togglebutton, gpointer user_data);
void callback_flag_pressed_saved(GtkToggleButton *togglebutton, gpointer user_data);
void callback_mode_selection(GtkMenuItem* menuitem, gpointer newmode);
void callback_mode_selection_saved(GtkMenuItem* menuitem, gpointer newmode);
void callback_mode_selection_all(int newmode, int PSR); /* called from the above two to generalise them*/
		
/* the following is not to be used as a callback function, but to set memory window value width */
void callback_set_memory_window_size(GtkMenuItem* menuitem, gpointer newvalue);
gint callback_console_keysnoop  (GtkWidget *grab_widget,GdkEventKey *event,gpointer func_data);
/* deals with events of keypresses */
gint callback_update_comms(gpointer);
gint callback_updateall(gpointer);
gint callback_walk(gpointer);
gint callback_updateregwindow(gpointer);
gint callback_updatememwindow(gpointer);
gint callback_console_update(gpointer data, gint source,
                                 GdkInputCondition condition);
void callback_comms (GtkEditable*, gpointer);                             
void callback_main_quit(gpointer ignore, gpointer ignore2);

/* the following are booleans describing the state of the flags in: */

/* 1) CPSR */

int nflag_current_boolean;
int zflag_current_boolean;
int cflag_current_boolean;
int vflag_current_boolean;

/* 2) SPSR */

int nflag_saved_boolean;
int zflag_saved_boolean;
int cflag_saved_boolean;		
int vflag_saved_boolean;

int compile_communication[2];
FILE *view_console_port;


#endif
