/*		Name:		misc.c
		Version:	1.2.0
		Date:		11/7/2002
		Misc functions mainly hex and binary conversions
*/
#include "interface.h"
#include <glib.h>
#include <ctype.h>
#include "misc.h"
#include "evaluate.c" /* evaluating arithmetic expressions */
#define uchar unsigned char

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Convert char array into hex string     Big Endian                          */
/* Calls "g_snprint" from "glib"                                              */
/* Allocates its own space, to which it returns a pointer                     */

char* view_chararr2hexstrbe(int count, uchar* values)
{
char* ret = g_new(char, count * 2 + 1);                   /* Allocates memory */
char* ptr = ret;

while (count--)
  { 
  g_snprintf(ptr, 3, "%02X", (int) values[count]); /* This also terminates \0 */
  ptr+=2;                                              /* Step string pointer */
  }
return ret;                                             /* Must g_free result */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Convert char array into hex string     Little Endian                       */
/* Calls "g_snprint" from "glib"                                              */
/* Allocates its own space, to which it returns a pointer                     */

char* view_chararr2hexstr(int count, uchar* values)
{
char* ret = g_new(char, count * 2 + 1);                   /* Allocates memory */
char* ptr = ret;

while (count--)
  {
  g_snprintf(ptr, 3, "%02X", (int) *values);
  ptr+=2;
  values++;                                         /* Pointer steps forwards */
  }
return ret;                                             /* Must g_free result */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Convert character array into printable string                              */
/* Allocates memory for return string                                         */

char* view_chararr2asciistr(int count, uchar* values)
{
char* ret   = g_new(char, count + 1);  /* Allocates memory  +1 for terminator */
int   index;

for (index = 0; index < count; index++)                  /* Copy/modify bytes */
  if ((values[index]<' ') || (values[index]>=0x7F)) ret[index] = '.';
  else                                              ret[index] = values[index];

ret[index] = '\0';                            /* Append NUL string terminator */

return ret;                                             /* Must g_free result */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* New routine to service the two following entry points only                 */
/* "acc" may be the same as "byte_string"                                     */

void view_chararrAdd(int count, uchar* byte_string, int number, uchar* acc)
{
int index, temp;

for (index = 0; index < count; index++)
  {
  temp       = byte_string[index] + (number & 0xFF);       /* Add next 8 bits */
  acc[index] = temp & 0xFF;
  number     = number >> 8;                             /* Shift to next byte */
  if (temp >= 0x100) number++;                             /* Propagate carry */
  }
return;
}                                                        /* Must g_free result*/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Add number to byte_string of length count and return result in newly       */
/*  allocated memory                                                          */

uchar* view_chararrAddint(int count, uchar* byte_string, int number)
{
uchar* acc = g_new(uchar, count);       /*bit array + int => bitarray */

view_chararrAdd(count, byte_string, number, acc);
return acc;                                              /* Must g_free result*/
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Add number to byte_string of length count and return result in byte_string */
/* MAY BE REMOVED AFTER REASSIGNING ALL CALLS TO "view_chararrAdd" ABOVE      */

void view_chararrAddintsame(int count, uchar* byte_string, int number)
{
view_chararrAdd(count, byte_string, number, byte_string);
return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Adds two input arrays of count bytes                                       */
/* Result returned in newly allocated array of similar type                   */

uchar* view_chararrAddchararr(int count, uchar* value1, uchar* value2)
{
uchar* ret   = g_new0(uchar, count);
int    index, temp;
int    carry = 0;

for (index = 0; index < count; index++)
  {
  temp       = value1[index] + value2[index] + carry;
  ret[index] = temp & 0xFF;
  carry      = temp >> 8;
  }
return ret;                                             /* Must g_free result */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

int view_char_array_sub_char_arr_to_int(int count, uchar* value1, uchar* value2)
{
int ret = 0;                                  /* bit array - bit array => int */
while (count--) ret = (ret<<8) + (int) value1[count] - (int) value2[count];
return ret;    /* Carry propagated because intermediate -int- can be negative */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

long view_chararrSublong (int count, uchar* value1, long value2)
{
while (count--) value2 -= value1[count]<<(count<<3);
return value2;                                    /* bit array - long => long */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Accumulate count bytes from memory array (little endian) and return as int */

int view_chararr2int(int count, uchar* input)
{
int ret = 0;                                              /* bit array => int */

while (count--) ret = (ret << 8) + (input[count] & 0xFF);
return ret;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Strip count bytes from value and place in memory (little endian)           */
/* Allocates own memory space                                                 */
/* Returns pointer to low end of array                                        */

uchar* view_int2chararr(int count, int value)
{
uchar* ret = g_new(uchar, count);                          /* Allocate memory */
uchar* c = ret;

while (count--)
  {                                                        /*int => bit array */
  *c    = value & 0xFF;
  value = value >> 8;
  c++;
  }

return ret;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Copy count characters from source to destination                           */

void view_chararrCpychararr(int count, uchar* source, uchar* destination)
{
while (count--) destination[count] = source[count];        /* Copy char array */
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

int view_hexstr2chararr2(int count, char* pass, uchar* ret)
{                              /* don't call this! */
int temp = 0;                  /* Hex string to bit array*/
int temp2= 0;
int regbank;
char* text = g_strdup(pass);

while (text[temp])
  {
  while (' ' == text[temp]) temp++;               /* go to beginning of text */
  if (!text[temp]) break;                /* also spot register names in text */
  if (!isxdigit(text[temp]))
    {
    g_free(text);
    regbank=board_main_regbank;
    temp = view_findstrinarr(pass, board_reg_banks[regbank].names,
                                   board_reg_banks[regbank].number);

    if (temp < 0) 
      {
      for (regbank = 0; temp < 0; regbank++)
        {
        if(regbank >= board_num_regbanks) break;
          temp = view_findstrinarr(pass, board_reg_banks[regbank].names,
                                         board_reg_banks[regbank].number);
        }
        regbank--;
      }

    if (temp < 0) 
      {
      temp = view_findstrinsymboltable(pass, symbol_table, symbol_count);
      if (temp < 0) return 0;         
      while (count--) ret[count] = (symbol_table[temp].value>>(count<<3)) & 0xff;
      return 1;
      }
    else  
    if (board_reg_banks[regbank].width)
      view_chararrCpychararr(MIN(count, board_reg_banks[regbank].width),
                             &board_reg_banks[regbank].values
                             [temp * board_reg_banks[regbank].width], ret);
    else 
      {
      if (board_reg_banks[regbank].values[temp])  ret[0] = 1;
      else                                        ret[0] = 0;
      while (count) ret[count--] = 0;
      return 1;
      }

    while (count-->board_reg_banks[regbank].width) ret[count] = 0;
    return 1;
    } // end if (!isxdigit(text[temp]))

  if (text[temp] > '9') text[temp] = text[temp] - 7;
  text[temp2++] = text[temp++];
  } //end while (text[temp])

if (!temp2)
  {
  g_free(text);
  return 0;
  }

temp = 0;
if (temp2 > (count<<1)) temp2 = (count<<1);
while (temp2--)
  {
  ret[temp] = text[temp2] & 15;
  if (!temp2--) 
    {
    temp++;
    break;
    }
  ret[temp++]+= (text[temp2] & 15) << 4;
  }

while (temp < count) ret[temp++] = 0;

g_free(text);

return 1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* 	CRUDE EXPRESSION PARSER - AWAITING REPLACEMENT                        */
/* Returns 1 if okay, 0 if a fault was found                                  */
/*                       array length  input string   pointer to result       */

int view_hexstr2chararr(int length_of_result, char* passed_string, uchar* returned_unsigned_string)
{
unsigned int status;	 /* retains the status of expression evaluation */
int result_integer; 	 /* the result as an integer			*/
int i;			 /* loop index					*/
evaluation_data eval_data;
eval_data.representation=HEXADECIMAL;

status=Evaluate(&passed_string,&result_integer,eval_data); /* evaluate string 	*/
for (i=0;i<(length_of_result);i++)
  returned_unsigned_string[i]=((result_integer>>(8*i)) & 0xFF);
			 /* shift hexadecimal and mask for safety	*/
	
return (!status); /* status equals 0 when successful, view_hexstr2chararr returns 1 when successful */
}		


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Compress any spaces from passed string                                     */

void remove_spaces(char* string)
{
int src, dest = 0;

for (src = 0; string[src] != '\0'; src++)     /* Compress spaces from string */
  if (' ' != string[src]) string[dest++] = string[src];

string[dest] = '\0';                        /* Terminator not copied in loop */

return;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Find string in an array of strings and return index of last numbered match */
/* Whole string must match, case insensitive and all input spaces are ignored */
/* Inputs are the string, a pointer to an array of strings and the array size */
/* Returns -1 if not found                                                    */

int view_findstrinarr(char* string, char** array, int arrcount)
{
char* mystr = g_strdup(string);

remove_spaces(mystr);

while ((arrcount--) && g_strcasecmp(mystr, array[arrcount]));
                                         /* Terminates if/when string matches */
g_free(mystr);

return arrcount;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Find string in a symbol table and return index                             */
/* As above with different record format                                      */

int view_findstrinsymboltable(char* string, symbol* array, int arrcount)
{
char* mystr = g_strdup(string);

remove_spaces(mystr);

while ((arrcount--) && g_strcasecmp(mystr, array[arrcount].name));
                                         /* Terminates if/when string matches */
g_free(mystr);

return arrcount;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Search for a value in a register bank and return the name of the first     */
/* register found.  A '+' is appended if other registers have the same value. */

char* view_chararrinregbank(int count, uchar* value)
{
int regbank;
int regno;
int byteno;
int match;
char* str_ptr = NULL;

for (regbank = board_num_regbanks-1; regbank>=0; regbank--)
  if ((board_reg_banks[regbank].pointer)            /* Real register bank ... */
   && (count == board_reg_banks[regbank].width))   /* ... of appropriate size */
    {
    for (regno = 0; regno < board_reg_banks[regbank].number; regno++)
      {                                              /* For each register ... */
      match = (0==0);                /* Assume match and then prove otherwise */
      for (byteno = board_reg_banks[regbank].width-1;
          (byteno >=0) && match; byteno--)     /* Also terminate on mismatch */
        match = (value[byteno] == board_reg_banks[regbank].values
                 [regno * board_reg_banks[regbank].width + byteno]);
      if (match)
        {
        if (str_ptr == NULL)                          /* First register found */
          str_ptr = g_strdup(board_reg_banks[regbank].names[regno]);
        else
          {
          char* temp;

          temp = str_ptr;                        /* Don't lose old string yet */
          str_ptr = g_strconcat(str_ptr,"+", NULL);/* Concat. into new string */
          g_free(temp);                                /* and free old string */
          return str_ptr;                   /* Bodge ... early get-out clause */
          }
        }
      }
    }

return str_ptr;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Return the specified bit_number of ASCII representation in bit 0           */
/* other bits all zero.                                                       */
/* Assumes input is valid ASCII hex character                                 */
/* REPLACEMENT for next four functions                                        */

int get_Nth_bin_of_char(char hex_character, int bit_number)
{
if (hex_character >= 'A') hex_character = hex_character - 'A' + 10;
return (hex_character>>bit_number) & 1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Convert a nibble to an ASCII character                                     */

char nibble_to_ASCII(int value)
{
value = value & 0X0F;                                 /* Mask here for safety */
if (value < 10) return '0' + value;
else            return 'A' + value - 10;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* the following function returns a byte representing the 4 bits */

/* REPLACEMENT */

char byte_of_bits(int msbit, int bit2, int bit1, int lsbit)
{
return(nibble_to_ASCII(((msbit & 1)<<3)
                      | ((bit2 & 1)<<2)
                      | ((bit1 & 1)<<1)
                      | (lsbit & 1)));
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Given a character, according to the ASCII table, it will return the ASCII  */
/*  value of the top nibble, for instance, for 'A' it will return '4'         */

/* REPLACEMENT */

char get_ascii_of_char_top_nibble(char character)
{
return nibble_to_ASCII(character>>4);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Given a character, according to the ASCII table, it will return the ASCII  */
/*  value of the bottom nibble, for instance, for 'A' it will return '1'      */

/* REPLACEMENT */

char get_ascii_of_char_bottom_nibble(char character)
{
return nibble_to_ASCII(character);
}    

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Return just the least significant byte                                     */
/* Used for masking key_press                                                 */

int mask_one_byte(int hexadecimal_input)
{
return (hexadecimal_input & 0x000000FF);
}		

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
