/*		Name:		view.c
		Version:	1.2.0
		Date:		11/7/2002
		General interface with the graphical front end.
        Most of these functions are created using glade and so comment free
*/



#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include "callbacks.h"
#include "view.h"
#include "interface.h"
#include "misc.h"
#include "serial.h"
#include "chump.h"
#include "breakview.h"
#include "breakcalls.h"
#include "config.h"
#include "pixmaps/tick.xpm"
#include "pixmaps/mulogo.xpm"
#include "pixmaps/chump.xpm"
#include "pixmaps/pkomodotitle.xpm"
#include "pixmaps/pkomodologo.xpm"
#include "pixmaps/pkomodoicon.xpm"
#include "pixmaps/pkomodominiicon.xpm"

int already_built_mem_window_button=FALSE;
int all_flag_windows_have_been_created=FALSE;
int call_number_to_notebook=0;

 
/* will keep count of the number of calls to notebook in the display list */		
		
mem_window_list* view_memwindowlist = NULL;
reg_window_list* view_regwindowlist = NULL;
GdkColor view_greycolour = {1,0xc000,0xc000,0xa000};
GdkColor view_redcolour  = {1,0xFFFF,0x0,0x0};

int spsr_exists=TRUE; /* indicates whether spsr register is currently displayed */

/*              These are ARM.C disassamble instructions replaced by LITHP in ASM.C
 * 
 * char* view_dis_arm(uchar* address, uchar* code) // disassamble an arm instruction
 * {
 *   char* str = g_new0(char, 100);
 *   dis_string_ptr = str;
 *   if(board_cpu_reference_number==1)
 *   	arm_disassemble(view_chararr2int(4,address),view_chararr2int(4,code));
 *   else if(board_cpu_reference_number==2)
 *   	mips_disassemble(str, view_chararr2int(4,code),view_chararr2int(4,address));
 *   return str;
 * }
 *  
 * char* view_dis_thumb(uchar* address, uchar* code) // disassamble an arm instruction
 * {
 *   char* str = g_new0(char, 100);
 *   dis_string_ptr = str;
 *   if(board_cpu_reference_number==1)
 * 	thumb_disassemble(view_chararr2int(4,address),view_chararr2int(2,code));
 *   return str;
 * }
 */

char* view_dis(uchar* address, uchar* code,int count, GList* isa) /* disassamble an instruction count specifies the length*/
{
 Disasm disasm;
 char* ret = g_strdup("");
 char* temp;
 GList* codelist;
 int intaddress = view_chararr2int(board_memory_ptr_width, address);
 codelist = uchararr2glist (count, (char *)code);
 disasm.bitfield= codelist;
// do     {
        disasm = asm_disassemble (disasm.bitfield, isa, intaddress);
        
          if (disasm.distext == &dis_error ) {
                            temp = g_strconcat (ret, "Undefined Instruction", NULL);
                            g_free (ret);
                            ret=temp;
                            disasm.bitfield=NULL;
                             } 
            else                    {
                            
                            ret = asm_sprint(disasm.distext, ret);
                            asm_freestringlist (disasm.distext);
                            }
                            
                            
 //        } while (disasm.bitfield);
         
 g_list_free(codelist);
 return ret;


/*  char* str;            These are MIPS.C and ARM.C disassamble instructions replaced by CHUMP in ASM.C  
  if (8 == count) {
    str = g_new0(char, 150);
	dis_string_ptr = str;
	if(board_cpu_reference_number==1)
		arm_disassemble(view_chararr2int(4,address)  ,view_chararr2int(4,code));
	if(board_cpu_reference_number==2)
  		mips_disassemble(dis_string_ptr, view_chararr2int(4,code),view_chararr2int(4,address));
	while(dis_string_ptr<str+50) (*(dis_string_ptr++))=' ';
	if(board_cpu_reference_number==1)
		arm_disassemble(view_chararr2int(4,address)+4,view_chararr2int(4,code+4));
	if(board_cpu_reference_number==2)
  		mips_disassemble(dis_string_ptr, view_chararr2int(4,code),view_chararr2int(4,address));
	return str;
    }
  if (4 == count) return view_dis_arm  (address, code);
  if (2 == count) return view_dis_thumb(address, code);
  return g_strdup("Unsupported instruction width"); */
}

int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
{
 GtkCList* clist = GTK_CLIST(memwindowptr->clist_ptr);
 int count = memwindowptr->count * memwindowptr->width;
 unsigned char* memdata = g_new(unsigned char, count);
 char* data;
 char* address;
 int temp;
 int flag; /* flags will indicate the type of memory entry to be displayed e.g. one which SPSR points to */
 char* test;
 char* test2;
 GList* breaks=NULL;
 GList* breaksinactive=NULL;
 GList* templist;
 GdkColor* colour;
 uchar* new_address = view_chararrAddchararr(board_memory_ptr_width,
                               memwindowptr->address, memwindowptr->regoffset);
  {
    unsigned int worda;
    unsigned int wordb;
    int temp, temp2;
    uchar* value;
    uchar repc;
    
    board_sendchar(BR_BP_GET);
    if (4 != board_getb32(&worda)) {g_free(new_address);return 0;}    //get breaks and breaksinactive list
    if (4 != board_getb32(&wordb)) {g_free(new_address);return 0;}
    for (temp=0;temp<32;temp++)
        if ((worda>>temp)&1){
 //if (VERBOSE) g_print("foundnumber:%d",temp);
            value=g_new(uchar,board_memory_ptr_width);
            board_sendchar(BR_BP_READ);
            board_sendchar(temp);
            flag=0;
            for (temp2=2;temp2;temp2--){
                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
                if (0xff != repc) flag=1;
                }
            if (board_memory_ptr_width != board_getchararray(board_memory_ptr_width,value)) 
                                               {g_free(new_address);return 0;}
 //if (VERBOSE) g_print("found:%x\n",view_chararr2int(board_memory_ptr_width,value));
            for (temp2=board_memory_ptr_width;temp2;temp2--){
                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
                if (0xff != repc) flag=1;
                }
            for (temp2=16;temp2;temp2--){
                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
                if (0x0 != repc) flag=1;
                }
            if (flag) g_free(value);
            else    if ((wordb>>temp)&1) breaks = g_list_append (breaks, value);
                    else breaksinactive = g_list_append (breaksinactive, value);
            }
  }
 if (board_wordalign) new_address[0] &= -memwindowptr->width;
 if (!board_get_memory (count / memwindowptr->gran, new_address, memdata, memwindowptr->gran)){
   if (!board_mini_ping(0) ||
       !board_get_memory (count / memwindowptr->gran, new_address, memdata, memwindowptr->gran)){       /*get new memory data*/
                              g_free(new_address);g_free(memdata); return 0;}}
 gtk_clist_freeze (clist); 
 count = 0;
 while (count < memwindowptr->count)
  {
   address = view_chararrAddint(board_memory_ptr_width,
                       new_address, count * memwindowptr->width);
   test = view_chararrinregbank(board_memory_ptr_width, address); /* gets reg name from bank */

   data = view_chararr2hexstrbe (board_memory_ptr_width, address);              //create an address string
   if (memwindowptr->count!=LENGTH_OF_MEMORY_WINDOW)
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(memwindowptr->elf_symbols_button))) /* says whether ELF symbols are to be displayed */
        {
           temp=symbol_count;
           while (temp--) /* search symbol table and change data contents if found */
            {       
               if ((!view_chararrSublong(board_memory_ptr_width,address,symbol_table[temp].value)) && (symbol_table[temp].flags & 3) && !(symbol_table[temp].flags & 0x100))
	          {
                   //g_print("%8x %s\n",symbol_table[temp].value, symbol_table[temp].name);
                     g_free(data);
                     data = g_strdup(symbol_table[temp].name);
                  }
            }
         } // end (show_elf_symbols==TRUE)    
  
   if (test) 
        {
          test2 = g_strconcat (data, " " , test, NULL); /* concatenate address and reg. name */
          g_free(data);
          g_free(test);
          data = test2; /* assign it to data */
        }

   flag=0;
   for (temp=0;temp<special_register_count;temp++) /* for all special registers known */
        if (special_registers[temp].active) /* if this register currently exists */
            if (!view_char_array_sub_char_arr_to_int(board_memory_ptr_width, special_registers[temp].value,address)) /* if the address is same as the value of the special reg. */
	       {
                colour = &special_registers[temp].colour; /* set to special colour */
                if(special_registers[temp].pixmap)
		  	    /* args below: clist, row, column, text, spcaing, bitmap, mask */
                     gtk_clist_set_pixtext(clist,count,0,data,2,special_registers[temp].pixmap,special_registers[temp].bitmap);
                else gtk_clist_set_text(clist,count,0,data);
                flag=3;
                }
   
   if (!flag)   /* if not a special register - not the case above */
                {
		data= g_strconcat ( "  ",data,NULL, NULL); /* add spaces to align w.r.t. arrows */
                if (!gtk_clist_get_text(clist,count,0,&test))  /* could not get text */
		 {
		 gtk_clist_set_text(clist,count,0,data);    // only update if it has changed
	         }
                 else
		    if (strcmp(data,test))
		       gtk_clist_set_text(clist,count,0,data);
		}
   
   g_free (data);
   for (temp=0;temp<(memwindowptr->width/memwindowptr->gran);temp++)
     {  /*insert new info into window*/
     data = view_chararr2hexstrbe (memwindowptr->gran, memdata  +
                                    count * memwindowptr->width +
                                    temp  * memwindowptr->gran);
     if (!gtk_clist_get_text(clist,count,temp+1,&test)) gtk_clist_set_text(clist,count,temp+1,data);
     else if (strcmp(data,test)) gtk_clist_set_text(clist,count,temp+1,data);
     g_free (data);
     }
   
   data = view_chararr2asciistr (memwindowptr->width, memdata + count
                               * memwindowptr->width);
   if (!gtk_clist_get_text(clist,count,17,&test)) gtk_clist_set_text(clist,count,17,data);
   else if (strcmp(data,test)) gtk_clist_set_text(clist,count,17,data);
   g_free (data);
   data = view_dis(address, memdata + count * memwindowptr->width,
                         memwindowptr->width, memwindowptr->isa);
   if (!gtk_clist_get_text(clist,count,18,&test)) gtk_clist_set_text(clist,count,18,data);
   else if (strcmp(data,test)) gtk_clist_set_text(clist,count,18,data);
   templist=breaksinactive;
   while(templist)
        {
        if (!view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, templist->data)) flag=1;
        templist=templist->next;
        }
   templist=breaks;
   while(templist)
        {
        if (!view_char_array_sub_char_arr_to_int(board_memory_ptr_width, address, templist->data)) flag=2;
         templist=templist->next;
        }
   switch (flag)
        {
        case (0):
            gtk_clist_set_foreground (clist,count ,&gtk_widget_get_default_style()->text[GTK_STATE_ACTIVE]);
            break;
        case (1):
            gtk_clist_set_foreground (clist,count, &view_greycolour);
            break;
        case (2):
            gtk_clist_set_foreground (clist,count, &view_redcolour);
            break;
        case (3):
            gtk_clist_set_foreground (clist,count, colour);
            break;
        }
    

   g_free (data);
   g_free (address);
   count++;
   } //end while
 templist=breaksinactive;
 while(templist)
        {
        g_free(templist->data);
        templist=templist->next;
        }
 templist=breaks;
 while(templist)
        {
        g_free(templist->data);
        templist=templist->next;
        }
  gtk_clist_thaw (clist);
  return 1;
}


int view_updateregwindow(reg_window* regwindowptr) /* refresh register window*/
{
 GtkCList* clist = GTK_CLIST(regwindowptr->clist_ptr);
 int count;
 char* data[2];
 char* test;
 char *cpsr_in_current;
 char *spsr_in_current;
 char *element[]={"SPSR","00000000"};
 reg_bank* regbank = &board_reg_banks[regwindowptr->regbank_no];
 /* loads the current register bank which is being refreshed */ 
 reg_bank* regbankincurrent = &board_reg_banks[0]; /* loads the 'current' reg. bank */
 if (!board_get_regbank(regwindowptr->regbank_no)){
   if (!board_mini_ping(0) || !board_get_regbank(regwindowptr->regbank_no))
                                                                return 0;}
 gtk_clist_freeze (clist);
 count = 0;
 while (count < regbank->number)
   { 
   data[0] = g_strdup(regbank->names[count]);  
   if (regbank->width) data[1] = view_chararr2hexstrbe (regbank->width,
                               &regbank->values[count * regbank->width]);
   // retrieving values of registers and setting them to be hexadecimal.
   else 
      {
        if (regbank->values[count]) data[1]="1"; 
	else data[1]="0";
      }

   if (!gtk_clist_get_text(clist,count,0,&test)) gtk_clist_set_text(clist,count,0,data[0]);
   else if (strcmp(data[0],test)) gtk_clist_set_text(clist,count,0,data[0]);
   
   if (!gtk_clist_get_text(clist,count,1,&test)) gtk_clist_set_text(clist,count,1,data[1]);
   else if (strcmp(data[1],test))  gtk_clist_set_text(clist,count,1,data[1]); 
   if (regwindowptr->regbank_no==0) //if 'current' reg. bank
      cpsr_in_current= view_chararr2hexstrbe (regbank->width,
                               &regbank->values[16 * regbank->width]);
   /* get value of CPSR in 'current' */
   
      spsr_in_current= view_chararr2hexstrbe (regbankincurrent->width,
                               &regbankincurrent->values[17 * regbankincurrent->width]);
   /* get value of SPSR in 'current' */
   g_free(data[0]);
   if (regbank->width) g_free(data[1]);
   count++;    
   }
   if (strcmp(board_cpu_name,"ARM")==0) /* the following only applies to ARM */
    if ( regwindowptr->regbank_no==0)
    { 
     if		  
      (cpsr_in_current[7]=='0'  || cpsr_in_current[7]=='F')
         /* if we deal with 'current' and the value of cpsr is as above */
       {
         gtk_clist_remove(clist,17);
	 spsr_exists=FALSE; /* unset flag as expected */
    /* if the value of the 4 bottom bits in cpsr in 'current' is 0xf or 0x0 hide SPSR */			
       }
   else /* return SPSR to 'current' reg. bank */
       {
        if (spsr_exists==FALSE) /* we don't want to add it more than once */
	  {
           /* add spsr */
	    element[1]=view_chararr2hexstrbe (regbank->width,
                                &regbank->values[17 * regbank->width]);
	     gtk_clist_insert(clist,17,element);   			    	
	    spsr_exists=TRUE; /* indicate that spsr is now displayed */
	  }  
       } 
     } // end if   
     
     
     
  if (all_flag_windows_have_been_created) /* we only want to refresh those after the widgets
  					have been created */
 /* the following part will ensure that the user is informed of the current mode as it is
    reported by the CPSR */
  {  
    switch (cpsr_in_current[7]) /* inspect bottom 4 bits */
    {
     case '0': /* this would mean we are in user */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"User");	 /* so set the label */   
	break;
     case '1': /* this would mean we are in fiq */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"FIQ"); /* so set the label */   
	break;
     case '2': /* this would mean we are in irq */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"IRQ"); /* so set the label */   
	break;
     case '3': /* this would mean we are in svc */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"Supervisor"); /* so set the label */   
	break;
     case '7': /* this would mean we are in abort */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"Abort"); /* so set the label */   
	break;
     case 'B': /* this would mean we are in undef. */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"Undefined"); /* so set the label */   
	break;
     case 'F': /* this would mean we are in sys. */
    	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"System"); /* so set the label */   
	break;
     default: /* no mode */
	gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"No Mode"); /* so set the label */   
	break;	
     }
     
      
    switch (spsr_in_current[7]) /* inspect bottom 4 bits */
    {
     case '0': /* this would mean we are in user */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"User"); /* so set the label */   
	break;
     case '1': /* this would mean we are in fiq */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"FIQ"); /* so set the label */   
	break;
     case '2': /* this would mean we are in irq */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"IRQ"); /* so set the label */   
	break;
     case '3': /* this would mean we are in svc */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"Supervisor"); /* so set the label */   
	break;
     case '7': /* this would mean we are in abort */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"Abort"); /* so set the label */   
	break;
     case 'B': /* this would mean we are in undef. */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"Undefined"); /* so set the label */   
	break;
     case 'F': /* this would mean we are in sys. */
    	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"System"); /* so set the label */   
	break;
     default: /* no mode */
	gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"No Mode"); /* so set the label */   
	break;	
     } 
     
     /* the following will set the flag according to the cpsr and spsr */

     if (get_Nth_bin_of_char(cpsr_in_current[0],3))              // n in current
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (nflag_current), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (nflag_current), FALSE);
     
     if (get_Nth_bin_of_char(spsr_in_current[0],3))            // n in saved
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (nflag_saved), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (nflag_saved), FALSE);

     
     
     
     
      if (get_Nth_bin_of_char(cpsr_in_current[0],2))              // z in current
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (zflag_current), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (zflag_current), FALSE);
     
     if (get_Nth_bin_of_char(spsr_in_current[0],2))            // z in saved
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (zflag_saved), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (zflag_saved), FALSE);
    
     
     
     
     
     
      if (get_Nth_bin_of_char(cpsr_in_current[0],1))              // c in current
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cflag_current), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cflag_current), FALSE);
     
     if (get_Nth_bin_of_char(spsr_in_current[0],1))            // c in saved
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cflag_saved), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (cflag_saved), FALSE);
     
     
     
      if (get_Nth_bin_of_char(cpsr_in_current[0],0))              // v in current
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (vflag_current), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (vflag_current), FALSE);
     
     if (get_Nth_bin_of_char(spsr_in_current[0],0))            // v in saved
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (vflag_saved), TRUE);  
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (vflag_saved), FALSE);
     
   } // end if 

 gtk_clist_thaw (clist); 
 return 1;
}

mem_window* view_getmemwindowptr(GtkWidget* widget) /*climb up the tree to got the parent who is a mem_window*/
{
  mem_window *mem_window_ptr = NULL;
  while (NULL == mem_window_ptr)
   {
     mem_window_ptr = gtk_object_get_data (GTK_OBJECT(widget),"mem_window");
     widget = (GTK_WIDGET(widget))->parent;
   }
  return mem_window_ptr;
}
		
reg_window* view_getregwindowptr(GtkWidget* widget) /*climb up the tree to got the parent who is a reg_window*/
{
  reg_window *reg_window_ptr = NULL;
  while (NULL == reg_window_ptr)
   {
   reg_window_ptr = gtk_object_get_data (GTK_OBJECT(widget),"reg_window");
   widget = (GTK_WIDGET(widget))->parent;
   }
  return reg_window_ptr;  
}

GtkWidget* view_create_h_mov_split (void) /*standard make a horisontal movable split*/
{
  GtkWidget* view_mov_h_split = gtk_hpaned_new ();
  gtk_widget_show (view_mov_h_split);
  gtk_paned_set_gutter_size (GTK_PANED (view_mov_h_split), 10);
  return view_mov_h_split;
}

GtkWidget* view_create_v_mov_split (void) /*standard make a vertical movable split*/
{
  GtkWidget* view_mov_v_split = gtk_vpaned_new ();
  gtk_widget_show (view_mov_v_split);
  gtk_paned_set_gutter_size (GTK_PANED (view_mov_v_split), 10);
  return view_mov_v_split;
}


void view_create_mainwindow (void) /*create main window*/
{
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *sepp;
  GtkWidget *window_body;
  GdkGeometry hints;

  
  view_tooltips = gtk_tooltips_new ();
  gdk_colormap_alloc_color(gdk_colormap_get_system(),&view_greycolour,0,1);
  gdk_colormap_alloc_color(gdk_colormap_get_system(),&view_redcolour,0,1);
  view_tick_pixmap =   gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_tick_bitmap,  NULL,tick_xpm);
  view_mulogo_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_mulogo_bitmap,NULL,mulogo_xpm);
  view_komodotitle_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_komodotitle_bitmap,NULL,komodotitle_xpm);
  view_komodoicon_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_komodoicon_bitmap,NULL,komodoicon_xpm);
  view_chump_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_chump_bitmap,NULL,chump_xpm);
  view_komodominiicon_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_komodominiicon_bitmap,NULL,komodominiicon_xpm);

  /*hints.base_width = 990;
  hints.base_height =  700;

  hints.max_height =  700;
  hints.max_width = 990;
  hints.min_width = 990;
  hints.min_height = 700;
	  
*/	  
  view_mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (view_mainwindow),PACKAGE " " VERSION " " DATE " (C) Manchester University");
  gtk_container_set_border_width (GTK_CONTAINER (view_mainwindow), 2);
  gtk_window_set_default_size (GTK_WINDOW (view_mainwindow), 990, 500);
  gtk_window_set_position(GTK_WINDOW (view_mainwindow),GTK_WIN_POS_CENTER);
  /*gtk_window_set_geometry_hints(GTK_WINDOW (view_mainwindow),
		                GTK_WIDGET (view_mainwindow),
		  		&hints,
		  		GDK_HINT_MAX_SIZE|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
		  
*/		  		
		  
		  
		  
  gtk_signal_connect (GTK_OBJECT (view_mainwindow), "destroy",   /*if closed then quit*/
                             GTK_SIGNAL_FUNC(callback_main_quit),
                             NULL);
  gtk_window_set_policy (GTK_WINDOW (view_mainwindow), FALSE, TRUE, FALSE);
  vbox = gtk_vbox_new (FALSE, 3);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  hbox = gtk_hbox_new (FALSE, 3);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);
  gtk_container_set_border_width (GTK_CONTAINER (hbox),3);
  gtk_container_add (GTK_CONTAINER (view_mainwindow), vbox);
  gtk_box_pack_start (GTK_BOX (hbox), view_create_prog_ctrl (), TRUE, TRUE, 0); /* add a program control bar*/
  sepp = gtk_vseparator_new();
  gtk_widget_ref (sepp);
  gtk_widget_show (sepp);
  gtk_box_pack_start (GTK_BOX (hbox), sepp, FALSE, FALSE, 0); 
  gtk_box_pack_start (GTK_BOX (hbox), view_create_filebar(), TRUE, TRUE, 0); /* add a file bar*/
  gtk_box_pack_start (GTK_BOX (vbox), view_create_menubar(), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  window_body=view_parse_list();
  gtk_widget_set_usize(window_body,600,560);
  gtk_box_pack_start (GTK_BOX (vbox),window_body , FALSE, FALSE, 0); 
  gtk_box_pack_start (GTK_BOX (vbox), create_status_bar(), FALSE, FALSE, 0);   
  view_maincontainer=vbox;
}



GtkWidget* view_create_menubar(void)
{
GtkWidget *menu_bar;
GtkWidget *file_bar;
GtkWidget *actions_bar;
GtkWidget *special_actions_bar;
GtkWidget *mode_bar;
GtkWidget *flags_bar;
GtkWidget *help_bar; 
GtkWidget *hbox;
GtkWidget *file_submenu;
GtkWidget *special_submenu;
GtkWidget *actions_submenu;
GtkWidget *mode_submenu;
GtkWidget *help_submenu;
GtkWidget *walk_submenu;
GtkWidget *multi_step_submenu;
GtkWidget *flags_submenu;
GtkWidget *compile;
//GtkWidget *browse_compile;
GtkWidget *load;
//GtkWidget *browse_load;
GtkWidget *quit;
GtkWidget *features;
GtkWidget *simple_breakpoints;
GtkWidget *breakpoints;
GtkWidget *simple_watchpoints;
GtkWidget *watchpoints;
GtkWidget *run;
GtkWidget *stop;
GtkWidget *single_step;
GtkWidget *multi_step;
GtkWidget *walk;
GtkWidget *continue_running;
GtkWidget *reset;
GtkWidget *ping;
GtkWidget *about;
GtkWidget *walk_10;
GtkWidget *walk_100;
GtkWidget *walk_1000;
GtkWidget *walk_as_below;
GtkWidget *multi_step_10;
GtkWidget *multi_step_100;
GtkWidget *multi_step_1000;
GtkWidget *multi_step_10000;
GtkWidget *multi_step_100000;
GtkWidget *multi_step_1000000;
GtkWidget *multi_step_as_below;
GtkWidget *current;
GtkWidget *user;
GtkWidget *supervisor;
GtkWidget *abort;
GtkWidget *undefined;
GtkWidget *irq;
GtkWidget *fiq;
GtkWidget *saved_flags;
GtkWidget *current_flags;
GtkWidget *window_memory; /* a widget to hold a memory window */
GtkWidget *memory_window_button; /* the button that activates the above */


/* definitions of subsubmenus - walk and multistep at current date*/

const static int zero = 0;
const static int one = 1;
const static int ten = 10;
const static int hundred = 100;
const static int thousand = 1000;
const static int ten_thousand = 10000;
const static int hundred_thousand = 100000;
const static int million = 1000000;


	/* walk submenu */
walk_submenu=gtk_menu_new();

walk_as_below=gtk_menu_item_new_with_label(_("1 step at a time       F9"));
gtk_widget_show (walk_as_below);
gtk_menu_append(GTK_MENU(walk_submenu),walk_as_below);
gtk_signal_connect (GTK_OBJECT (walk_as_below), "activate",
                      GTK_SIGNAL_FUNC (callback_button_walk),(gpointer)&view_step_number);

walk_10=gtk_menu_item_new_with_label(_("10 steps at a time"));
gtk_widget_show (walk_10);
gtk_menu_append(GTK_MENU(walk_submenu),walk_10);
gtk_signal_connect (GTK_OBJECT (walk_10), "activate",
                      GTK_SIGNAL_FUNC (callback_button_walk),(gpointer)&ten);

walk_100=gtk_menu_item_new_with_label(_("100 steps at a time"));
gtk_widget_show (walk_100);
gtk_menu_append(GTK_MENU(walk_submenu),walk_100);
gtk_signal_connect (GTK_OBJECT (walk_100), "activate",
                      GTK_SIGNAL_FUNC (callback_button_walk),(gpointer)&hundred);

walk_1000=gtk_menu_item_new_with_label(_("1000 steps at a time"));
gtk_widget_show (walk_1000);
gtk_menu_append(GTK_MENU(walk_submenu),walk_1000);
gtk_signal_connect (GTK_OBJECT (walk_1000), "activate",
                      GTK_SIGNAL_FUNC (callback_button_walk),(gpointer)&thousand);

	/* multi-step submenu */

multi_step_submenu=gtk_menu_new();

multi_step_as_below=gtk_menu_item_new_with_label(_("Steps as in box below      F8"));
gtk_widget_show (multi_step_as_below);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_as_below);
gtk_signal_connect (GTK_OBJECT (multi_step_as_below), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&view_step_number);

multi_step_10=gtk_menu_item_new_with_label(_("10 steps"));
gtk_widget_show (multi_step_10);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_10);
gtk_signal_connect (GTK_OBJECT (multi_step_10), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&ten);

multi_step_100=gtk_menu_item_new_with_label(_("100 steps"));
gtk_widget_show (multi_step_100);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_100);
gtk_signal_connect (GTK_OBJECT (multi_step_100), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&hundred);

multi_step_1000=gtk_menu_item_new_with_label(_("1000 steps"));
gtk_widget_show (multi_step_1000);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_1000);
gtk_signal_connect (GTK_OBJECT (multi_step_1000), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&thousand);

multi_step_10000=gtk_menu_item_new_with_label(_("10000 steps"));
gtk_widget_show (multi_step_10000);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_10000);
gtk_signal_connect (GTK_OBJECT (multi_step_10000), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&ten_thousand);

multi_step_100000=gtk_menu_item_new_with_label(_("100000 steps"));
gtk_widget_show (multi_step_100000);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_100000);
gtk_signal_connect (GTK_OBJECT (multi_step_100000), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&hundred_thousand);

multi_step_1000000=gtk_menu_item_new_with_label(_("1000000 steps"));
gtk_widget_show (multi_step_1000000);
gtk_menu_append(GTK_MENU(multi_step_submenu),multi_step_1000000);
gtk_signal_connect (GTK_OBJECT (multi_step_1000000), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&million);






/* definitions of submenus */


	/* file submenu */


file_submenu=gtk_menu_new();

//browse_load=gtk_menu_item_new_with_label(_("Browse file to load..."));
//gtk_widget_show (browse_load);
//gtk_menu_append(GTK_MENU(file_submenu),browse_load);

load=gtk_menu_item_new_with_label(_("Load file          "));
gtk_widget_show (load);
gtk_menu_append(GTK_MENU(file_submenu),load);
gtk_signal_connect (GTK_OBJECT (load), "activate",
                      GTK_SIGNAL_FUNC (callback_button_load),centry_load);

//browse_compile=gtk_menu_item_new_with_label(_("Browse file to compile...              "));
//gtk_widget_show (browse_compile);
//gtk_menu_append(GTK_MENU(file_submenu),browse_compile);

compile=gtk_menu_item_new_with_label(_("Compile file"));
gtk_widget_show (compile);
gtk_menu_append(GTK_MENU(file_submenu),compile);
gtk_signal_connect (GTK_OBJECT (compile), "activate",
                      GTK_SIGNAL_FUNC (callback_button_compile),centry_compile);


quit=gtk_menu_item_new_with_label(_("Quit Program"));
gtk_widget_show (quit);
gtk_menu_append(GTK_MENU(file_submenu),quit);
gtk_signal_connect (GTK_OBJECT(quit), "activate",
                   GTK_SIGNAL_FUNC (callback_main_quit),NULL);

	/* special submenu */

special_submenu=gtk_menu_new();

breakpoint_mask=0;
view_breakwindow[0][0] = create_breakwindow();
gtk_widget_ref (view_breakwindow[0][0]);

breakpoints=gtk_menu_item_new_with_label(_("Breakpoints"));
gtk_widget_show (breakpoints);
gtk_menu_append(GTK_MENU(special_submenu),breakpoints);
gtk_signal_connect_object(GTK_OBJECT (breakpoints), "activate", gtk_widget_show ,GTK_OBJECT (view_breakwindow[0][0]));
 
   {
    int temp;
    char *startdata[] = {"","Address A","<X<","Address B","Data A","<X<","Data B","other"};
    GtkWidget* clist = gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][0]),"clist");
    for (temp=0;temp < 32;temp++) gtk_clist_append(GTK_CLIST(clist),startdata);
    gtk_widget_set_style(clist,fixed_style);
   }
   
breakpoint_line[0][0]=0;
breakpoint_select( GTK_CLIST(gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][0]),"clist")),breakpoint_line[0][0],0,NULL,NULL);

breakpoint_mask=0x100000;

view_breakwindow[1][0] = create_simplebreakwindow();
gtk_widget_ref (view_breakwindow[1][0]);

simple_breakpoints=gtk_menu_item_new_with_label(_("Simple Breakpoints       "));
gtk_widget_show (simple_breakpoints);
gtk_menu_append(GTK_MENU(special_submenu),simple_breakpoints);
gtk_signal_connect_object(GTK_OBJECT (simple_breakpoints), "activate", gtk_widget_show ,GTK_OBJECT (view_breakwindow[1][0]));   

   {
    int temp;
    char *startdata[] = {"","Address"};
    GtkWidget* clist = gtk_object_get_data (GTK_OBJECT(view_breakwindow[1][0]),"clist");
    for (temp=0;temp < 32;temp++) gtk_clist_append(GTK_CLIST(clist),startdata);
    gtk_widget_set_style(clist,fixed_style);
   }
   
breakpoint_line[1][0]=0;
breakpoint_refresh (0);
breakpoint_select( GTK_CLIST(gtk_object_get_data (GTK_OBJECT(view_breakwindow[1][0]),"clist")),breakpoint_line[1][0],0,NULL,NULL);

breakpoint_mask=0x10000;

view_breakwindow[0][1] = create_breakwindow();
gtk_window_set_title (GTK_WINDOW(view_breakwindow[0][1]),"Watchpoints");

watchpoints=gtk_menu_item_new_with_label(_("Watchpoints"));
gtk_widget_show (watchpoints);
gtk_menu_append(GTK_MENU(special_submenu),watchpoints);
gtk_signal_connect_object(GTK_OBJECT (watchpoints), "activate", gtk_widget_show ,GTK_OBJECT (view_breakwindow[0][1]));
gtk_signal_connect(GTK_OBJECT(GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][1]),"optionaddress"))))), "selection-done",GTK_SIGNAL_FUNC(breakpoint_toggled),GINT_TO_POINTER(breakpoint_mask|0x4));
gtk_signal_connect(GTK_OBJECT(GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][1]),"optiondata"))))), "selection-done",GTK_SIGNAL_FUNC(breakpoint_toggled),GINT_TO_POINTER(breakpoint_mask|0x1));

   {
    int temp;
    char *startdata[] = {"","Address A","<X<","Address B","Data A","<X<","Data B","other"};
    GtkWidget* clist = gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][1]),"clist");
    for (temp=0;temp < 32;temp++) gtk_clist_append(GTK_CLIST(clist),startdata);
    gtk_widget_set_style(clist,fixed_style);
   }
breakpoint_line[0][1]=0;
breakpoint_select( GTK_CLIST(gtk_object_get_data (GTK_OBJECT(view_breakwindow[0][1]),"clist")),breakpoint_line[0][1],0,NULL,NULL);
  
breakpoint_mask=0x110000;  
  
view_breakwindow[1][1] = create_simplebreakwindow();
gtk_widget_ref (view_breakwindow[1][1]);
gtk_window_set_title (GTK_WINDOW(view_breakwindow[0][1]),"Simple-Watchpoints");
    
simple_watchpoints=gtk_menu_item_new_with_label(_("Simple Watchpoints"));
gtk_widget_show (simple_watchpoints);
gtk_menu_append(GTK_MENU(special_submenu),simple_watchpoints);
gtk_signal_connect_object(GTK_OBJECT (simple_watchpoints), "activate", gtk_widget_show ,GTK_OBJECT (view_breakwindow[1][1]));

   {
    int temp;
    char *startdata[] = {"","Address"};
    GtkWidget* clist = gtk_object_get_data (GTK_OBJECT(view_breakwindow[1][1]),"clist");
    for (temp=0;temp < 32;temp++) gtk_clist_append(GTK_CLIST(clist),startdata);
    gtk_widget_set_style(clist,fixed_style);
   }
breakpoint_line[1][1]=0;
breakpoint_refresh (1);
breakpoint_select( GTK_CLIST(gtk_object_get_data (GTK_OBJECT(view_breakwindow[1][0]),"clist")),breakpoint_line[1][1],0,NULL,NULL);

memory_window_button=gtk_menu_item_new_with_label(_("Memory Window"));
gtk_widget_show (memory_window_button);
gtk_menu_append(GTK_MENU(special_submenu),memory_window_button);	
window_memory = create_memorywindow();
gtk_signal_connect_object(GTK_OBJECT (memory_window_button), "activate", gtk_widget_show ,GTK_OBJECT (window_memory));  

features=gtk_menu_item_new_with_label(_("Features"));
gtk_widget_show (features);
gtk_menu_append(GTK_MENU(special_submenu),features);
gtk_signal_connect_object(GTK_OBJECT (features), "activate", gtk_widget_show ,GTK_OBJECT (window_feature));


/* actions submenu */

actions_submenu=gtk_menu_new();

reset=gtk_menu_item_new_with_label(_("Reset"));
gtk_widget_show (reset);
gtk_menu_append(GTK_MENU(actions_submenu),reset);
gtk_signal_connect (GTK_OBJECT (reset), "activate",
                      GTK_SIGNAL_FUNC (callback_button_reset),NULL);

ping=gtk_menu_item_new_with_label(_("Ping"));
gtk_widget_show (ping);
gtk_menu_append(GTK_MENU(actions_submenu),ping);
gtk_signal_connect (GTK_OBJECT (ping), "activate",
                      GTK_SIGNAL_FUNC (callback_button_ping),NULL);
 
run=gtk_menu_item_new_with_label(_("Run                    F5"));
gtk_widget_show (run);
gtk_menu_append(GTK_MENU(actions_submenu),run);
gtk_signal_connect (GTK_OBJECT (run), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&zero);

stop=gtk_menu_item_new_with_label(_("Stop                   F6"));
gtk_widget_show (stop);
gtk_menu_append(GTK_MENU(actions_submenu),stop);
gtk_signal_connect (GTK_OBJECT (stop), "activate",
                      GTK_SIGNAL_FUNC (callback_button_stop),NULL);

continue_running=gtk_menu_item_new_with_label(_("Continue           F10"));
gtk_widget_show (continue_running);
gtk_menu_append(GTK_MENU(actions_submenu),continue_running);
gtk_signal_connect (GTK_OBJECT (continue_running), "activate",
                      GTK_SIGNAL_FUNC (callback_button_continue),NULL);

single_step=gtk_menu_item_new_with_label(_("Single-Step        F7"));
gtk_widget_show (single_step);
gtk_menu_append(GTK_MENU(actions_submenu),single_step);
gtk_signal_connect (GTK_OBJECT (single_step), "activate",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&one);

multi_step=gtk_menu_item_new_with_label(_("Multi-Step"));
gtk_menu_item_configure(GTK_MENU_ITEM(multi_step),0,TRUE); /* show arrow */
gtk_widget_show (multi_step);
gtk_menu_append(GTK_MENU(actions_submenu),multi_step);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(multi_step),multi_step_submenu);

walk=gtk_menu_item_new_with_label(_("Walk")); 
gtk_menu_item_configure(GTK_MENU_ITEM(walk),0,TRUE); /* show arrow */
gtk_widget_show (walk);
gtk_menu_append(GTK_MENU(actions_submenu),walk);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(walk),walk_submenu);


	/* mode submenu */
mode_submenu=gtk_menu_new();

current=gtk_menu_item_new_with_label(_("Current             "));
gtk_widget_show (current);
gtk_menu_append(GTK_MENU(mode_submenu),current);
gtk_signal_connect (GTK_OBJECT (current), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)0);

user=gtk_menu_item_new_with_label(_("User/System"));
gtk_widget_show (user);
gtk_menu_append(GTK_MENU(mode_submenu),user);
gtk_signal_connect (GTK_OBJECT (user), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)1);

supervisor=gtk_menu_item_new_with_label(_("Supervisor"));
gtk_widget_show (supervisor);
gtk_menu_append(GTK_MENU(mode_submenu),supervisor);
gtk_signal_connect (GTK_OBJECT (supervisor), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)2);

abort=gtk_menu_item_new_with_label(_("Abort"));
gtk_widget_show (abort);
gtk_menu_append(GTK_MENU(mode_submenu),abort);
gtk_signal_connect (GTK_OBJECT (abort), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)3);

undefined=gtk_menu_item_new_with_label(_("Undefined"));
gtk_widget_show (undefined);
gtk_menu_append(GTK_MENU(mode_submenu),undefined);
gtk_signal_connect (GTK_OBJECT (undefined), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)4);

irq=gtk_menu_item_new_with_label(_("IRQ            "));
gtk_widget_show (irq);
gtk_menu_append(GTK_MENU(mode_submenu),irq);
gtk_signal_connect (GTK_OBJECT (irq), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)5);

fiq=gtk_menu_item_new_with_label(_("FIQ"));
gtk_widget_show (fiq);
gtk_menu_append(GTK_MENU(mode_submenu),fiq);
gtk_signal_connect (GTK_OBJECT (fiq), "activate",
                      GTK_SIGNAL_FUNC (callback_mode_selected_from_menu), (gpointer)6);

	/* flags submenu */

flags_submenu=gtk_menu_new();

current_flags=gtk_menu_item_new_with_label(_("Current Flags             "));
gtk_widget_show (current_flags);
gtk_menu_append(GTK_MENU(flags_submenu),current_flags);
gtk_signal_connect (GTK_OBJECT (current_flags), "activate",
                      GTK_SIGNAL_FUNC (callback_flags_selected_from_menu), (gpointer)0);

saved_flags=gtk_menu_item_new_with_label(_("Saved Flags"));
gtk_widget_show(saved_flags);
gtk_menu_append(GTK_MENU(flags_submenu),saved_flags);
gtk_signal_connect (GTK_OBJECT (saved_flags), "activate",
                      GTK_SIGNAL_FUNC (callback_flags_selected_from_menu), (gpointer)1);


	/* actions submenu */

help_submenu=gtk_menu_new();

about=gtk_menu_item_new_with_label(_("About              "));
gtk_widget_show (about);
gtk_menu_append(GTK_MENU(help_submenu),about);
gtk_signal_connect_object(GTK_OBJECT (about), "activate", gtk_widget_show ,GTK_OBJECT (window_about));









hbox=gtk_hbox_new(FALSE,2);	/* the container of the menu entries themselves */
gtk_widget_ref (hbox);
gtk_widget_show (hbox); 	 
		 		 	 
menu_bar=gtk_menu_bar_new();
gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menu_bar),GTK_SHADOW_ETCHED_IN);
gtk_widget_ref (menu_bar);
gtk_widget_show (menu_bar);

file_bar=gtk_menu_item_new_with_label(_("File"));
gtk_widget_show (file_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), file_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_bar),file_submenu);

actions_bar=gtk_menu_item_new_with_label(_("Actions"));
gtk_widget_show (actions_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), actions_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(actions_bar),actions_submenu);

special_actions_bar=gtk_menu_item_new_with_label(_("Special"));
gtk_widget_show (special_actions_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), special_actions_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(special_actions_bar),special_submenu);

mode_bar=gtk_menu_item_new_with_label(_("Mode"));
gtk_widget_show (mode_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), mode_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(mode_bar),mode_submenu);

flags_bar=gtk_menu_item_new_with_label(_("Flags"));
gtk_widget_show (flags_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), flags_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(flags_bar),flags_submenu);


help_bar=gtk_menu_item_new_with_label(_("Help"));
gtk_widget_show (help_bar);
gtk_menu_bar_append (GTK_MENU_BAR(menu_bar), help_bar);
gtk_menu_item_set_submenu(GTK_MENU_ITEM(help_bar),help_submenu);

help_bar=gtk_pixmap_new(view_komodominiicon_pixmap,view_komodominiicon_bitmap);
gtk_widget_show (help_bar);
gtk_widget_ref (help_bar);

gtk_box_pack_start (GTK_BOX (hbox),menu_bar, FALSE, FALSE, 0);
gtk_box_set_spacing(GTK_BOX (hbox),670);
gtk_box_pack_start (GTK_BOX (hbox),help_bar, FALSE, FALSE, 0);

return hbox;
}		
		
GtkWidget* view_create_filebar (void)
{
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *button_compile;
  GtkWidget *combo_compile;
  GtkWidget *button_findcompile;
  GtkWidget *button_load;
  GtkWidget *combo_load;
  GtkWidget *button_findload;
  GtkWidget *fileselection;
  GtkWidget *ok_button;
  GtkWidget *cancel_button;
  // GtkWidget *button_about;
  GtkWidget *dialog_vbox1;
  GtkWidget *label;
  GtkWidget *dialog_action_area1;
  GtkWidget *buttonok;
  GtkWidget *buttoncancel;
  

  window_about = create_aboutwindow(); /* create an about window and set window_about to point to it */
  /* declared in view.h */
  
  vbox = gtk_vbox_new (FALSE, 2);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);

  hbox = gtk_hbox_new (FALSE, 4);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);

  button_compile = gtk_button_new_with_label (_("Compile ->"));
  gtk_widget_ref (button_compile);
  gtk_widget_show (button_compile);
  gtk_box_pack_start (GTK_BOX (hbox), button_compile, FALSE, FALSE, 4);
  combo_compile = gtk_combo_new ();
  gtk_widget_ref (combo_compile);
  gtk_widget_show (combo_compile);
  gtk_box_pack_start (GTK_BOX (hbox), combo_compile, TRUE, TRUE, 0);
  centry_compile = GTK_COMBO (combo_compile)->entry;
  gtk_widget_ref (centry_compile);  gtk_widget_show (centry_compile);
  button_findcompile = gtk_button_new_with_label (_("Browse..."));
  gtk_widget_ref (button_findcompile); 
  gtk_widget_show (button_findcompile);
  gtk_box_pack_start (GTK_BOX (hbox), button_findcompile, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (combo_compile), "list", NULL);
  gtk_signal_connect (GTK_OBJECT (button_compile), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_compile),centry_compile);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);

  button_load = gtk_button_new_with_label (_("Load ->"));
  gtk_widget_ref (button_load);
  gtk_widget_show (button_load);
  gtk_box_pack_start (GTK_BOX (hbox), button_load, FALSE, FALSE, 4);

  combo_load = gtk_combo_new ();
  gtk_widget_ref (combo_load);
  gtk_widget_show (combo_load);
  gtk_box_pack_start (GTK_BOX (hbox), combo_load, TRUE, TRUE, 0);
  centry_load = GTK_COMBO (combo_load)->entry;
  gtk_widget_ref (centry_load);
  gtk_widget_show (centry_load);
  button_findload = gtk_button_new_with_label (_("Browse..."));
  gtk_widget_ref (button_findload);  gtk_widget_show (button_findload);
  gtk_box_pack_start (GTK_BOX (hbox), button_findload, FALSE, FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (combo_load), "list", NULL);
  gtk_signal_connect (GTK_OBJECT (button_load), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_load),centry_load);
  

  view_fileerror = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (view_fileerror), "view_fileerror", view_fileerror);
  gtk_window_set_title (GTK_WINDOW (view_fileerror), _("File load error"));

  
#ifdef GTK2
  gtk_window_set_type_hint (GTK_WINDOW(view_fileerror),GDK_WINDOW_TYPE_HINT_DIALOG);
#endif

#ifndef GTK2
  GTK_WINDOW (view_fileerror)->type = GTK_WINDOW_DIALOG;
#endif
  

  gtk_window_set_position (GTK_WINDOW (view_fileerror), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (view_fileerror), FALSE, TRUE, TRUE);

  dialog_vbox1 = GTK_DIALOG (view_fileerror)->vbox;
  gtk_widget_show (dialog_vbox1);

  label = gtk_label_new (_("This file is not an ELF executable!\nDo you wish to load it as a binary file?"));
  gtk_widget_ref (label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label, TRUE, TRUE, 0);                //This is the contents of the file error window

  view_binary_load_address = gtk_entry_new_with_max_length (64);
  gtk_widget_ref (view_binary_load_address);
  gtk_widget_show (view_binary_load_address);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), view_binary_load_address, TRUE, TRUE, 0);
  
  label = gtk_label_new (_("Valid:0"));
  gtk_widget_ref (label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label, TRUE, TRUE, 0);                //This is the contents of the file error window


  dialog_action_area1 = GTK_DIALOG (view_fileerror)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);         //Action area is the bottom box which holds the OK and Cancel buttons

  buttonok = gtk_button_new_with_label (_("OK"));
  gtk_widget_ref (buttonok);
  gtk_widget_show (buttonok);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), buttonok, TRUE, TRUE, 0);

  buttoncancel = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (buttoncancel);
  gtk_widget_show (buttoncancel);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), buttoncancel, TRUE, TRUE, 0);

  gtk_signal_connect_object (GTK_OBJECT (view_fileerror), "delete_event",           //If window is closed then just hide and dont destroy
                            GTK_SIGNAL_FUNC (gtk_widget_hide),
                            GTK_OBJECT (view_fileerror));

  gtk_signal_connect (GTK_OBJECT (buttonok), "clicked",                             //If ok pressed the load
                      GTK_SIGNAL_FUNC (callback_load_binary),
                      centry_load);
                      
  gtk_signal_connect_object (GTK_OBJECT (buttoncancel), "clicked",                  //If cancel pressed then hide
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (view_fileerror));
                             
  gtk_signal_connect_after (GTK_OBJECT (view_binary_load_address), "changed",                  //If cancel pressed then hide
                             GTK_SIGNAL_FUNC (callback_load_binary_address),
                             GTK_OBJECT (label));
  
  gtk_entry_set_text (GTK_ENTRY(view_binary_load_address),"0");


  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 /* hbox = gtk_hbox_new (FALSE, 4);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox); */
  /*
  progressbar = gtk_progress_bar_new ();
  gtk_widget_ref (progressbar);
  gtk_widget_show (progressbar);
  gtk_box_pack_start (GTK_BOX (hbox), progressbar, TRUE, TRUE, 4);
  gtk_progress_configure (GTK_PROGRESS (progressbar), 100, 100, 200);

  gtk_progress_set_show_text (GTK_PROGRESS (progressbar), TRUE);
  view_progressbar=progressbar; */
  
  
 // removed since to current availability from menu
   
 /* button_about = gtk_button_new_with_label ("About");
  gtk_widget_ref (button_about);
  gtk_widget_show (button_about);  
  gtk_tooltips_set_tip (view_tooltips,button_about , "About KMD", NULL);
  gtk_box_pack_start (GTK_BOX (hbox), button_about, FALSE, FALSE, 0);
  gtk_signal_connect_object(GTK_OBJECT (button_about), "clicked", gtk_widget_show ,GTK_OBJECT (window_about));
 */
		 
 /* gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0); */

  fileselection = gtk_file_selection_new (_("Select Source File"));
  gtk_container_set_border_width (GTK_CONTAINER (fileselection), 10);
  ok_button = GTK_FILE_SELECTION (fileselection)->ok_button;
  gtk_widget_show (ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  cancel_button = GTK_FILE_SELECTION (fileselection)->cancel_button;
  gtk_widget_show (cancel_button);
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_file_selection_complete (GTK_FILE_SELECTION(fileselection),"*.s");
  gtk_signal_connect_object (GTK_OBJECT (button_findcompile), "clicked", /* these are the two file selection satelite windows*/
                             GTK_SIGNAL_FUNC (gtk_widget_show),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_ok_file),centry_compile);

  
  fileselection = gtk_file_selection_new (_("Select Object File"));
  gtk_container_set_border_width (GTK_CONTAINER (fileselection), 10);
  ok_button = GTK_FILE_SELECTION (fileselection)->ok_button;
  gtk_widget_show (ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  cancel_button = GTK_FILE_SELECTION (fileselection)->cancel_button;
  gtk_widget_show (cancel_button);
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_file_selection_complete (GTK_FILE_SELECTION(fileselection),"*.elf");
  gtk_signal_connect_object (GTK_OBJECT (button_findload), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_show),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_ok_file),centry_load);
  
  
  return vbox;
}




GtkWidget*
create_featurewindow (void)
{
  GtkWidget *window;
  GtkWidget *notebook;
  GtkWidget *label;
  GtkWidget *segment;  
  int count;
  int flag_unknown_feature_found;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL); /* build features window */
  gtk_window_set_default_size(GTK_WINDOW (window),400,400);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), _("Features"));
  if (board_num_features) /* if features exist */
  { 
    int actual_found_feature_found_number=-1; /* numbering of recognised features, will start at 0 when a feature is found and increments -1 */
    notebook = gtk_notebook_new (); /* to hold multiple features */
    gtk_widget_ref (notebook);
    gtk_object_set_data_full (GTK_OBJECT (window), "notebook", notebook,
                             (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (notebook);
    gtk_container_add (GTK_CONTAINER (window), notebook); /* attach notebook to window */
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    for (count=0; count<board_num_features; count++)
    { 
      flag_unknown_feature_found=FALSE;
      /* step through all features */
     switch (board_features[count].type) 
         {
         case XILINX_FPGA:
	          if (VERBOSE) 
	             g_print("%s detected\n", board_features[count].name);
                  segment = create_xfpgasegment();
                  actual_found_feature_found_number++;
		  break;
         case CONSOLE:
		  if (VERBOSE) 
	             g_print("Console detected\n");
                  segment = view_create_comms(&board_features[count]);
                  actual_found_feature_found_number++;
		  break;
         default:
                  if (VERBOSE) 
		     g_print("Unknown feature!\n");
	             flag_unknown_feature_found=TRUE;
         }
     if (flag_unknown_feature_found==FALSE)
	{
          gtk_object_set_data (GTK_OBJECT (segment), "feature", GINT_TO_POINTER(count));
          gtk_container_add (GTK_CONTAINER (notebook), segment);
          label = gtk_label_new (board_features[count].name); /* get name of feature */
          gtk_widget_ref (label);
          gtk_widget_show (label);
          gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook),
               gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), actual_found_feature_found_number), label);
        }
     /* put the name of the feature in the tab and add that feature to the window*/
     }
   } 
   else 
    {
     label = gtk_label_new ("There are no features available");
     gtk_widget_ref (label);
     gtk_widget_show (label);
     gtk_misc_set_padding (GTK_MISC (label), 100, 50);
     gtk_container_add (GTK_CONTAINER (window), label);
    }
  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
                            GTK_SIGNAL_FUNC (gtk_widget_hide),
                            GTK_OBJECT (window));
  return window;
}

/* the following creates a feature entry for the FPGA */

GtkWidget*
create_xfpgasegment (void)
{
  GtkWidget *container;
  GtkWidget *hbox1;
  GtkWidget *buttonupdate;
  GtkWidget *combo;
  GtkWidget *fileentry;
  GtkWidget *browse;
  GtkWidget *hbox2;
  GtkWidget *table;
  GtkWidget *label27;
  GtkWidget *labelfilename;
  GtkWidget *label29;
  GtkWidget *labelfiledate;
  GtkWidget *label42;
  GtkWidget *labelfiletime;
  GtkWidget *buttondownload;
  GtkWidget *progressbar;
  GtkWidget *hbox3;
  GtkWidget *table8;
  GtkWidget *label44;
  GtkWidget *labelfpganame;
  GtkWidget *label46;
  GtkWidget *labelfpgadate;
  GtkWidget *label48;
  GtkWidget *labelfpgatime;
  GtkWidget *buttonerase;
  GtkWidget *fileselection;
  GtkWidget *ok_button;
  GtkWidget *cancel_button;

  container = gtk_vbox_new (FALSE, 11);
  gtk_object_set_data (GTK_OBJECT (container), "list", NULL);
  gtk_widget_ref (container);
  gtk_widget_show (container);
  gtk_container_set_border_width (GTK_CONTAINER (container), 6);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (container), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (container), hbox1, FALSE, FALSE, 0);

  buttonupdate = gtk_button_new_with_label (_("Update"));
  gtk_widget_ref (buttonupdate);
  gtk_object_set_data_full (GTK_OBJECT (container), "buttonupdate", buttonupdate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (buttonupdate);
  gtk_box_pack_start (GTK_BOX (hbox1), buttonupdate, FALSE, FALSE, 0);
  
  gtk_signal_connect (GTK_OBJECT (buttonupdate), "clicked",
                      GTK_SIGNAL_FUNC (callback_fpgaupdate),container);

  combo = gtk_combo_new ();
  gtk_widget_ref (combo);
  gtk_object_set_data_full (GTK_OBJECT (container), "combo", combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo);
  gtk_box_pack_start (GTK_BOX (hbox1), combo, TRUE, TRUE, 0);

  fileentry = GTK_COMBO (combo)->entry;
  gtk_widget_ref (fileentry);
  gtk_object_set_data_full (GTK_OBJECT (container), "fileentry", fileentry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry);

  browse = gtk_button_new_with_label (_("Browse..."));
  gtk_widget_ref (browse);
  gtk_object_set_data_full (GTK_OBJECT (container), "browse", browse,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (browse);
  gtk_box_pack_start (GTK_BOX (hbox1), browse, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (container), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (container), hbox2, FALSE, FALSE, 0);

  table = gtk_table_new (3, 2, FALSE);
  gtk_widget_ref (table);
  gtk_object_set_data_full (GTK_OBJECT (container), "table", table,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table);
  gtk_box_pack_start (GTK_BOX (hbox2), table, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table), 30);

  label27 = gtk_label_new (_("Name:"));
  gtk_widget_ref (label27);
  gtk_object_set_data_full (GTK_OBJECT (container), "label27", label27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label27);
  gtk_table_attach (GTK_TABLE (table), label27, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label27), GTK_JUSTIFY_LEFT);

  labelfilename = gtk_label_new (_("Empty"));
  gtk_widget_ref (labelfilename);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfilename", labelfilename,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfilename);
  gtk_table_attach (GTK_TABLE (table), labelfilename, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label29 = gtk_label_new (_("Date:"));
  gtk_widget_ref (label29);
  gtk_object_set_data_full (GTK_OBJECT (container), "label29", label29,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label29);
  gtk_table_attach (GTK_TABLE (table), label29, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label29), GTK_JUSTIFY_LEFT);

  labelfiledate = gtk_label_new (_("0000/00/00"));
  gtk_widget_ref (labelfiledate);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfiledate", labelfiledate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfiledate);
  gtk_table_attach (GTK_TABLE (table), labelfiledate, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label42 = gtk_label_new (_("Time"));
  gtk_widget_ref (label42);
  gtk_object_set_data_full (GTK_OBJECT (container), "label42", label42,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label42);
  gtk_table_attach (GTK_TABLE (table), label42, 0, 1, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  labelfiletime = gtk_label_new (_("00:00:00"));
  gtk_widget_ref (labelfiletime);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfiletime", labelfiletime,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfiletime);
  gtk_table_attach (GTK_TABLE (table), labelfiletime, 1, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  buttondownload = gtk_button_new_with_label (_("Download"));
  gtk_widget_ref (buttondownload);
  gtk_object_set_data_full (GTK_OBJECT (container), "buttondownload", buttondownload,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (buttondownload);
  gtk_box_pack_start (GTK_BOX (hbox2), buttondownload, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (buttondownload), "clicked",
                      GTK_SIGNAL_FUNC (callback_fpgaload),container);
                      
  progressbar = gtk_progress_bar_new ();
  gtk_widget_ref (progressbar);  gtk_widget_show (progressbar);
  gtk_object_set_data_full (GTK_OBJECT (container), "progressbar", progressbar,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_box_pack_start (GTK_BOX (container), progressbar, FALSE, FALSE, 0);
  gtk_progress_configure (GTK_PROGRESS (progressbar), 0, 0, 100);
  gtk_progress_set_show_text (GTK_PROGRESS (progressbar), TRUE);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (container), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (container), hbox3, FALSE, FALSE, 0);

  table8 = gtk_table_new (3, 2, FALSE);
  gtk_widget_ref (table8);
  gtk_object_set_data_full (GTK_OBJECT (container), "table8", table8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table8);
  gtk_box_pack_start (GTK_BOX (hbox3), table8, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table8), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table8), 30);

  label44 = gtk_label_new (_("Name:"));
  gtk_widget_ref (label44);
  gtk_object_set_data_full (GTK_OBJECT (container), "label44", label44,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label44);
  gtk_table_attach (GTK_TABLE (table8), label44, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label44), GTK_JUSTIFY_LEFT);

  labelfpganame = gtk_label_new (_("Empty"));
  gtk_widget_ref (labelfpganame);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfpganame", labelfpganame,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfpganame);
  gtk_table_attach (GTK_TABLE (table8), labelfpganame, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label46 = gtk_label_new (_("Date:"));
  gtk_widget_ref (label46);
  gtk_object_set_data_full (GTK_OBJECT (container), "label46", label46,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label46);
  gtk_table_attach (GTK_TABLE (table8), label46, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label46), GTK_JUSTIFY_LEFT);

  labelfpgadate = gtk_label_new (_("0000/00/00"));
  gtk_widget_ref (labelfpgadate);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfpgadate", labelfpgadate,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfpgadate);
  gtk_table_attach (GTK_TABLE (table8), labelfpgadate, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label48 = gtk_label_new (_("Time"));
  gtk_widget_ref (label48);
  gtk_object_set_data_full (GTK_OBJECT (container), "label48", label48,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label48);
  gtk_table_attach (GTK_TABLE (table8), label48, 0, 1, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  labelfpgatime = gtk_label_new (_("00:00:00"));
  gtk_widget_ref (labelfpgatime);
  gtk_object_set_data_full (GTK_OBJECT (container), "labelfpgatime", labelfpgatime,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (labelfpgatime);
  gtk_table_attach (GTK_TABLE (table8), labelfpgatime, 1, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  buttonerase = gtk_button_new_with_label (_("Erase"));
  gtk_widget_ref (buttonerase);
  gtk_object_set_data_full (GTK_OBJECT (container), "buttonerase", buttonerase,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (buttonerase);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonerase, FALSE, FALSE, 0);


  fileselection = gtk_file_selection_new (_("Select FPGA .bit File"));
  gtk_container_set_border_width (GTK_CONTAINER (fileselection), 10);
  ok_button = GTK_FILE_SELECTION (fileselection)->ok_button;
  gtk_widget_show (ok_button);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
  cancel_button = GTK_FILE_SELECTION (fileselection)->cancel_button;
  gtk_widget_show (cancel_button);
  GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
  gtk_file_selection_complete (GTK_FILE_SELECTION(fileselection),"*.bit");
  gtk_signal_connect_object (GTK_OBJECT (browse), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_show),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (fileselection));
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_ok_file),fileentry);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (callback_fpgaupdate),container);


  return container;
}


/*

#ifdef GTK2
GtkWidget* view_create_comms (feature* console) 
{
  GtkWidget *vbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *text1;
  
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_widget_show (vbox1);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  text1 = gtk_text_new (NULL, NULL);
  gtk_widget_ref (text1);
  gtk_widget_show (text1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);

  console_entry = gtk_entry_new ();
  gtk_widget_ref (console_entry);
  gtk_widget_show (console_entry);
  gtk_box_pack_start (GTK_BOX (vbox1), console_entry, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(console_entry), "activate",
                         GTK_SIGNAL_FUNC (callback_comms),console);
  console->data.console.text=text1;
  g_timeout_add(POLLING_PERIOD, callback_update_comms, console);
  return vbox1;
}
#endif

#ifndef GTK2 

*/
GtkWidget* view_create_comms (feature* console)  /*create a console window in features*/
{
  GtkWidget *vbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *text1;
  GtkWidget *console_clear;
 
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_widget_show (vbox1);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  text1 = gtk_text_new (NULL, NULL);
  gtk_widget_ref (text1);
  gtk_widget_show (text1);
  gtk_tooltips_set_tip (view_tooltips, text1, "Type here to input/output data", NULL);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);

  /*console_entry = gtk_entry_new ();
  gtk_widget_ref (console_entry);
  gtk_widget_show (console_entry);
  gtk_box_pack_start (GTK_BOX (vbox1), console_entry, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(console_entry), "activate",
                         GTK_SIGNAL_FUNC (callback_comms),console);*/
			  	    
  console_clear = gtk_button_new_with_label("Clear"); 
  gtk_widget_ref (console_clear);
  gtk_widget_show (console_clear);  
  gtk_tooltips_set_tip (view_tooltips,console_clear , "Clear", NULL);
  gtk_box_pack_start (GTK_BOX (vbox1), console_clear, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(console_clear), "clicked",
                         GTK_SIGNAL_FUNC (callback_clear_console),console);
  			/* change callback function */
  
  
  		  
  console->data.console.text=text1; /* set the text in console to text1 */
  gtk_key_snooper_install(callback_console_keysnoop,console); /* send to callback with pointer to the text area */
  g_timeout_add(POLLING_PERIOD, callback_update_comms, console); /* poll and communicate with the devices every POLLING_PERIOD ms */
  if (console->dev_number>=CONSOLE_UPPER_BOUND) /* we won't allow console as 20th feature or higher */
     {
      g_print("Console feature found at predefined upper bound.\n");
      exit(1);
     } 
  else
     text_in_console[console->dev_number]=text1; /* save text entry, indexed intuitively */
  return vbox1;
}
// #endif



void create_splashwindow (void)
{
  GtkWidget *vbox; 
  GtkWidget *pixmap;
  view_komodologo_pixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, gdk_colormap_get_system(), &view_komodologo_bitmap,NULL,komodologo_xpm);
  
  view_splashwindow = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_window_set_position (GTK_WINDOW(view_splashwindow),GTK_WIN_POS_CENTER);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (view_splashwindow), vbox);

  pixmap = gtk_pixmap_new  (view_komodologo_pixmap, view_komodologo_bitmap);
  gtk_widget_ref (pixmap);
  gtk_widget_show (pixmap);
  gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
  gtk_widget_show (view_splashwindow);
  gtk_widget_add_events  (view_splashwindow, 1);
  while (gtk_events_pending())  gtk_main_iteration();
  return;
}



GtkWidget*
create_aboutwindow (void)
{
  GtkWidget *aboutwindow;
  GtkWidget *hbox1;
  GtkWidget *pixmap1;
  GtkWidget *vbox;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *button1;

  aboutwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (aboutwindow), "aboutwindow", aboutwindow);
  gtk_window_set_title (GTK_WINDOW (aboutwindow), _("About"));

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "vbox", vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (aboutwindow), vbox);

  pixmap1 = gtk_pixmap_new  (view_komodotitle_pixmap, view_komodotitle_bitmap);
  gtk_widget_ref (pixmap1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "pixmap1", pixmap1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (vbox), pixmap1, FALSE, FALSE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);
  
  pixmap1 = gtk_pixmap_new  (view_mulogo_pixmap, view_mulogo_bitmap);
  gtk_widget_ref (pixmap1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "pixmap1", pixmap1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (hbox1), pixmap1, FALSE, FALSE, 0);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ((PACKAGE "\nVersion " VERSION " (" DATE ")\n\nCopyright " YEAR ":\nUniversity of Manchester\n\nAuthors:\nCharlie Brej\nhttp://www.cs.man.ac.uk/~brejc8\nJim Garside\nhttp://www.cs.man.ac.uk/amulet/staff/details/jim.html \n\nBug reports and feature requests:\n komodo@cs.man.ac.uk\nWebsite:\nhttp://www.cs.man.ac.uk/teaching/electronics/komodo"));
  gtk_misc_set_padding (GTK_MISC (label1), 10, 10);
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);

  button1 = gtk_button_new_with_label (_("Dismiss"));
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (vbox1), button1, FALSE, FALSE, 0);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Powered By");
  gtk_misc_set_padding (GTK_MISC (label1), 10, 10);
  gtk_widget_ref (label1);  
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  
  pixmap1 = gtk_pixmap_new  (view_chump_pixmap, view_chump_bitmap);
  gtk_widget_ref (pixmap1);
  gtk_object_set_data_full (GTK_OBJECT (aboutwindow), "pixmap1", pixmap1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (vbox1), pixmap1, TRUE, TRUE, 0);


  gtk_signal_connect_object (GTK_OBJECT (aboutwindow), "delete_event",
                            GTK_SIGNAL_FUNC (gtk_widget_hide),
                            GTK_OBJECT (aboutwindow));
  gtk_signal_connect_object (GTK_OBJECT (button1), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_hide),
                             GTK_OBJECT (aboutwindow));

  return aboutwindow;
}



GtkWidget* view_create_prog_ctrl (void)
{ 
  GtkAccelGroup *accel_group = gtk_accel_group_new ();
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *button_start;
  GtkWidget *button_stop;
  GtkWidget *button_single;
  GtkObject *stepnumber_adj;
  GtkWidget *stepnumber;
  GtkWidget *button_multi;
  GtkWidget *cbutton_break;
  GtkWidget *cbutton_swi;
  GtkWidget *cbutton_proc;
  GtkWidget *label;
  GtkWidget *button_feature;
  
  const static int zero = 0;
  const static int one = 1;
  
  
  window_feature = create_featurewindow(); 
  /* create a features window and set window_feature to point to it */
  /* window_feature declared in view.h */

  vbox = gtk_vbox_new (FALSE, 2);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);

  
  button_start = gtk_button_new_with_label ("Ping");
  gtk_widget_ref (button_start);
  gtk_widget_show (button_start);
  gtk_box_pack_start (GTK_BOX (hbox), button_start, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_start), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_ping),NULL);
  gtk_tooltips_set_tip (view_tooltips, button_start, "Ping board and refresh display", NULL);
                      

  
  
  button_start = gtk_button_new_with_label (_("Run"));
  gtk_widget_ref (button_start);
  gtk_widget_show (button_start);
  gtk_box_pack_start (GTK_BOX (hbox), button_start, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_start), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&zero);/* link to callback and pass stepcount*/
  gtk_widget_add_accelerator (button_start, "clicked", accel_group,
                              GDK_F5, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_start, "Start execution [F5]", NULL);

  button_stop = gtk_button_new_with_label (_("Stop"));
  gtk_widget_ref (button_stop);
  gtk_widget_show (button_stop);
  gtk_box_pack_start (GTK_BOX (hbox), button_stop, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_stop), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_stop),NULL);
  gtk_widget_add_accelerator (button_stop, "clicked", accel_group,
                              GDK_F6, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_stop, "Stop execution [F6]", NULL);

  
  button_multi = gtk_button_new_with_label (_("Continue"));
  gtk_widget_ref (button_multi);
  gtk_widget_show (button_multi);
  gtk_box_pack_start (GTK_BOX (hbox), button_multi, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_multi), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_continue),NULL);
  gtk_widget_add_accelerator (button_multi, "clicked", accel_group,
                              GDK_F10, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_multi, "Continue execution [F10]", NULL);
    
  
  
  
  button_start = gtk_button_new_with_label ("Reset");
  gtk_widget_ref (button_start);
  gtk_widget_show (button_start);
  gtk_box_pack_start (GTK_BOX (hbox), button_start, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_start), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_reset),NULL);
  gtk_tooltips_set_tip (view_tooltips, button_start, "Reset board", NULL);
  
  
  button_single = gtk_button_new_with_label (_("Single-Step"));
  gtk_widget_ref (button_single);
  gtk_widget_show (button_single);
  gtk_box_pack_start (GTK_BOX (hbox), button_single, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_single), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&one);
  gtk_widget_add_accelerator (button_single, "clicked", accel_group,
                              GDK_F7, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_single, "Take one step [F7]", NULL);

  stepnumber_adj = gtk_adjustment_new (1, 1, 10000000000, 1, 1000, 10);
  stepnumber = gtk_spin_button_new (GTK_ADJUSTMENT (stepnumber_adj), 1, 0);
  gtk_widget_ref (stepnumber);
  gtk_widget_show (stepnumber);
  gtk_box_pack_start (GTK_BOX (hbox), stepnumber, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (stepnumber), "changed",
                      GTK_SIGNAL_FUNC (callback_step_number), &view_step_number);
  gtk_widget_set_usize (stepnumber, 30, -2);
  gtk_tooltips_set_tip (view_tooltips, stepnumber, "Change number of steps to take. Remember to press return after entering a new value", NULL);
  
  button_multi = gtk_button_new_with_label (_("Multi-Step"));
  gtk_widget_ref (button_multi);
  gtk_widget_show (button_multi);
  gtk_box_pack_start (GTK_BOX (hbox), button_multi, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_multi), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_start),(gpointer)&view_step_number);
  gtk_widget_add_accelerator (button_multi, "clicked", accel_group,
                              GDK_F8, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_multi, "Take a number of steps [F8]", NULL);
  
  stepnumber_adj = gtk_adjustment_new (1000, 1, 10000, 1, 1000, 10);
  stepnumber = gtk_spin_button_new (GTK_ADJUSTMENT (stepnumber_adj), 1, 0);
  gtk_widget_ref (stepnumber);
  gtk_widget_show (stepnumber);
  gtk_box_pack_start (GTK_BOX (hbox), stepnumber, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (stepnumber), "changed",
                      GTK_SIGNAL_FUNC (callback_step_number), &view_step_freq);
  gtk_widget_set_usize (stepnumber, 30, -2);
  gtk_tooltips_set_tip (view_tooltips, stepnumber, "Change period (ms) between steps when walking. Remember to press return after entering a new value", NULL);
                              
  button_multi = gtk_button_new_with_label (_("Walk"));
  gtk_widget_ref (button_multi);
  gtk_widget_show (button_multi);
  gtk_box_pack_start (GTK_BOX (hbox), button_multi, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button_multi), "clicked",
                      GTK_SIGNAL_FUNC (callback_button_walk),(gpointer)&view_step_number);
  gtk_widget_add_accelerator (button_multi, "clicked", accel_group,
                              GDK_F9, 0, GTK_ACCEL_VISIBLE);
  gtk_tooltips_set_tip (view_tooltips, button_multi, "Repeatedly take a number of steps [F9]", NULL);

  view_step_number=1;
  view_step_freq=1000;
  
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  hbox = gtk_hbox_new (FALSE, 4);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);
 
  button_feature = gtk_button_new_with_label ("Features");
  gtk_widget_ref (button_feature);
  gtk_widget_show (button_feature);
  gtk_tooltips_set_tip (view_tooltips,button_feature , "Features Available", NULL);
  gtk_box_pack_start (GTK_BOX (hbox), button_feature, FALSE, FALSE, 0);
  gtk_signal_connect_object(GTK_OBJECT (button_feature), "clicked", gtk_widget_show ,GTK_OBJECT (window_feature));
  
  
  
  cbutton_break = gtk_check_button_new_with_label ("Breakpoints");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_break),TRUE);
  gtk_widget_ref (cbutton_break);
  gtk_widget_show (cbutton_break);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_break, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_break), "toggled",
                      GTK_SIGNAL_FUNC (callback_start_toggle), GINT_TO_POINTER(0x10));
  gtk_tooltips_set_tip (view_tooltips, cbutton_break, "Activate breakpoints", NULL);

  cbutton_break = gtk_check_button_new_with_label ("Watchpoints");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_break),TRUE);
  gtk_widget_ref (cbutton_break);
  gtk_widget_show (cbutton_break);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_break, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_break), "toggled",
                      GTK_SIGNAL_FUNC (callback_start_toggle), GINT_TO_POINTER(0x20));
  gtk_tooltips_set_tip (view_tooltips, cbutton_break, "Activate watchpoints", NULL);

  label = gtk_label_new("Service:");
  gtk_widget_ref (label);   gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  cbutton_swi = gtk_check_button_new_with_label ("SWI");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_swi),FALSE);
  gtk_widget_ref (cbutton_swi);
  gtk_widget_show (cbutton_swi);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_swi, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_swi), "toggled",
                      GTK_SIGNAL_FUNC (callback_start_toggle), GINT_TO_POINTER(0x4));
  gtk_tooltips_set_tip (view_tooltips, cbutton_swi, "Enter software interrupt code", NULL);

  cbutton_proc = gtk_check_button_new_with_label ("BL");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_proc),FALSE);
  gtk_widget_ref (cbutton_proc);
  gtk_widget_show (cbutton_proc);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_proc, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_proc), "toggled",
                      GTK_SIGNAL_FUNC (callback_start_toggle), GINT_TO_POINTER(0x2));
  gtk_tooltips_set_tip (view_tooltips, cbutton_proc, "Enter sub-procedure code", NULL);
  
  cbutton_proc = gtk_check_button_new_with_label ("Abort");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_proc),FALSE);
  gtk_widget_ref (cbutton_proc);                    gtk_widget_show (cbutton_proc); 
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_proc, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_proc), "toggled",
                      GTK_SIGNAL_FUNC (callback_start_toggle), GINT_TO_POINTER(0x8));
  gtk_tooltips_set_tip (view_tooltips, cbutton_proc, "Enter Bad Memory access exception code", NULL);
  
  label = gtk_label_new("Active:");
  gtk_widget_ref (label);   gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);


  cbutton_proc = gtk_check_button_new_with_label ("IRQ");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_proc),FALSE);
  gtk_widget_ref (cbutton_proc);
  gtk_widget_show (cbutton_proc);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_proc, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_proc), "toggled",
                      GTK_SIGNAL_FUNC (callback_rtf_toggle), GINT_TO_POINTER(0x2));
  gtk_tooltips_set_tip (view_tooltips, cbutton_proc, "Activate automatic execution of interrupt code", NULL);
  
  cbutton_proc = gtk_check_button_new_with_label ("FIQ");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_proc),FALSE);
  gtk_widget_ref (cbutton_proc);                    gtk_widget_show (cbutton_proc); 
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_proc, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_proc), "toggled",
                      GTK_SIGNAL_FUNC (callback_rtf_toggle), GINT_TO_POINTER(0x1));
  gtk_tooltips_set_tip (view_tooltips, cbutton_proc, "Activate automatic execution of Fast interrupt code", NULL);
  
  cbutton_proc = gtk_toggle_button_new_with_label ("Refresh");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton_proc),FALSE);
  gtk_widget_ref (cbutton_proc);                  
  gtk_widget_show (cbutton_proc);
  gtk_box_pack_start (GTK_BOX (hbox), cbutton_proc, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (cbutton_proc), "toggled",
                      GTK_SIGNAL_FUNC (callback_refresh_toggle), NULL);
  gtk_tooltips_set_tip (view_tooltips, cbutton_proc, "Refresh screen regularly", NULL);
  view_refreshbutton = cbutton_proc;

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
  gtk_window_add_accel_group (GTK_WINDOW (view_mainwindow), accel_group);
  
  
  return vbox;
}

GtkWidget*
create_memorywindow (void)
{ 
  GtkWidget *window;
  GtkWidget *clist;
  GtkWidget *scrolledwindow;           //create memwindow
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *vbox1;
  GtkWidget *entry;
  
  GtkWidget *optionmenu_length;
  GtkWidget *optionmenu_isa;
  GtkWidget *optionmenu_menu;
  GtkWidget *menuitem;
  GtkObject *listsize_adj;
  GtkWidget *listsize;
  GtkWidget *vscale;
  GtkObject *vscale_adj;
  GtkWidget *button_memory;
  GtkWidget *window_memory; 
  GdkGeometry hints;
  
  char *startdata[] = {"Address","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","ASCII","DISS"};
  char *slidelabeldata[] = {"-1000","-100","-10","0","+10","+100","+1000"};
  int temp;
  DefinitionStack* isa_list;
  mem_window *memwindow = g_new(mem_window,1);
  mem_window_list **memwindowlist = &view_memwindowlist;
  
  hints.base_width = 1000;
  hints.base_height =  590;
  hints.max_height = 590;
  hints.max_width = 1000;
  hints.min_width = 1000;
  hints.min_height = 590;
  
   
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size (GTK_WINDOW (window), 1000, 590); 
  gtk_window_set_position(GTK_WINDOW (window),GTK_WIN_POS_NONE);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), _("Memory Window"));  
  gtk_window_set_geometry_hints(GTK_WINDOW (window),
		                GTK_WIDGET (window),
		  		&hints,
		  		GDK_HINT_MAX_SIZE|GDK_HINT_MIN_SIZE|GDK_HINT_BASE_SIZE);
		  
  while (NULL != *memwindowlist) memwindowlist = &((*memwindowlist)->next);
  *memwindowlist = g_new(mem_window_list,1);
  (*memwindowlist)->mem_data_ptr = memwindow;
  (*memwindowlist)->next = NULL;
  
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox);   
  gtk_widget_show (hbox);
  gtk_object_set_data (GTK_OBJECT (hbox), "mem_window", memwindow);
  
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

  memwindow->address= g_new0(uchar,4);
  memwindow->regoffset = board_nulreg;
  memwindow->count=LENGTH_OF_MEMORY_WINDOW;
  memwindow->width=4;
  memwindow->gran=4;
//  memwindow->wait=100;
//  memwindow->timer=0;
  
  vscale_adj = gtk_adjustment_new (0, -3.25, 3.25, 3, 3, 0);
  vscale = gtk_vscale_new (GTK_ADJUSTMENT (vscale_adj));
  gtk_widget_ref (vscale);  gtk_widget_show (vscale);
  gtk_box_pack_start (GTK_BOX (hbox), vscale, FALSE, FALSE, 0);
  gtk_scale_set_draw_value (GTK_SCALE (vscale), FALSE);
  gtk_scale_set_digits(GTK_SCALE (vscale), 2);
  gtk_range_set_update_policy (GTK_RANGE (vscale), GTK_UPDATE_CONTINUOUS);
  gtk_signal_connect (GTK_OBJECT (vscale_adj), "value_changed",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollmove), NULL);
  gtk_signal_connect (GTK_OBJECT (vscale), "button-release-event",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollrelease), NULL);
  gtk_signal_connect_after (GTK_OBJECT (vscale), "key-press-event",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollrelease), NULL);
  gtk_object_set_data (GTK_OBJECT (vscale_adj), "mem_window", memwindow);
  memwindow->scroll = vscale;
  
  vbox1 = gtk_vbox_new (TRUE, 0);
  gtk_widget_ref (vbox1);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox), vbox1, FALSE, FALSE, 0);
  
for (temp=0;temp<7;temp++) /* set slidebar */
    {
      label = gtk_label_new (slidelabeldata[temp]);
      gtk_widget_ref (label);
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (vbox1), label, FALSE, FALSE, 0);
    }

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, FALSE, FALSE, 0);
  
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);

  optionmenu_isa = gtk_option_menu_new();
  gtk_widget_ref (optionmenu_isa);
  gtk_widget_show (optionmenu_isa);
  optionmenu_menu = gtk_menu_new();
  gtk_widget_ref (optionmenu_menu);
  
  isa_list = board_asm_tables;
  while (isa_list)
   {
    menuitem = gtk_menu_item_new_with_label (isa_list->string);
    gtk_widget_show (menuitem);
    gtk_menu_prepend (GTK_MENU (optionmenu_menu), menuitem);
    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                      GTK_SIGNAL_FUNC (callback_memwindow_isa),(gpointer)isa_list->rules);
    memwindow->isa = isa_list->rules;
    isa_list=isa_list->next;
   }


  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_isa),
                                             optionmenu_menu);
  gtk_object_set_data (GTK_OBJECT (optionmenu_menu), "mem_window", memwindow);

  entry = gtk_entry_new_with_max_length (40);
  memwindow->address_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE, 0);
  gtk_widget_set_usize (entry, 107, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect (GTK_OBJECT(entry), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_address),NULL);

  entry = gtk_entry_new_with_max_length (64);
  memwindow->hex_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE,0);
  gtk_widget_set_usize (entry, 800, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect_after (GTK_OBJECT(entry), "activate",
                  GTK_SIGNAL_FUNC (callback_memwindow_hex),NULL);
 
  

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_end (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);

  clist = gtk_clist_new (9);
  memwindow->clist_ptr = clist;
  for (temp=0;temp < memwindow->count;temp++)
       gtk_clist_append(GTK_CLIST(clist),startdata);

  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
  gtk_signal_connect (GTK_OBJECT(clist), "select-row",
               GTK_SIGNAL_FUNC (callback_memory_window_row_selected),NULL);
  gtk_widget_show (clist);
  
  gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
  
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 3, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 4, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 5, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 6, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 7, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 8, 100);
  
  
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  label = gtk_label_new (_("Address"));
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label); 
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 3, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 4, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 5, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 6, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 7, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 8, label);
  
  
  callback_set_memory_window_size(GTK_MENU_ITEM(menuitem),(gpointer)REPRESENTATION_8_WORDS);
  /* set the culumn list to display 8 words in each line */
  view_updatememwindow(memwindow);
  gtk_widget_set_style(clist,fixed_style);

  gtk_container_add (GTK_CONTAINER (window), hbox);

  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
                            GTK_SIGNAL_FUNC (gtk_widget_hide),
                            GTK_OBJECT (window)); 
  return window; 
}


GtkWidget* view_create_memory_clist (void)
{
  
  GtkWidget *clist;
  GtkWidget *scrolledwindow;           /*create memwindow*/
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *vbox1;
  GtkWidget *entry;
  
  GtkWidget *optionmenu_length;
  GtkWidget *optionmenu_isa;
  GtkWidget *optionmenu_menu;
  GtkWidget *menuitem;
  GtkObject *listsize_adj;
  GtkWidget *listsize;
  GtkWidget *cbutton;
  GtkWidget *vscale;
  GtkObject *vscale_adj;
  GtkWidget *button_memory;
  GtkWidget *window_memory; 
  
  char *startdata[] = {"Address","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","Hex","ASCII","DISS"};
  char *slidelabeldata[] = {"-1000","-100","-10","0","+10","+100","+1000"};
  int temp;
  DefinitionStack* isa_list;
  
  
  
  mem_window *memwindow = g_new(mem_window,1);
  mem_window_list **memwindowlist = &view_memwindowlist;
  
  while (NULL != *memwindowlist) memwindowlist = &((*memwindowlist)->next);
  *memwindowlist = g_new(mem_window_list,1);
  (*memwindowlist)->mem_data_ptr = memwindow;
  (*memwindowlist)->next = NULL;
  
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox);   gtk_widget_show (hbox);
  gtk_object_set_data (GTK_OBJECT (hbox), "mem_window", memwindow);
  
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
  
  memwindow->address= g_new0(uchar,4);
  memwindow->regoffset = board_nulreg;
// printf("%d\n",get_width(hbox));
   memwindow->count=10;
  memwindow->width=4;
  memwindow->gran=4;
//  memwindow->wait=100;
//  memwindow->timer=0;
  
  vscale_adj = gtk_adjustment_new (0, -3.25, 3.25, 3, 3, 0);
  vscale = gtk_vscale_new (GTK_ADJUSTMENT (vscale_adj));
  gtk_widget_ref (vscale);  gtk_widget_show (vscale);
  gtk_box_pack_start (GTK_BOX (hbox), vscale, FALSE, FALSE, 0);
  gtk_scale_set_draw_value (GTK_SCALE (vscale), FALSE);
  gtk_scale_set_digits(GTK_SCALE (vscale), 2);
  gtk_range_set_update_policy (GTK_RANGE (vscale), GTK_UPDATE_CONTINUOUS);
  gtk_signal_connect (GTK_OBJECT (vscale_adj), "value_changed",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollmove), NULL);
  gtk_signal_connect (GTK_OBJECT (vscale), "button-release-event",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollrelease), NULL);
  gtk_signal_connect_after (GTK_OBJECT (vscale), "key-press-event",
                      GTK_SIGNAL_FUNC (callback_memwindow_scrollrelease), NULL);
  gtk_object_set_data (GTK_OBJECT (vscale_adj), "mem_window", memwindow);
  memwindow->scroll = vscale;
  
  vbox1 = gtk_vbox_new (TRUE, 0);
  gtk_widget_ref (vbox1);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox), vbox1, FALSE, FALSE, 0);
  
for (temp=0;temp<7;temp++){
    label = gtk_label_new (slidelabeldata[temp]);
    gtk_widget_ref (label);
  gtk_widget_show (label);
    gtk_box_pack_start (GTK_BOX (vbox1), label, FALSE, FALSE, 0);
    }

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, FALSE, FALSE, 0);
  
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
 if (!already_built_mem_window_button)
 {
  button_memory = gtk_button_new_with_label ("New Memory Window");
  // gtk_widget_set_usize(button_memory,140,40);
  gtk_widget_ref (button_memory);
  gtk_widget_show (button_memory);
  gtk_box_pack_start (GTK_BOX (hbox1), button_memory, FALSE, FALSE, 0);
  window_memory = create_memorywindow();
  gtk_signal_connect_object(GTK_OBJECT (button_memory), "clicked", gtk_widget_show ,GTK_OBJECT (window_memory));  
  gtk_tooltips_set_tip (view_tooltips, button_memory, "Open a separate memory window ", NULL);
  already_built_mem_window_button=TRUE;  /*++ record placement of button */
 } 
  optionmenu_length = gtk_option_menu_new();
  gtk_widget_ref (optionmenu_length);
  gtk_widget_show (optionmenu_length);
  optionmenu_menu = gtk_menu_new();
  gtk_widget_ref (optionmenu_menu);

  menuitem = gtk_menu_item_new_with_label (_("1 Byte"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_BYTE);
  
  menuitem = gtk_menu_item_new_with_label (_("2 Bytes"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_2_BYTES);
  
  menuitem = gtk_menu_item_new_with_label (_("4 Bytes"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_4_BYTES);
  
  menuitem = gtk_menu_item_new_with_label (_("8 Bytes"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_8_BYTES);
  
  menuitem = gtk_menu_item_new_with_label (_("16 Bytes"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_16_BYTES);
  
  menuitem = gtk_menu_item_new_with_label (_("1 Halfword"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_HALFWORD);
  
  menuitem = gtk_menu_item_new_with_label (_("2 Halfwords"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_2_HALFWORDS);
  
  menuitem = gtk_menu_item_new_with_label (_("4 Halfwords"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_4_HALFWORDS);
  
  
  menuitem = gtk_menu_item_new_with_label (_("8 Halfwords"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_8_HALFWORDS);
    
  menuitem = gtk_menu_item_new_with_label (_("1 Word"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_WORD);
  
  menuitem = gtk_menu_item_new_with_label (_("2 Words"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_2_WORDS);
    
  menuitem = gtk_menu_item_new_with_label (_("4 Words"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_4_WORDS);
    
  menuitem = gtk_menu_item_new_with_label (_("8 Words"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_8_WORDS);
  
  menuitem = gtk_menu_item_new_with_label (_("16 Words"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_16_WORDS);
  
  menuitem = gtk_menu_item_new_with_label (_("ASCII Map"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)REPRESENTATION_ASCII_MAP);
  
  
  
  
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_length),
                                             optionmenu_menu);
  gtk_option_menu_set_history     (GTK_OPTION_MENU (optionmenu_length), 9);
  gtk_tooltips_set_tip (view_tooltips, optionmenu_length, "Choose memory representation", NULL);
  gtk_box_pack_start (GTK_BOX (hbox1), optionmenu_length, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (optionmenu_menu), "mem_window", memwindow);




  optionmenu_isa = gtk_option_menu_new();
  gtk_widget_ref (optionmenu_isa);
  gtk_widget_show (optionmenu_isa);
  optionmenu_menu = gtk_menu_new();
  gtk_widget_ref (optionmenu_menu);
  
  isa_list = board_asm_tables;
  while (isa_list){
    menuitem = gtk_menu_item_new_with_label (isa_list->string);
    gtk_widget_show (menuitem);
    gtk_menu_prepend (GTK_MENU (optionmenu_menu), menuitem);
    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                      GTK_SIGNAL_FUNC (callback_memwindow_isa),(gpointer)isa_list->rules);
    memwindow->isa = isa_list->rules;
    isa_list=isa_list->next;
    }


  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_isa),
                                             optionmenu_menu);
  gtk_box_pack_start (GTK_BOX (hbox1), optionmenu_isa, TRUE, TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (optionmenu_menu), "mem_window", memwindow);







  listsize_adj = gtk_adjustment_new (10, 0, 40, 1, 1, 1);
  listsize = gtk_spin_button_new (GTK_ADJUSTMENT (listsize_adj), 1, 0);
  gtk_widget_ref (listsize);
  gtk_widget_show (listsize);
  gtk_box_pack_start (GTK_BOX (hbox1), listsize, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (listsize), "changed",
                      GTK_SIGNAL_FUNC (callback_memwindow_listsize), 0);
  
  cbutton = gtk_check_button_new_with_label (_("Symbols"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton),TRUE);
  gtk_widget_ref (cbutton);
  gtk_widget_show (cbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), cbutton, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(cbutton), "toggled",
                   GTK_SIGNAL_FUNC (callback_memwindow_elf_symbols_toggle),NULL);
  gtk_tooltips_set_tip (view_tooltips, cbutton, "Display/hide ELF symbols", NULL);
  memwindow->elf_symbols_button=cbutton;

  
  
  cbutton = gtk_check_button_new_with_label (_("ASCII"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton),TRUE);
  gtk_widget_ref (cbutton);
  gtk_widget_show (cbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), cbutton, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(cbutton), "toggled",
                   GTK_SIGNAL_FUNC (callback_memwindow_ascii_toggle),NULL);
  gtk_tooltips_set_tip (view_tooltips, cbutton, "Toggle ASCII column", NULL);
  memwindow->ascii_button=cbutton;
  
  
  cbutton = gtk_check_button_new_with_label (_("Disassembly"));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cbutton),TRUE);
  gtk_widget_ref (cbutton);
  gtk_widget_show (cbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), cbutton, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT(cbutton), "toggled",
                   GTK_SIGNAL_FUNC (callback_memwindow_dis_toggle),NULL);
  gtk_tooltips_set_tip (view_tooltips, cbutton, "Toggle disassembly column", NULL);
  memwindow->dis_button=cbutton;
  
  	

    
  entry = gtk_entry_new_with_max_length (40);
  memwindow->address_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE, 0);
  gtk_widget_set_usize (entry, 107, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect (GTK_OBJECT(entry), "activate",
                   GTK_SIGNAL_FUNC (callback_memwindow_address),NULL);

  entry = gtk_entry_new_with_max_length (64);
  memwindow->hex_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE,0);
  gtk_widget_set_usize (entry, 200, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect_after (GTK_OBJECT(entry), "activate",
                  GTK_SIGNAL_FUNC (callback_memwindow_hex),NULL);

  entry = gtk_entry_new_with_max_length (30);
  memwindow->ascii_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE, 0);
  gtk_widget_set_usize (entry, 87, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect_after (GTK_OBJECT(entry), "activate",
                  GTK_SIGNAL_FUNC (callback_memwindow_ascii),NULL);

  entry = gtk_entry_new_with_max_length (200);
  memwindow->dis_entry=entry;
  gtk_widget_ref (entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE, 0);
  gtk_widget_set_usize (entry, 257, -2);
  gtk_widget_set_style(entry,fixed_style);
  gtk_signal_connect_after (GTK_OBJECT(entry), "activate",
                  GTK_SIGNAL_FUNC (callback_memwindow_dis),NULL);

  entry = gtk_label_new (_("  "));
  gtk_widget_ref (entry);  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (hbox2), entry, FALSE, TRUE, 0);
 
  
  
  
  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_end (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);

  clist = gtk_clist_new (19);
  memwindow->clist_ptr = clist;
  for (temp=0;temp < memwindow->count;temp++)
                        gtk_clist_append(GTK_CLIST(clist),startdata);

  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
  gtk_signal_connect (GTK_OBJECT(clist), "select-row",
                   GTK_SIGNAL_FUNC (callback_memwindow_clist),NULL);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 193);
  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 3, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 4, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 5, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 6, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 7, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 8, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 9, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 10, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 11, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 12, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 13, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 14, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 15, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 16, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 17, 200);
  gtk_clist_set_column_width (GTK_CLIST (clist), 18, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 19, 250);
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  temp=19;
  while(temp--) gtk_clist_set_column_resizeable (GTK_CLIST (clist), temp, FALSE);
  for (temp=17;temp>1;temp--)
    gtk_clist_set_column_visibility (GTK_CLIST (clist), temp, TRUE);
  
  gtk_clist_set_column_visibility (GTK_CLIST (clist), 18, TRUE);
  gtk_clist_set_column_visibility (GTK_CLIST (clist), 19, TRUE);
  

  
  label = gtk_label_new (_("Address"));
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 3, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 4, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 5, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 6, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 7, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 8, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 9, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 10, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 11, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 12, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 13, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 14, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 15, label);
  label = gtk_label_new (_("Hex"));     gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 16, label);
 label = gtk_label_new (_("ASCII"));   gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 17, label);
  label = gtk_label_new (_("Disassembly"));   gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 18, label);  
  callback_memwindow_length (GTK_MENU_ITEM(menuitem), (gpointer)REPRESENTATION_WORD);
  
  view_updatememwindow(memwindow);
  gtk_widget_set_style(clist,fixed_style);
  return hbox;
}

GtkWidget* view_create_flag_frame(int regbanknumber)
{
  GtkWidget *frame; /* the frame which will be returned */
  GtkWidget *hbox; /* horizontal box for the flag buttons */
  GtkWidget *optionmenu_mode;
  GtkWidget *optionmenu_menu;  /* both used to build the menu selection widget */
  GtkWidget *menuitem; /* will hold each menu item in mode selection in turn */	
  
  	  
		  
  reg_window *regwindow = g_new(reg_window,1);
  regwindow->regbank_no = regbanknumber+7; /* retrieves the appropriate data of the reg. bank 
  						currently dealt with */
  
  /* there is an offset of 7 because charlie defined flag windows as register windows and
     their names are stored there */
    
  frame = gtk_frame_new(board_reg_banks[regwindow->regbank_no].name); /* names the frame */
  gtk_widget_ref (frame);
  gtk_widget_show (frame);
  
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox),4);
  gtk_widget_set_usize(hbox,5,5);
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);
  
  gtk_container_add (GTK_CONTAINER (frame), hbox);
  
  /* the following will create the buttons which stand for the flags */
  
  if (regbanknumber==0) // if current -> build flags for current
  {
  nflag_current = gtk_toggle_button_new_with_label ("N"); /* build a switch to represent
  							the N flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(nflag_current),FALSE);
  gtk_widget_ref (nflag_current);                  
  gtk_widget_show (nflag_current);
  gtk_box_pack_start (GTK_BOX (hbox), nflag_current, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (nflag_current), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed), NULL);
  gtk_tooltips_set_tip (view_tooltips, nflag_current, "Toggle N", NULL);
 
  zflag_current = gtk_toggle_button_new_with_label ("Z"); /* build a switch to represent
  							the Z flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(zflag_current),FALSE);
  gtk_widget_ref (zflag_current);                  
  gtk_widget_show (zflag_current);
  gtk_box_pack_start (GTK_BOX (hbox), zflag_current, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (zflag_current), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed), NULL);
  gtk_tooltips_set_tip (view_tooltips, zflag_current, "Toggle Z", NULL);
  
  cflag_current = gtk_toggle_button_new_with_label ("C"); /* build a switch to represent
  							the C flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cflag_current),FALSE);
  gtk_widget_ref (cflag_current);                  
  gtk_widget_show (cflag_current);
  gtk_box_pack_start (GTK_BOX (hbox), cflag_current, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (cflag_current), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed), NULL);
  gtk_tooltips_set_tip (view_tooltips, cflag_current, "Toggle C", NULL);
  
  vflag_current = gtk_toggle_button_new_with_label ("V"); /* build a switch to represent
  							the V flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(vflag_current),FALSE);
  gtk_widget_ref (vflag_current);                  
  gtk_widget_show (vflag_current);
  gtk_box_pack_start (GTK_BOX (hbox), vflag_current, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (vflag_current), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed), NULL);
  gtk_tooltips_set_tip (view_tooltips, vflag_current, "Toggle V", NULL);  
  }
  
  
  if (regbanknumber==1) // if saved - possibly the second window created
  {
  nflag_saved = gtk_toggle_button_new_with_label ("N"); /* build a switch to represent
  							the N flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(nflag_saved),FALSE);
  gtk_widget_ref (nflag_saved);                  
  gtk_widget_show (nflag_saved);
  gtk_box_pack_start (GTK_BOX (hbox), nflag_saved, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (nflag_saved), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed_saved), NULL);
  gtk_tooltips_set_tip (view_tooltips, nflag_saved, "Toggle N", NULL);
 
  zflag_saved = gtk_toggle_button_new_with_label ("Z"); /* build a switch to represent
  							the Z flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(zflag_saved),FALSE);
  gtk_widget_ref (zflag_saved);                  
  gtk_widget_show (zflag_saved);
  gtk_box_pack_start (GTK_BOX (hbox), zflag_saved, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (zflag_saved), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed_saved), NULL);
  gtk_tooltips_set_tip (view_tooltips, zflag_saved, "Toggle Z", NULL);
  
  cflag_saved = gtk_toggle_button_new_with_label ("C"); /* build a switch to represent
  							the C flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(cflag_saved),FALSE);
  gtk_widget_ref (cflag_saved);                  
  gtk_widget_show (cflag_saved);
  gtk_box_pack_start (GTK_BOX (hbox), cflag_saved, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (cflag_saved), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed_saved), NULL);
  gtk_tooltips_set_tip (view_tooltips, cflag_saved, "Toggle C", NULL);
  
  vflag_saved = gtk_toggle_button_new_with_label ("V"); /* build a switch to represent
  							the V flag bit */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(vflag_saved),FALSE);
  gtk_widget_ref (vflag_saved);                  
  gtk_widget_show (vflag_saved);
  gtk_box_pack_start (GTK_BOX (hbox), vflag_saved, FALSE, FALSE, 0);/* add it to the horizontal box */
  gtk_signal_connect (GTK_OBJECT (vflag_saved), "toggled",
                      GTK_SIGNAL_FUNC (callback_flag_pressed_saved), NULL);
  gtk_tooltips_set_tip (view_tooltips, vflag_saved, "Toggle V", NULL);          
  }  

  
  /*  The following creates the mode selection */

  optionmenu_mode = gtk_option_menu_new(); /* new option menu to choose to current mode e.g. FIQ */
  gtk_widget_show (optionmenu_mode);
  optionmenu_menu = gtk_menu_new();
  gtk_widget_ref (optionmenu_menu);

  menuitem = gtk_menu_item_new_with_label (_("User"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)1); /* user mode is numbered 1 */
   if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)1); /* user mode is numbered 1 */
  
  menuitem = gtk_menu_item_new_with_label (_("Supervisor"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)2);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)2);
  
  menuitem = gtk_menu_item_new_with_label (_("Abort"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)3);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)3);
  
  menuitem = gtk_menu_item_new_with_label (_("Undefined"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)4);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)4);
  
  menuitem = gtk_menu_item_new_with_label (_("IRQ"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)5);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)5);
  
   menuitem = gtk_menu_item_new_with_label (_("FIQ"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)6);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)6);
  
  menuitem = gtk_menu_item_new_with_label (_("System"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)7); /* user mode is numbered 1 */
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)7);
  
  menuitem = gtk_menu_item_new_with_label (_("No Mode"));
  gtk_widget_show (menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
  if (regbanknumber==0) gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection),(gpointer)8);
  if (regbanknumber==1) /* we want to divert to a different callback func. */
      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                   GTK_SIGNAL_FUNC (callback_mode_selection_saved),(gpointer)8);
  
  
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_mode),
                                             optionmenu_menu);

  gtk_option_menu_set_history(GTK_OPTION_MENU (optionmenu_mode), 0);
  gtk_tooltips_set_tip (view_tooltips, optionmenu_mode, "Choose mode", NULL); 
  gtk_box_pack_start (GTK_BOX (hbox), optionmenu_mode, TRUE, TRUE, 0);
  
  
    
  
  /* the following is responsible for making the label with the current mode/saved mode */
  
  if (regbanknumber==0) // if current
  {
    view_current_reg_bank = gtk_label_new(NULL);
    gtk_label_set_text (GTK_LABEL(view_current_reg_bank),"User"); /* this should be 
       called every iteration and set the text according to top 4 bits of CSPR 
       and then a function that refreshes this screen should be called */ 
    gtk_widget_ref (view_current_reg_bank);                  
    gtk_widget_show (view_current_reg_bank);
    gtk_box_pack_start (GTK_BOX (hbox), view_current_reg_bank, FALSE, FALSE, 0);
  }
  
  if (regbanknumber==1) // if saved - possibly the second window created
  {
    view_saved_reg_bank = gtk_label_new(NULL);
    gtk_label_set_text (GTK_LABEL(view_saved_reg_bank),"User"); /* this should be 
       called every iteration and set the text according to top 4 bits of CSPR 
       and then a function that refreshes this screen should be called */
    gtk_widget_ref (view_saved_reg_bank);                  
    gtk_widget_show (view_saved_reg_bank);
    gtk_box_pack_start (GTK_BOX (hbox), view_saved_reg_bank, FALSE, FALSE, 0);        
  } 
  
  
  
  
  
  
  if (regbanknumber==1) all_flag_windows_have_been_created=TRUE;
  gtk_widget_set_usize(frame,250,40);
  return frame; /* return the frame with the appropriate flags */
}

  		
GtkWidget* view_create_register_clist (int regbanknumber)  /*creates a register bank window*/
{
  GtkWidget *clist;
  GtkWidget *scrolledwindow;
  GtkWidget *labela;
  GtkWidget *labelb;
  GtkWidget *vbox;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *hbox3;
  GtkWidget *entrya;
  GtkWidget *entryh;
  GtkWidget *frame;
  
  char *startdata[] = {"RegName","Value"};      /*all self explaining stuff*/
  int temp;
 
  reg_window *regwindow = g_new(reg_window,1);
  reg_window_list **regwindowlist = &view_regwindowlist;

  regwindow->regbank_no = regbanknumber;
  while (NULL != *regwindowlist) regwindowlist = &((*regwindowlist)->next);
  *regwindowlist = g_new(reg_window_list,1);
  (*regwindowlist)->reg_data_ptr = regwindow;
  (*regwindowlist)->next = NULL;
  
  frame = gtk_frame_new (board_reg_banks[regwindow->regbank_no].name);
  gtk_widget_ref (frame);
  gtk_widget_show (frame);
  
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox);
  gtk_widget_show (vbox);
  gtk_object_set_data (GTK_OBJECT (vbox), "reg_window", regwindow);
  regwindow->wait=100;
  regwindow->timer=0;
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox), hbox3, FALSE, FALSE, 0);

  
  entrya = gtk_entry_new_with_max_length (5);
  regwindow->address_entry=entrya;
  gtk_widget_ref (entrya);
  gtk_widget_show (entrya);
  gtk_box_pack_start (GTK_BOX (hbox2), entrya, FALSE, TRUE, 0);
  gtk_widget_set_usize (entrya, 37, -2);

  entryh = gtk_entry_new_with_max_length
		  (board_reg_banks[regwindow->regbank_no].width*2+3);
  regwindow->hex_entry=entryh;
  gtk_widget_ref (entryh);
  gtk_widget_show (entryh);
  gtk_box_pack_start (GTK_BOX (hbox2), entryh, FALSE, TRUE, 0);
  gtk_widget_set_usize (entryh, 
            board_reg_banks[regwindow->regbank_no].width * 12+23, -2);
  gtk_signal_connect (GTK_OBJECT(entryh), "activate",
                   GTK_SIGNAL_FUNC (callback_regwindow_hex),NULL);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_end (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
  clist = gtk_clist_new (2);
  regwindow->clist_ptr  = clist;
  for (temp=0;temp < (board_reg_banks[regwindow->regbank_no]).number;temp++)
          gtk_clist_append(GTK_CLIST(clist),startdata);	
  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
  
  gtk_signal_connect (GTK_OBJECT(clist), "select-row",
                   GTK_SIGNAL_FUNC (callback_regwindow_clist),NULL);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 28);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 
		  board_reg_banks[regwindow->regbank_no].width * 12+10); 
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  labela = gtk_label_new (_("Reg"));
  gtk_widget_show (labela);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, labela);
  labelb = gtk_label_new (_("Val"));
  gtk_widget_show (labelb);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, labelb);
  view_updateregwindow(regwindow);
  gtk_widget_set_style(clist,fixed_style);
  if (board_reg_banks[regwindow->regbank_no].pointer) board_reg_banks[regwindow->regbank_no].pointer++;
  gtk_widget_set_usize(frame,150,400);
  return frame;
}

#ifdef GTK2
GtkWidget* view_create_console (void)  /*create a console window*/
{
  GtkWidget *scrolledwindow;
  GtkWidget *view;
  GtkTextBuffer *buffer;

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow);
  gtk_widget_show (scrolledwindow);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  view = gtk_text_view_new ();
  gtk_widget_ref (view);
  gtk_widget_show (view);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view),GTK_WRAP_WORD);
  gtk_text_view_set_editable(GTK_TEXT_VIEW (view),FALSE);
  
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_container_add (GTK_CONTAINER (scrolledwindow), view);
  gtk_text_buffer_set_text (buffer, "Console Window\n", -1);
  view_console = buffer;
  
  return scrolledwindow;
}
#endif

#ifndef GTK2
GtkWidget* view_create_console (void)  /*create a console window*/
{
  GtkWidget *scrolledwindow;
  GtkWidget *text;

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow);
  gtk_widget_show (scrolledwindow);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  text = gtk_text_new (NULL, NULL);
  gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
  gtk_text_set_editable(GTK_TEXT(text), FALSE);
  gtk_widget_show (text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), text);
  gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,
                   "Console Window\n", -1);
  view_console = text;
  return scrolledwindow;
}
#endif

GtkWidget* create_status_bar(void)
{
  GtkWidget *status;
  GtkWidget *hbox;
  GtkWidget *status_bar; 
  GtkWidget *progresslabel; 
  GtkWidget *progressbar;
  
  status_bar=gtk_frame_new("Status Bar");
  gtk_widget_ref (status_bar);
  gtk_widget_show (status_bar);	
  	  
  gtk_frame_set_label_align(GTK_FRAME(status_bar),0.5,0); // set to middle of frame
  gtk_frame_set_shadow_type(GTK_FRAME(status_bar),GTK_SHADOW_ETCHED_OUT);
  
  hbox=gtk_hbox_new(FALSE,0);	
  gtk_widget_ref (hbox);
  gtk_widget_show (hbox);
  gtk_widget_set_usize(hbox,700,40); 
  
  
  status = gtk_label_new ("State:");
  gtk_widget_ref (status);
  gtk_widget_show (status); 
  gtk_box_pack_start (GTK_BOX (hbox), status, FALSE, FALSE, 2);
  
  view_enqlabel = gtk_label_new(NULL);
  gtk_widget_ref (view_enqlabel);   
  gtk_widget_show (view_enqlabel);
  gtk_box_pack_start (GTK_BOX (hbox), view_enqlabel, FALSE, FALSE, 0);
  
  view_steplabel = gtk_label_new(NULL);
  gtk_widget_ref (view_steplabel);   
  gtk_widget_show (view_steplabel);
  gtk_box_pack_start (GTK_BOX (hbox), view_steplabel, FALSE, FALSE, 200);
  gtk_box_pack_start (GTK_BOX (hbox), gtk_vseparator_new()  , FALSE, FALSE, 0);
  
  progresslabel = gtk_label_new ("File progress:");
  gtk_widget_ref (progresslabel);
  gtk_widget_show (progresslabel); 
  gtk_box_pack_start (GTK_BOX (hbox), progresslabel, FALSE, FALSE, 2);
  
  progressbar = gtk_progress_bar_new ();
  gtk_widget_ref (progressbar);
  gtk_widget_show (progressbar);
  gtk_box_pack_start (GTK_BOX (hbox), progressbar, TRUE, TRUE, 2);
  gtk_progress_configure (GTK_PROGRESS (progressbar), 100, 100, 200);

  gtk_progress_set_show_text (GTK_PROGRESS (progressbar), TRUE);
  view_progressbar=progressbar;
  
  gtk_container_add(GTK_CONTAINER(status_bar),hbox);

  //gtk_widget_set_usize(status_bar,900,4);
  return status_bar; 
}  










/* the following subroutine goes to the already retrieved window display list
  and parses it accordingly.  R within the parsing list stands for register,
  			      M for memory list,
			      C for colsole,
			      F for flag window,
			      ! for vertical line,
			      ~ for horizontal line
			      
  appropriate functions that create to appropriate widget are then called
  and the current display list can be found in the entry window_list
  in the file ./.komodo	*/		      
			      

GtkWidget* view_parse_list(void)        /*make the subwindows*/
{
  GtkWidget* ret;
  int tempint;
 switch (*(view_window_display_list++)){
   case '!':
     ret = view_create_h_mov_split();
     gtk_paned_pack1 (GTK_PANED (ret),view_parse_list() , FALSE, FALSE);
     gtk_paned_pack2 (GTK_PANED (ret),view_parse_list() , FALSE, FALSE);
	 return ret;
   case '~':
     ret = view_create_v_mov_split();
     gtk_paned_pack1 (GTK_PANED (ret),view_parse_list() , FALSE, FALSE);
     gtk_paned_pack2 (GTK_PANED (ret),view_parse_list() , FALSE, FALSE);
	 return ret;
   case '|':
     ret = gtk_hbox_new (FALSE, 4);
     gtk_widget_ref (ret);
     gtk_widget_show (ret);
     do gtk_box_pack_start (GTK_BOX (ret), view_parse_list(), TRUE, TRUE, 0);
	 while (','==*(view_window_display_list++));
     view_window_display_list--;
	 return ret;
   case '-':
     ret = gtk_vbox_new (FALSE, 4);
     gtk_widget_ref (ret);
     gtk_widget_show (ret);
     do gtk_box_pack_start (GTK_BOX (ret), view_parse_list(), TRUE, TRUE, 0);
	 while (','==(*(view_window_display_list++)));
     view_window_display_list--;
	 return ret;
   case 'N':
     ret = gtk_notebook_new ();
     if (call_number_to_notebook==0) register_notebook=ret; /* save reference to this item if it is the first call to 'N' */
     if (call_number_to_notebook==1) flags_notebook=ret; /* save reference to this item if it is the second call to 'N' */
     gtk_notebook_set_tab_pos (GTK_NOTEBOOK (ret), GTK_POS_LEFT);
     gtk_widget_ref (ret);
     gtk_widget_show (ret);
     tempint=0;
     do  
           { 
            char* temp =view_window_display_list;
            GtkWidget *label;
            while ('.' != *view_window_display_list) view_window_display_list++;
            *(view_window_display_list++) = '\0';
            gtk_container_add (GTK_CONTAINER (ret), label=view_parse_list());
            gtk_container_set_border_width (GTK_CONTAINER (label),4);
            label = gtk_label_new (temp);
            gtk_widget_ref (label);
	    gtk_widget_show (label);
            gtk_notebook_set_tab_label (GTK_NOTEBOOK (ret), 
                    gtk_notebook_get_nth_page (GTK_NOTEBOOK (ret), tempint++), label);
            }
	 while ('.'==(*(view_window_display_list++)));
         view_window_display_list--;
         call_number_to_notebook++;
	 return ret;
   
   case 'R':
	 return view_create_register_clist (*(view_window_display_list++)-0x30);
   case 'M':
	 return view_create_memory_clist ();
   case 'F':
         return view_create_flag_frame (0); // 0 for current  
   case 'S':
         return view_create_flag_frame (1); // 1 for saved	 
   case 'C':
	 return view_create_console ();
   }
   if (VERBOSE) /* an unknown charcter has been passes and could not be processed */
     g_print("Write a proper window line!");
   return NULL;
}
