#ifndef VIEW_H
#define VIEW_H
#include <glib.h>
#include <gtk/gtk.h>
#include "interface.h"
#define _(String) (String)
#define uchar unsigned char
#define CONSOLE_UPPER_BOUND 20

#define ADDRESS_ENTRY			0
#define ASCII_ENTRY			17
#define DIS_ENTRY			18

#define REPRESENTATION_BYTE		0
#define REPRESENTATION_2_BYTES		1
#define REPRESENTATION_4_BYTES		2
#define REPRESENTATION_8_BYTES		3
#define REPRESENTATION_16_BYTES		4
#define REPRESENTATION_HALFWORD 	5
#define REPRESENTATION_2_HALFWORDS	6
#define REPRESENTATION_4_HALFWORDS	7
#define REPRESENTATION_8_HALFWORDS	8
#define REPRESENTATION_WORD 		9
#define REPRESENTATION_2_WORDS 		10
#define REPRESENTATION_4_WORDS 		11
#define REPRESENTATION_8_WORDS 		12
#define REPRESENTATION_16_WORDS		13
#define REPRESENTATION_ASCII_MAP	14

#define LENGTH_OF_MEMORY_WINDOW		36




typedef struct mem_window_name{
 GtkWidget* clist_ptr;
 GtkWidget* address_entry;
 GtkWidget* hex_entry; 		/* the hex blank entry		 	 */	
 GtkWidget* ascii_entry;	/* the ascii blank entry		 */	
 GtkWidget* dis_entry;		/* the disassembly blank entry 		 */
 GtkWidget* ascii_button;	/* the ascii toggle button of the window */
 GtkWidget* dis_button;		/* the disassembly toggle button of the window */
 GtkWidget* elf_symbols_button;	/* the ELF symbols toggle button of the window */
 GtkWidget* scroll;
 unsigned char* address;
 unsigned char* regoffset;
 int count;
 int width; /* number of hex value columns */
 int gran; /* the number of bytes shown in each of the above columns */
// int wait;
// gint timer;
 GList* isa;
} mem_window;

typedef struct mem_window_list_name{
 mem_window* mem_data_ptr;
 struct mem_window_list_name* next;
} mem_window_list;

typedef struct reg_window_name{
 GtkWidget* clist_ptr;
 GtkWidget* address_entry;
 GtkWidget* hex_entry;
 int regbank_no;
 int wait;
 gint timer;
} reg_window;

typedef struct reg_window_list_name{
 reg_window* reg_data_ptr;
 struct reg_window_list_name* next;
} reg_window_list;

typedef struct dat_window_name{
 GtkWidget* clist_ptr;
 GtkWidget* address_entry;
 GtkWidget* hex_entry;
 GtkWidget* ascii_entry;
 GtkWidget* dis_entry;
 GtkWidget* scroll; 
 unsigned char* address;
 unsigned char* regoffset;
 int count;
 int width;
 int gran;
} dat_window;

typedef struct dat_window_list_name{
 mem_window* dat_data_ptr;
 struct dat_window_list_name* next;
} dat_window_list;

// char* view_dis_arm(uchar*, uchar*);
// char* view_dis_thumb(uchar*, uchar*);
char* view_dis(uchar*, uchar*,int, GList*);
		
/* some prototypes */
GtkWidget* create_memorywindow (void);
GtkWidget* view_create_flag_frame(int regbanknumber);
int view_updatememwindow(mem_window*);
int view_updateregwindow(reg_window*);
mem_window* view_getmemwindowptr(GtkWidget* widget);
reg_window* view_getregwindowptr(GtkWidget* widget);
void view_create_mainwindow (void);
GtkWidget* view_create_filebar (void);
GtkWidget* create_featurewindow (void);
GtkWidget* create_xfpgasegment (void);
void create_splashwindow (void);
GtkWidget* create_aboutwindow (void);
GtkWidget* view_create_prog_ctrl (void);
GtkWidget* view_create_h_mov_split (void);
GtkWidget* view_create_v_mov_split (void);
GtkWidget* view_create_memory_clist (void);
GtkWidget* view_create_register_clist (int);
GtkWidget* view_create_console (void);
GtkWidget* view_create_comms (feature*);
GtkWidget* view_parse_list(void);
GtkWidget* view_create_menubar(void);
GtkWidget* create_status_bar(void); /* creates the status bar at bottom */

gpointer view_console; /* a pointer to the text in the string console */
gpointer view_comms;

char* view_window_display_list; /* a variable to hold the display list string
		that was derived from .komodo */

mem_window_list* view_memwindowlist; /* a group of memory windows/containers that we have */
reg_window_list* view_regwindowlist; /* as above, for register banks */
dat_window_list* view_datwindowlist; /* a list of memory windows/containers data e.g. number of lines */

GtkWidget* view_splashwindow; /* global referring to the splash window created */
GtkWidget* view_mainwindow; /* refers to our main GTK window */
GtkWidget* view_maincontainer; /* refers to the actual container within the window */
GtkWidget* view_progressbar; /* the progress bar widget which is used to make other local progress bar global by assignment */
GtkWidget* view_enqlabel; /* the labels that reports the current state of the board e.g. stopped */
GtkWidget* view_steplabel; /* the labels that reports the current step number */
GtkWidget* view_current_reg_bank; /* will be the label reporting the current mode in the CPSR */
GtkWidget* view_saved_reg_bank; /* will be the label reporting the current mode in the SPSR */
GtkWidget* view_breakwindow[2][4];
GtkWidget* view_fileerror; /* a widger for error messages, will be assigned to a new gtk dialogue at times */
GtkWidget* view_refreshbutton; /* the widget of the toggle button refresh */
GtkWidget* view_binary_load_address; /* the address of a binary being loaded */

int view_step_number; /* defines the initial number of steps to be taken when step is pressed */
int view_step_freq; /*defines the initial value in the second box next to muti-step */


/* definitions of colours and pixmaps */
GdkColor view_greycolour; /* defines the grey colour used in the application */

/* the pixmaps used: */
GdkPixmap* view_chump_pixmap; 
GdkBitmap* view_chump_bitmap;
GdkPixmap* view_tick_pixmap;
GdkBitmap* view_tick_bitmap;
GdkPixmap* view_mulogo_pixmap;
GdkBitmap* view_mulogo_bitmap;
GdkPixmap* view_komodologo_pixmap;
GdkBitmap* view_komodologo_bitmap;
GdkPixmap* view_komodotitle_pixmap;
GdkBitmap* view_komodotitle_bitmap;
GdkPixmap* view_komodoicon_pixmap;
GdkBitmap* view_komodoicon_bitmap;
GdkPixmap* view_komodominiicon_pixmap;
GdkBitmap* view_komodominiicon_bitmap;

/* the following are toggle-button widgets to be used for flags */

/* for flags in CPSR */
GtkWidget *nflag_current;
GtkWidget *zflag_current;
GtkWidget *cflag_current;
GtkWidget *vflag_current;

/* for flags in SPSR */

GtkWidget *nflag_saved;
GtkWidget *zflag_saved;
GtkWidget *cflag_saved;
GtkWidget *vflag_saved;

GtkTooltips* view_tooltips; /* one to hold the tooltips of all items in turn */

// GtkWidget *console_entry; /* the entry within the console/terminal is now global */

		
gint view_global_refresh_timer; /* boolean to indicate if the display needs to be refreshed */
GtkWidget *text_in_console[CONSOLE_UPPER_BOUND];
/* caters for up to CONSOLE_UPPER_BOUND features where the highest serial number for a console is 20 */


GtkWidget *window_about; /* will be a pointer to the about window */
GtkWidget *window_feature; /* will be a pointer to the feature window */
  
GtkWidget *register_notebook;/* will be a pointer to the register banks notebook */
GtkWidget *flags_notebook;/* will be a pointer to the flags notebook */

GtkWidget *centry_compile;   /* the current file entry for compilation */
GtkWidget *centry_load; /* the current file entry for loading */ 



#endif
